package de.duehl.pentris.logic;

import java.awt.EventQueue;

import de.duehl.pentris.logic.field.GameField;
import de.duehl.pentris.logic.gameLoop.GameLoop;
import de.duehl.pentris.ui.Gui;
import de.duehl.pentris.ui.PentrisGui;
import de.duehl.pentris.ui.gameloop.GameLoopGui;
import de.duehl.basics.logic.Version;

/**
 * Diese Klasse stellt die Logik des Pentris-Programms dar.
 *
 * @version 1.01     2013-09-03
 * @author Christian Dühl
 */

public class PentrisLogic implements Logic, GameOver {

    /** Programmversion. */
    private Version version;

    /** Spielfeld. */
    private volatile GameField field;
    /*
     * Da es sowohl in PentrisPanel als auch in der GameLoop benötigt wird,
     * erstellen wir es hier. Da es in zwei Threads verwendet wird, ist es
     * volatile, auch wenn nur die Game-Loop Veränderungen daran vornimmt.
     */

    /** Verbindung zur grafischen Oberfläche. */
    private Gui gui;

    /** Die Game-Loop. */
    private GameLoop gameLoop;

    /** Gibt an, ob das Spiel gerade pausiert wird. */
    private boolean pause;

    /** Oberfläche der GameLoop. */
    private GameLoopGui gameLoopGui;

    /**
     * Konstruktor.
     *
     * @param version
     *            Programmversion.
     */
    public PentrisLogic(Version version) {
        this.version = version;

        field = new GameField();
        pause = false;

        gui = new PentrisGui(this);
        gui.show();
    }

    /** Getter für die Programmversion. */
    @Override
    public Version getVersion() {
        return version;
    }

    /** Beendet das Programm. */
    @Override
    public void quit() {
        pause(); // Pause beginnen
        boolean quit = gui.askUserToQuit();
        if (quit) {
            internalQuit();
        }
        else {
            pause(); // Pause beenden
        }
    }

    /** Beendet das Programm. */
    private void internalQuit() {
        if (gameLoop != null) {
            gameLoop.quit();
        }

        gui.quit();
    }

    /** Getter für das Spielfeld. */
    @Override
    public GameField getField() {
        return field;
    }

    /**
     *  Setter für die Gui des Game-Loops, die Gui wird in PentrisGui erstellt.  <br /><br />
     *
     *  Wird aus dem EDT aufgerufen.
     */
    @Override
    public void setGameLoopGui(GameLoopGui gameLoopGui) {
        this.gameLoopGui = gameLoopGui;
        askUserNewGameOrExit();
    }

    /**
     * Wird aufgerufen, wenn das Spiel beendet ist.                              <br /><br />
     *
     * Aufruf erfolgt aus der GameLoop.
     */
    @Override
    public void gameOver() {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                askUserNewGameOrExit();
            }
        });
        /*
         * Eigentlich ist es nicht schön, in der Logik mit
         * EventQueue.invokeLater zu arbeiten, aber wie sich das hier vermeiden
         * ließe, ist mir gerade nicht klar.
         *
         * Denn askUserNewGameOrExit wird auch weiter oben aus dem EDT
         * aufgerufen.
         *
         * Wenn ich das ganze in die Gui verlagern wollte, müsste ich da
         * abfragen, ob der aktuelle Thread der EDT ist und wenn nicht, den
         * Namen des Benutzers mit invokeAndWait abfragen, dafür brauche ich
         * dann wegen des Rückgabewertes eine eigentlich nutzlose
         * Klassenvariable, deshalb lasse ich das erstmal hier.
         */
    }

    /**
     * Fragt den Benutzer, ob er ein neues Spiel spielen oder das Programm
     * beenden möchte.
     */
    private void askUserNewGameOrExit() {
        GameType gameType = gui.askUserToStartANewGame();
        if (null == gameType) {
            internalQuit();
        }
        else {
            gameLoop = new GameLoop(gameType, field, gameLoopGui,
                    (GameOver) this);
            Thread thread = new Thread(gameLoop);
            thread.start();
            gui.getToFront();
        }
    }

    /** Schaltet die Pause an oder aus. */
    @Override
    public void pause() {
        pause = !pause;
        if (pause) {
            gameLoop.pause();
        }
        else {
            gameLoop.pause();
        }
    }

    /** Lässt den aktuellen Stein so weit Fallen, wie er fallen kann. */
    @Override
    public void freeFall() {
        gameLoop.freeFall();
    }

    /** Bewegt den aktuellen Stein nach links, falls möglich. */
    @Override
    public void moveStoneLeft() {
        gameLoop.moveStoneLeft();
    }

    /** Bewegt den aktuellen Stein nach rechts, falls möglich. */
    @Override
    public void moveStoneRight() {
        gameLoop.moveStoneRight();
    }

    /** Dreht den aktuellen Stein. */
    @Override
    public void turnStone() {
        gameLoop.turnStone();
    }

    /** Bewegt den aktuellen Stein eine Reihe nach unten, falls möglich. */
    @Override
    public void dropStoneOneRow() {
        gameLoop.dropStoneOneRow();
    }

    /** Schaltet das Grid an oder aus. */
    @Override
    public void switchGrid() {
        gameLoop.switchGrid();
    }

}
