package de.duehl.pentris.logic.field;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import de.duehl.pentris.logic.Constants;
import de.duehl.pentris.logic.Coordinate;
import de.duehl.pentris.logic.stones.Stone;
import de.duehl.pentris.logic.stones.StoneFactory;

/**
 * Diese Klasse stellt das Spielfeld dar.
 *
 * @version 1.01     2013-09-03
 * @author Christian Dühl
 */

public class GameField {

    /** Spielfeld. */
    private List<List<OneField>> field;

    /** Bunt gewürfeltes Feld, das in der Pause angezeigt wird. */
    private List<List<OneField>> pausedField;

    /** Liste mit den Farben der erzeugbaren Steine. */
    private List<String> stoneColors;

    /** Konstruktor. */
    public GameField() {
        clear();

        /* Alle Steinfarben ermitteln: */
        StoneFactory stoneFactory = new StoneFactory();
        stoneColors = stoneFactory.getStoneColors();
    }

    /** Erzeugt eine leere Zeile. */
    private List<OneField> createEmptyRow() {
        List<OneField> row = new ArrayList<>();
        for (int col=0; col<Constants.COLS; ++col) {
            row.add(new OneField(true, ""));
        }
        return row;
    }

    /** Schaltet die Pause ein. */
    synchronized
    public void pauseOn() {
        pausedField = field;
        field = new ArrayList<>();
        Random random = new Random();
        for (int r=0; r<Constants.ROWS; ++r) {
            List<OneField> row = new ArrayList<>();
            for (int c=0; c<Constants.COLS; ++c) {
                int index = random.nextInt(stoneColors.size());
                String color = stoneColors.get(index);
                row.add(new OneField(false, color));
            }
            field.add(row);
        }
    }

    /** Schaltet die Pause aus. */
    synchronized
    public void pauseOff() {
        field = pausedField;
    }

    /**
     * Gibt zurück, ob an der angegebenen Stelle das Feld frei ist.
     *
     * @param row
     *            Zeile
     * @param col
     *            Spalte
     * @return true = Das Feld in Zeile row und Spalte col ist frei.
     */
    synchronized
    public boolean isFree(int row, int col) {
        OneField oneField = getTheOneFieldAt(row, col);
        return oneField.isEmpty();
    }

    /**
     * Ermittelt die Farbe an einer bestimmten Stelle des Feldes. Macht nur Sinn
     * für Stellen des Feldes, die nicht frei sind.
     *
     * @param row
     *            Zeile
     * @param col
     *            Spalte
     * @return Farbe des Feldes in Zeile row und Spalte col.
     */
    synchronized
    public String getColor(int row, int col) {
        OneField oneField = getTheOneFieldAt(row, col);
        return oneField.getFilledColor();
    }

    /**
     * Gibt das Feld an der angegebenen Stelle zurück.
     *
     * @param row
     *            Zeile
     * @param col
     *            Spalte
     * @return Das Feld in Zeile row und Spalte col.
     * @throws IllegalArgumentException
     *             Falls Zeile oder Spalte unpassend sind.
     */
    private OneField getTheOneFieldAt(int row, int col) {
        if (row >= field.size()) {
            throw new IllegalArgumentException("Zeile " + row + " zu groß.");
        }
        if (row < 0) {
            throw new IllegalArgumentException("Zeile " + row + " zu klein.");
        }
        if (col >= field.get(0).size()) {
            throw new IllegalArgumentException("Spalte " + col + " zu groß.");
        }
        if (col < 0) {
            throw new IllegalArgumentException("Spalte " + col + " zu klein.");
        }
        List<OneField> rowList = field.get(row);
        OneField oneField = rowList.get(col);
        return oneField;
    }

    /**
     * Fügt einen Stein an seinen Koordinaten in das Feld ein.
     *
     * @param stone
     *            Einzufügender Stein.
     */
    synchronized
    public void insertStone(Stone stone) {
        String color = stone.getColor();
        for (Coordinate coord : stone.getCoordinates()) {
            int row = coord.getRow();
            int col = coord.getCol();
            OneField oneField = getTheOneFieldAt(row, col);
            oneField.setEmpty(false);
            oneField.setFilledColor(color);
        }
    }

    /**
     * Entfernt volle Zeilen und gibt die Anzahl der entfernten vollen Zeilen
     * zurück.
     *
     * @return Anzahl der gelöschten Zeilen.
     */
    synchronized
    public List<Integer> deleteFullLines() {
        List<Integer> clearedRows = new ArrayList<>();
        for (int row = field.size()-1; row >=0; --row) {
            List<OneField> theRow = field.get(row);
            boolean fullRow = true;
            for (int col = 0; col < theRow.size(); ++col) {
                OneField field = theRow.get(col);
                if (field.isEmpty()) {
                    fullRow = false;
                    break;
                }
            }
            if (fullRow) {
                clearedRows.add(row);
                field.remove(theRow);
                field.add(createEmptyRow());
            }
        }
        return clearedRows;

    }

    /**
     * Ermittelt, ob das Feld ganz frei ist.
     *
     * @return true: Feld ist völlig leer.
     */
    synchronized
    public boolean isEmpty() {
        for (List<OneField> theRow : field) {
            for (OneField theField : theRow) {
                if (!theField.isEmpty()) {
                    return false;
                }
            }
        }
        return true;
    }

    /** Leert das Spielfeld. */
    synchronized
    public void clear() {
        field = new ArrayList<>();
        for (int row=0; row<Constants.ROWS; ++row) {
            List<OneField> theRow = createEmptyRow();
            field.add(theRow);
        }
    }

}
