package de.duehl.pentris.logic.gameLoop;

import java.util.List;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

import de.duehl.pentris.logic.Constants;
import de.duehl.pentris.logic.GameOver;
import de.duehl.pentris.logic.GameType;
import de.duehl.pentris.logic.field.GameField;
import de.duehl.pentris.logic.highscore.HighScore;
import de.duehl.pentris.logic.stones.Stone;
import de.duehl.pentris.logic.stones.StoneFactory;
import de.duehl.pentris.ui.gameloop.GameLoopGui;

/**
 * Diese Klasse enthält die Game-Loop, das Kernstück der Spielabfolge, das
 * in einem eigenen Thread läuft und von dort alles verwaltet.                   <br /><br />
 *
 * Zu unterscheiden sind Methoden, die von außen (public) aus dem EDT der
 * Logik und von Swing aufgerufen werden, und Methoden (private), die aus
 * der Game-Loop gestartet werden und daher in dieser laufen.
 *
 * @version 1.01     2013-09-06
 * @author Christian Dühl
 */

public class GameLoop implements Runnable {

    /**
     * Eingehende Tastaturbefehle aus dem EDT werden in einer Queue gesammelt
     * und in der Reihenfolge abgearbeitet, wenn die Game-Loop ausgeführt wird
     *
     * @author Christian Dühl
     */
    public enum KeyCommand {
        FREE_FALL, LEFT, RIGHT, TURN, DROP_ONE_ROW, PAUSE, GRID
    }

    /**
     * Spielgeschwindigkeit (Wartezeit zwischen dem Heruntersinken der Steine in
     * Millisekunden) zu Beginn des Spiels
     */
    private static final long START_WAIT_TIME_IN_MILLIS = 1000;

    /** Punkte für das Platzieren eines Steins im Spielfeld. */
    private static final int POINTS_FOR_INSERTED_STONE = 3;

    /** Multiplikator für eine geleerte Zeile. */
    private static final int POINTS_MULTIPLIER_FOR_CLEARED_ROWS = 13;

    /** Multiplikator für viele geleerte Zeilen. */
    private static final int POINTS_MULTIPLIER_FOR_MANY_ROWS = 512;

    /**
     * Erhöhung der Spielgeschwindigkeit (Wartezeit zwischen dem Heruntersinken
     * der Steine in Millisekunden)pro geleerte Zeile.
     */
    private static final int SPEED_MULTIPLIER_FOR_CLEARED_LINES = 10;

    /**
     * Queue mit den eingegangenen Tastaturbefehlen aus dem EDT. Da diese aus
     * dem EDT befüllt wird, wird eine ConcurrentLinkedQueue verwendet, die
     * thread-save ist.
     */
    private Queue<KeyCommand> commands;

    /** Gui der GameLoop (läuft natürlich nicht hier, sondern im EDT). */
    private GameLoopGui gameLoopGui;

    /** Fabrik, die Spielsteine herstellen kann. */
    private StoneFactory factory;

    /**
     * Ist wahr, solange die Game-Loop läuft, also solange das Spiel nicht
     * beendet ist.
     */
    private volatile boolean running; /* wird von außen in quit geändert! */

    /** Aktueller, fallender Spielstein. */
    private Stone actualStone;

    /** Nächster Stein. */
    private Stone nextStone;

    /** Spielfeld. */
    private volatile GameField field;

    /** Gibt an, ob das Spiel gerade pausiert wird. */
    private boolean paused;

    /** Letzter Zeitpunkt, als die Game-Loop durchlaufen wurde. */
    private long lastTime;

    /** Zeitpunkt, zu dem die Pause begann. */
    private long pauseStart;

    /** Gibt an, ob das Gitter eingezeichnet werden soll oder nicht. */
    private boolean showGrid;

    /** Erzielte Punkte des Spielers. */
    private int points;

    /** Art des aktuellen Spiels. */
    private GameType gameType;

    /** Wartezeit, bis der Stein wieder sinkt. */
    private long toWait;

    /**
     * Objekt, das über das Spielende benachrichtigt wird (die Logik, durch ein
     * Interface unabhängig gemacht).
     */
    private GameOver gameOver;

    /**
     * Konstruktor.
     *
     * @param gameLoopGui
     *            Gui der GameLoop (läuft natürlich nicht hier, sondern im EDT).
     * @param field
     *            Spielfeld
     * @param gameOver
     *            Objekt, das über das Spielende benachrichtigt wird (die Logik,
     *            durch ein Interface unabhängig gemacht).
     */
    public GameLoop(GameType gameType, GameField field,
            GameLoopGui gameLoopGui, GameOver gameOver) {
        this.gameType = gameType;
        this.field = field;
        this.gameLoopGui = gameLoopGui;
        this.gameOver = gameOver;

        factory = new StoneFactory();
        showGrid = true;

        commands = new ConcurrentLinkedQueue<>();

        initializeHighScore();

        /*
         * Alles, was hier angelegt wird, ist dem Thread (in run()) beim Start
         * bekannt, nur spätere Änderungen in anderen Threads wären ein Problem,
         * das mit volatile oder synchronized behandelt werden sollte (oder aber
         * durch thread-save Container wie etwa ConcurrentLinkedQueue).
         */
    }

    /**
     * Hier befindet sich die eigentliche Game-Loop. Sie wird im eigenen
     * Thread - dem Game-Loop-Thread - gestartet.
     */
    @Override
    public void run() {
        initGame();
        while (running) {
            while (!commands.isEmpty()) {
                KeyCommand command = commands.remove();
                handleKeyCommand(command);
            }
            if (!paused) {
                long actualTime = System.currentTimeMillis();
                long diff = actualTime - lastTime;
                if (diff >= toWait) {
                    lastTime = actualTime;
                    gameLoopStep();
                }
            }
            sleep();
        }
    }

    /** Hier wird alles für ein neues Spiel initialisiert. */
    private void initGame() {
        factory.setGameType(gameType);
        nextStone = factory.createRandomStone();
        createNewStone();

        lastTime = System.currentTimeMillis();
        running = true;
        toWait = START_WAIT_TIME_IN_MILLIS;
        points = 0;
        field.clear();
    }

    /**
     * Wird ausgeführt, wenn es an der Zeit ist, den Stein eine Reihe nach unten
     * sinken zu lassen.
     */
    private void gameLoopStep() {
        boolean success = actualStone.drop(field); // nur lesend
        if (!success) {
            /* Stein einfügen und Punkte dafür berechnen: */
            insertActualStoneIntoField();
            points += POINTS_FOR_INSERTED_STONE;
            points += actualStone.getBonus();

            /* Volle Zeilen leeren und Punkte dafür gutschreiben: */
            List<Integer> clearedRows = deleteFullLines();
            int cleared = clearedRows.size();
            for (int row : clearedRows) {
                points += POINTS_MULTIPLIER_FOR_CLEARED_ROWS
                        * (Constants.ROWS - row) * (cleared + 1);
            }
            if (cleared > 3) {
                int rowsOverThree = cleared - 3;
                // 1 oder 2, da die Steine maximal Länge 5 haben.
                points += POINTS_MULTIPLIER_FOR_MANY_ROWS
                        * rowsOverThree;
            }
            /* Es wäre zu überlegen, auch 3 Reihen schon zu belohnen wären. */

            /* Für ein ganz leeres Feld Punkte verdoppeln: */
            if (field.isEmpty()) {
                points *= 2;
            }

            actualizePoints();


            /* Neuen Stein erzeugen und testen, ob er noch in das Feld passt: */
            createNewStone();
            boolean fieldIsFreeForStone = actualStone.fitsOnField(field);
            if (!fieldIsFreeForStone) {
                running = false;
                gameOver();
            }

            /* Geschwindigkeit erhöhen: */
            toWait -= cleared
                    * SPEED_MULTIPLIER_FOR_CLEARED_LINES;
        }
        repaintGameLoop();
    }

    /** Aktualisiert die Anzeige des Punktestandes. */
    private void actualizePoints() {
        gameLoopGui.actualizePoints(points);
        /*
         * Innerhalb von gameLoopGui.actualizePoints wird der EDT per
         * invokeLater beauftragt, die Anzeige des Punktestandes zu
         * aktualisieren.
         */
    }

    /**
     * Wird aufgerufen, wenn der letzte Stein keinen Platz mehr findet und das
     * Spiel deshalb beendet ist.
     */
    private void gameOver() {
        /* Eintrag in der Highscoreliste vornehmen: */
        HighScore highScore = determineHighscoreFile();
        highScore.load();
        int place = highScore.determinePlace(points);

        /* Gratulieren und Namen erfragen: */
        String userName = gameLoopGui.gratulation(place, points, gameType);

        /* Eintrag in der Highscoreliste vornehmen: */
        highScore.addUserScore(points, userName);
        highScore.save();

        /* Logik (im EDT) über Ende des aktuellen Spiels benachrichtigen: */
        gameLoopGui.actualizeHighscore(highScore.getElements());
        /*
         * Innerhalb von gameLoopGui.actualizeHighscore wird der EDT per
         * invokeLater beauftragt, die Anzeige der Highscoreliste zu
         * aktualisieren.
         */
        gameOver.gameOver();
        /*
         * Innerhalb von gameOver.gameOver wird der EDT per invokeLater
         * beauftragt, nach dem Start eines neuen Spiels zu fragen.
         */
    }

    /**
     * Bildet ein passendes HighScore-Object nach gewählter Spielart. Es wird
     * die Datei "highscore.pentris" bzw. "highscore.tetris" verwendet.
     *
     * @return Objekt mit der richtigen HighScore-Datei.
     */
    private HighScore determineHighscoreFile() {
        HighScore highScore;
        if (gameType == GameType.PENTRIS) {
            highScore = new HighScore("highscore.pentris");
        }
        else {
            highScore = new HighScore("highscore.tetris");
        }
        return highScore;
    }

    /**
     * Lädt zu Beginn den Highscore passend zur ausgewählten Spielart, um
     * ihn in der Gui anzuzeigen.
     */
    private void initializeHighScore() {
        HighScore highScore = determineHighscoreFile();
        highScore.load();
        gameLoopGui.actualizeHighscore(highScore.getElements());
        /*
         * Innerhalb von gameLoopGui.actualizeHighscore wird der EDT per
         * invokeLater beauftragt, die Anzeige der Highscoreliste zu
         * aktualisieren.
         */
    }

    /**
     * Fügt den aktuellen Stein ins Spielfeld ein.                               <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void insertActualStoneIntoField() {
//        try {
//            EventQueue.invokeAndWait(new Runnable() {
//                @Override
//                public void run() {
//                    field.insertStone(actualStone);
//                }
//            });
//        }
//        catch (InvocationTargetException | InterruptedException e) {
//            e.printStackTrace();
//        }
        field.insertStone(actualStone);
        // Da field im EDT volatile ist, kann der Aufruf von invokeAndWait
        // entfallen.
    }

    /**
     * Löscht die geleerten Zeilen im Spielfeld.                                 <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private List<Integer> deleteFullLines() {
//        final int number = 0; // das geht so nicht, ohne final auch nicht,
//                              // müsste Klassenvariable werden.
//        try {
//            EventQueue.invokeAndWait(new Runnable() {
//                @Override
//                public void run() {
//                    number = field.deleteFulLines();
//                }
//            });
//        }
//        catch (InvocationTargetException | InterruptedException e) {
//            e.printStackTrace();
//        }
//
//        return number;
        return field.deleteFullLines();
        // Da field im EDT volatile ist, kann der Aufruf von invokeAndWait
        // entfallen.
    }

    /**
     * Zeichnet das Spiel neu.                                                   <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void repaintGameLoop() {
        gameLoopGui.repaint();
        /*
         * Innerhalb von gameLoopGui.repaint wird der EDT per
         * invokeLater beauftragt, die Anzeige des Spielfeldes zu
         * aktualisieren.
         */
    }

    /**
     * Erstellt einen neuen Stein. Da der nächste Stein oben links immer schon
     * angezeigt wird, muss eigentlich der nächste nächste Stein erzeugt werden
     * und der ehemals nächste der aktuelle Stein werden.                        <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void createNewStone() {
        actualStone = nextStone;
        nextStone = factory.createRandomStone();

        gameLoopGui.showActualStone(actualStone);
        /*
         * Innerhalb von gameLoopGui.showActualStone wird der EDT per
         * invokeLater beauftragt, die Anzeige des aktuellen Spielsteins zu
         * aktualisieren.
         */
        gameLoopGui.setNextStone(nextStone);
        /*
         * Innerhalb von gameLoopGui.setNextStone wird der EDT per invokeLater
         * beauftragt, die Anzeige des nächsten Spielsteins zu aktualisieren.
         */
    }

    /** Schläft 10 Millisekunden. */
    private void sleep() {
        try {
            Thread.sleep(10);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
            // Sollte nicht auftreten, da der Thread nicht aufgehoben wird, so
            // kann er auch nicht unterbrochen werden.
        }
    }

    /**
     * Beendet die Game-Loop von außen, wenn der Benutzer das Fenster
     * schließt.                                                                 <br /><br />
     *
     * Achtung, Aufruf aus EDT!
     */
    public void quit() {
        running = false;
        /*
         * Da running volatile ist, macht es nichts, diese Variable im EDT zu
         * verändern.
         */
    }

    /**
     * Lässt den aktuellen Stein so weit fallen, wie er fallen kann.             <br /><br />
     *
     * Achtung, Aufruf aus EDT!
     */
    public void freeFall() {
        commands.add(KeyCommand.FREE_FALL);
    }

    /**
     * Bewegt den aktuellen Stein nach links, falls möglich.                     <br /><br />
     *
     * Achtung, Aufruf aus EDT!
     */
    public void moveStoneLeft() {
        commands.add(KeyCommand.LEFT);
    }

    /**
     * Bewegt den aktuellen Stein nach rechts, falls möglich.                    <br /><br />
     *
     * Achtung, Aufruf aus EDT!
     */
    public void moveStoneRight() {
        commands.add(KeyCommand.RIGHT);
    }

    /**
     * Dreht den aktuellen Stein.                                                <br /><br />
     *
     * Achtung, Aufruf aus EDT!
     */
    public void turnStone() {
        commands.add(KeyCommand.TURN);
    }

    /**
     * Bewegt den aktuellen Stein eine Reihe nach unten, falls möglich.          <br /><br />
     *
     * Achtung, Aufruf aus EDT!
     */
    public void dropStoneOneRow() {
        commands.add(KeyCommand.DROP_ONE_ROW);
    }

    /**
     * Beginn oder Ende einer Pause.                                             <br /><br />
     *
     * Achtung, Aufruf aus EDT!
     */
    public void pause() {
        commands.add(KeyCommand.PAUSE);
    }

    /**
     * Schaltet das Grid an oder aus.                                            <br /><br />
     *
     * Achtung, Aufruf aus EDT!
     */
    public void switchGrid() {
        commands.add(KeyCommand.GRID);
    }

    /**
     * Verarbeitet einen Tastendruck in der Game-Loop.                           <br /><br />
     *
     * Aufruf aus der Game-Loop!
     *
     * @param command
     *            Tastendruck.
     */
    private void handleKeyCommand(KeyCommand command) {
        switch (command) {
            case FREE_FALL:
                if (!paused) {
                    internalFreeFall();
                }
                break;
            case LEFT:
                if (!paused) {
                    internalMoveStoneLeft();
                }
                break;
            case RIGHT:
                if (!paused) {
                    internalMoveStoneRight();
                }
                break;
            case TURN:
                if (!paused) {
                    internalTurnStone();
                }
                break;
            case DROP_ONE_ROW:
                if (!paused) {
                    internalDropOneRow();
                }
                break;
            case PAUSE:
                if (paused) {
                    internalPauseOff();
                }
                else {
                    internalPauseOn();
                }
                break;
            case GRID:
                if (showGrid) {
                    showGrid = false;
                }
                else {
                    showGrid = true;
                }
                gameLoopGui.setShowGrid(showGrid);
                /*
                 * Innerhalb von gameLoopGui.setShowGrid wird der EDT per
                 * invokeLater beauftragt, die Anzeige des Gitters zu ändern.
                 */
                break;
        }
    }

    /**
     * Lässt den aktuellen Stein so weit fallen, wie er fallen kann.             <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void internalFreeFall() {
        while (actualStone.drop(field)) {
            ; // nichts zu tun
        }
        repaintGameLoop();
    }

    /**
     * Bewegt den aktuellen Stein nach links, falls möglich.                     <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void internalMoveStoneLeft() {
        actualStone.moveLeft(field);
        repaintGameLoop();
    }

    /**
     * Bewegt den aktuellen Stein nach rechts, falls möglich.                    <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void internalMoveStoneRight() {
        actualStone.moveRight(field);
        repaintGameLoop();
    }

    /**
     * Dreht den aktuellen Stein.                                                <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void internalTurnStone() {
        actualStone.turn(field);
        repaintGameLoop();
    }

    /**
     * Lässt den aktuellen Stein eine Reihe nach unten sinken.                   <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void internalDropOneRow() {
        actualStone.drop(field);
        repaintGameLoop();
    }

    /**
     * Lässt eine Pause beginnen.                                                <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void internalPauseOn() {
        paused = true;
        pauseStart = System.currentTimeMillis();

        field.pauseOn();
        gameLoopGui.pauseOn();
        /*
         * Innerhalb von gameLoopGui.pauseOn wird der EDT per invokeLater
         * beauftragt, die Anzeige der Pause vorzunehmen.
         */
    }

    /**
     * Lässt eine Pause enden.                                                   <br /><br />
     *
     * Aufruf aus der Game-Loop.
     */
    private void internalPauseOff() {
        paused = false;

        field.pauseOff();
        gameLoopGui.pauseOff();
        /*
         * Innerhalb von gameLoopGui.pauseOff wird der EDT per invokeLater
         * beauftragt, die Anzeige der Pause vorzunehmen.
         */

        /*
         * Alten Zeitpunkt um Differenz der Unterbrechung erhöhen, dann
         * kann durch Pausen keine Sinkzeit verloren gehen:
         */
        long diff =  System.currentTimeMillis() - pauseStart;
        lastTime += diff;
    }

}
