package de.duehl.pentris.logic.highscore;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import de.duehl.basics.datetime.date.ImmutualDate;
import de.duehl.basics.io.FileHelper;
import de.duehl.basics.io.FineFileReader;
import de.duehl.basics.io.FineFileWriter;

/**
 * Diese Klasse stellt die Verwaltung einer Bestenliste dar.
 *
 * @version 1.01     2014-12-09
 * @author Christian Dühl
 */

public class HighScore {

    /** Liste mit den Einträgen der Bestenliste. */
    private List<HighScoreElement> elements;

    /** Vollständiger Pfad zur Datei mit der Bestenliste. */
    private String filename;

    /**
     * Konstruktor.
     *
     * @param fileName
     *            Dateiname der Bestenliste ohne Pfad.
     */
    public HighScore(String fileName) {
        elements = new ArrayList<>();
        String path = FileHelper.getAndPerhapsCreateDirectoryAtHome(".Pentris");
        this.filename = path + File.separator + fileName;
    }

    // Es ist die Frage, ob eine Highscoreliste IM BENUTZERVERZEICHNIS
    // Sinn macht...
    // Macht sie nur bei Einzelspielern.
    // Eventuell sollte eine eigene Klasse her mit eigenem Interface, die
    // den Zugriff auf die bestenliste gewährt, einmal lokal, einmal übers
    // Netz auf einen Server. Alternativ könnten sich die Pentrisinstallationen
    // auch ihre Bestenlisten abgleichen, dann müssten die sich aber irgendwie
    // finden.

    /** Lädt die Bestenliste. */
    public void load() {
        elements.clear();

        if (!new File(filename).exists()) {
            return;
        }

        FineFileReader reader = new FineFileReader(filename);
        for (String line : reader.readFileToListOfStrings()) {
            String[] parts = line.split("\t", 3);
            String name = parts[0];
            String scorePre = parts[1];
            int score = Integer.parseInt(scorePre);
            String datePre = parts[2];
            ImmutualDate date = new ImmutualDate(datePre);
            HighScoreElement element = new HighScoreElement(name, score, date);
            elements.add(element);
        }
        sortElements();
    }

    /**
     * Ermittelt den Platz für den Score eines Benutzers.
     *
     * @param score
     *            Score des Benutzers.
     * @return Platzierung beginnend bei 1.
     */
    public int determinePlace(int score) {
        int internalPlace = internalDeterminePlace(score);

        return internalPlace + 1;
    }

    /**
     * Ermittelt den internen Platz für den Score eines Benutzers.
     *
     * @param score
     *            Score des Benutzers.
     * @return Platzierung beginnend bei 0.
     */
    private int internalDeterminePlace(int score) {
        /* Einträge absteigend nach Score sortieren: */
        sortElements();

        /* Eigenen Platz ermitteln: */
        int place = -1;
        for (int i=0; i<elements.size(); ++i) {
            HighScoreElement element = elements.get(i);
            int elementScore = element.getScore();
            if (score > elementScore) {
                place = i;
                break;
            }
        }
        if (place == -1) {
            place = elements.size();
        }
        return place;
    }

    /**
     * Trägt den Score eines Benutzers ein.
     *
     * @param score
     *            Score des Benutzers.
     * @param userName
     *            Name des Benutzers.
     * @return Platzierung beginnend bei 1.
     */
    public void addUserScore(int score, String userName) {
        int internalPlace = internalDeterminePlace(score);

        /* Eigenes Ergebnis eintragen: */
        ImmutualDate now = new ImmutualDate();
        HighScoreElement element = new HighScoreElement(userName, score, now);
        elements.add(internalPlace, element);
        sortElements();
    }

    /** Sortiert die Elemente des Highscores nach den Punkten. */
    private void sortElements() {
        Collections.sort(elements, new Comparator<HighScoreElement>() {
            @Override
            public int compare(HighScoreElement hse1, HighScoreElement hse2) {
                int score1 = hse1.getScore();
                int score2 = hse2.getScore();
                int compare = score1 - score2;
                return -compare;
            }
        });
    }

    /** Sichert die Bestenliste. */
    public void save() {
        FineFileWriter writer = new FineFileWriter(filename);
        for (HighScoreElement element : elements) {
            writer.write(element.getName());
            writer.write("\t");
            writer.write(Integer.toString(element.getScore()));
            writer.write("\t");
            writer.writeln(element.getDate().toString());
        }
        writer.close();
    }

    /** Getter für die Elemente der Highscoreliste. */
    public List<HighScoreElement> getElements() {
        return elements;
    }

}
