package de.duehl.pentris.logic.stones;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.duehl.pentris.logic.Constants;
import de.duehl.pentris.logic.Coordinate;
import de.duehl.pentris.logic.GameType;
import de.duehl.pentris.logic.field.GameField;

/**
 * Diese Klasse stellt einen Spielstein dar.
 *
 * @version 1.01     2013-08-30
 * @author Christian Dühl
 */

public class Stone {

    /** Art des Steins. */
    private StoneType stoneType;

    /** Farbe des Steins. */
    private String color;

    /** Extrapunkte für schwierige Steine. */
    private int bonus;

    /** Spielart für diesen Stein (Pentris oder Tetris). */
    private GameType gameType;

    /** Koordinaten, die der Stein einnimmt. */
    private List<Coordinate> coordinates;

    /**
     * Konstruktor.
     *
     * @param stoneType
     *            Art des Steins.
     * @param color
     *            Farbe des Steins.
     * @param bonus
     *            Extrapunkte für schwierige Steine.
     * @param gameType
     *            Spielart für diesen Stein.
     * @param coordinates
     *            Koordinaten, die der Stein einnimmt.
     */
    public Stone(StoneType stoneType, String color, int bonus,
            GameType gameType, List<Coordinate> coordinates) {
        this.stoneType = stoneType;
        this.color = color;
        this.bonus = bonus;
        this.gameType = gameType;
        this.coordinates = coordinates;
    }

    /**
     * Lässt den Stein eine Reihe nach unten sinken.
     *
     * @param field
     *            Spielfeld
     * @return Gibt an, ob der Stein sinken konnte. Wenn er auf ein Hindernis
     *         stieß oder nach ganz unten gesunken ist, wird false
     *         zurückgegeben.
     */
    synchronized
    public boolean drop(GameField field) {
        /* Feststellen, ob gesunken werden darf: */
        if (canDrop(field)) {
            /* Da wir sinken können, tun wir dies: */
            for (Coordinate coord : coordinates) {
                coord.drop();
            }
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Prüft, ob der Stein sinken kann.
     *
     * @param field
     *            Spielfeld
     * @return Gibt an, ob der Stein sinken kann. Wenn er auf ein Hindernis
     *         stoßen oder nach ganz unten sinken würde, wird false
     *         zurückgegeben, anderenfalls true.
     */
    private boolean canDrop(GameField field) {
        for (Coordinate coord : coordinates) {
            int row = coord.getRow();
            int col = coord.getCol();
            if (row < 1) {
                return false; // Unten angekommen: 0 ist unterste Zeile.
            }
            --row;
            if (!field.isFree(row, col)) {
                return false; // Feld darunter ist nicht frei.
            }
        }
        return true;
    }

    /**
     * Prüft, ob das Feld an den Positionen des Steines frei ist.
     *
     * @param field
     *            Spielfeld
     * @return Wenn auf der Position des Steines schon andere Steine liegen,
     *         wird false zurückgegeben, anderenfalls true.
     */
    synchronized
    public boolean fitsOnField(GameField field) {
        for (Coordinate coord : coordinates) {
            int row = coord.getRow();
            int col = coord.getCol();
            if (!field.isFree(row, col)) {
                return false; // Feld ist nicht frei.
            }
        }
        return true;
    }

    /** Getter für die Art des Steins. */
    public StoneType getStoneType() {
        return stoneType;
    }

    /** Getter für die Farbe des Steins. */
    public String getColor() {
        return color;
    }

    /** Getter für die Extrapunkte für diesen Stein. */
    public int getBonus() {
        return bonus;
    }

    /** Getter für die Spielart für diesen Stein (Pentris oder Tetris). */
    public GameType getGameType() {
        return gameType;
    }

    /** Getter für die Koordinaten, die der Stein einnimmt. */
    public List<Coordinate> getCoordinates() {
        return coordinates;
    }

    /**
     * Bewegt den Stein nach links.
     *
     * @param field
     *            Spielfeld
     * @return Gibt an, ob der Stein nach links bewegt werden konnte. Wenn er
     *         auf ein Hindernis oder die linke Wand gestoßen ist, wird false
     *         zurückgegeben.
     */
    synchronized
    public boolean moveLeft(GameField field) {
        /* Feststellen, ob wir nach links können darf: */
        if (canMoveLeft(field)) {
            /* Da wir sinken können, tun wir dies: */
            for (Coordinate coord : coordinates) {
                coord.left();
            }
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Prüft, ob der Stein nach links kann.
     *
     * @param field
     *            Spielfeld
     * @return Gibt an, ob der Stein nach links bewegt werden konnte. Wenn er
     *         auf ein Hindernis oder die linke Wand gestoßen ist, wird false
     *         zurückgegeben, anderenfalls true.
     */
    private boolean canMoveLeft(GameField field) {
        for (Coordinate coord : coordinates) {
            int row = coord.getRow();
            int col = coord.getCol();
            if (col < 1) {
                return false; // Links angekommen: 0 ist die Spalte ganz links
            }
            --col;
            if (!field.isFree(row, col)) {
                return false; // Feld daneben ist nicht frei.
            }
        }
        return true;
    }

    /**
     * Bewegt den Stein nach rechts.
     *
     * @param field
     *            Spielfeld
     * @return Gibt an, ob der Stein nach rechts bewegt werden konnte. Wenn er
     *         auf ein Hindernis oder die rechts Wand gestoßen ist, wird false
     *         zurückgegeben.
     */
    synchronized
    public boolean moveRight(GameField field) {
        /* Feststellen, ob wir nach links können darf: */
        if (canMoveRight(field)) {
            /* Da wir sinken können, tun wir dies: */
            for (Coordinate coord : coordinates) {
                coord.right();
            }
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * Prüft, ob der Stein nach rechts kann.
     *
     * @param field
     *            Spielfeld
     * @return Gibt an, ob der Stein nach rechts bewegt werden konnte. Wenn er
     *         auf ein Hindernis oder die rechts Wand gestoßen ist, wird false
     *         zurückgegeben, anderenfalls true.
     */
    private boolean canMoveRight(GameField field) {
        for (Coordinate coord : coordinates) {
            int row = coord.getRow();
            int col = coord.getCol();
            if (col >= Constants.COLS - 1) {
                return false; // Rechts angekommen: GameData.COLS - 1 ist die
                              // Spalte ganz rechts
            }
            ++col;
            if (!field.isFree(row, col)) {
                return false; // Feld daneben ist nicht frei.
            }
        }
        return true;
    }

    /**
     * Dreht den Stein.
     *
     * @param field
     *            Spielfeld
     * @return Gibt an, ob der Stein gedreht werden konnte.
     */
    synchronized
    public boolean turn(GameField field) {
        /*
         * Hier ist das ganze etwas komplizierter.
         *
         * 1) Vorkommende Koordinaten ermitteln:
         */
        Set<Integer> rows = new HashSet<>();
        Set<Integer> cols = new HashSet<>();
        for (Coordinate coord : coordinates) {
            rows.add(coord.getRow());
            cols.add(coord.getCol());
        }

        /*
         * 2) Mittelpunkt der vorkommenden Koordinaten berechnen:
         */
        int sumRow = 0;
        for (int row : rows) {
            sumRow += row;
        }
        int centerRow = sumRow / rows.size();

        int sumCol = 0;
        for (int col : cols) {
            sumCol += col;
        }
        int centerCol = sumCol / cols.size();

        /*
         * 3) Alle Koordinaten drehen:
         */
        List<Coordinate> newCoordinates = new ArrayList<>();
        for (Coordinate coord : coordinates) {
            int row = coord.getRow();
            int col = coord.getCol();
            int newRow = (int) (centerRow + (col - centerCol));
            int newCol = (int) (centerCol - (row - centerRow));
            Coordinate newCoordinate = new Coordinate(newRow, newCol);
            newCoordinates.add(newCoordinate);
        }

        /*
         * 4) Maximalwerte mit vorher vergleichen und eventuell verschieben:
         */
        int oldMinRow = getMinRow(coordinates);
        int oldMaxRow = getMaxRow(coordinates);
        int oldMinCol = getMinCol(coordinates);
        int oldMaxCol = getMaxCol(coordinates);
        int newMinRow = getMinRow(newCoordinates);
        int newMaxRow = getMaxRow(newCoordinates);
        int newMinCol = getMinCol(newCoordinates);
        int newMaxCol = getMaxCol(newCoordinates);

        if (newMinRow < oldMinRow && newMaxRow < oldMaxRow) {
            for (Coordinate coord : newCoordinates) {
                coord.up();
            }
        }
        if (newMinCol < oldMinCol && newMaxCol < oldMaxCol) {
            for (Coordinate coord : newCoordinates) {
                coord.right();
            }
        }

        /*
         * 5) Testen, ob der gedrehte Stein aus dem Spielfeld ragt oder über
         *    bereits belegte Spielfelder gezeichnet würde:
         */
        for (Coordinate coord : newCoordinates) {
            int row = coord.getRow();
            int col = coord.getCol();
            if (!coord.isValid()) {
                return false;
            }
            if (!field.isFree(row, col)) {
                return false;
            }
        }

        coordinates = newCoordinates;


        return true;
    }

    /**
     * Berechnet die kleinste Zeile in den gegebenen Koordinaten.
     *
     * @param coords
     *            Koordinaten
     * @return kleinste Zeile.
     */
    private int getMinRow(List<Coordinate> coords) {
        int min = Integer.MAX_VALUE;
        for (Coordinate coord : coords) {
            int row = coord.getRow();
            if (row < min) {
                min = row;
            }
        }
        return min;
    }

    /**
     * Berechnet die größte Zeile in den gegebenen Koordinaten.
     *
     * @param coords
     *            Koordinaten
     * @return größte Zeile.
     */
    private int getMaxRow(List<Coordinate> coords) {
        int max = Integer.MIN_VALUE;
        for (Coordinate coord : coords) {
            int row = coord.getRow();
            if (row > max) {
                max = row;
            }
        }
        return max;
    }

    /**
     * Berechnet die kleinste Spalte in den gegebenen Koordinaten.
     *
     * @param coords
     *            Koordinaten
     * @return kleinste Spalte.
     */
    private int getMinCol(List<Coordinate> coords) {
        int min = Integer.MAX_VALUE;
        for (Coordinate coord : coords) {
            int col = coord.getCol();
            if (col < min) {
                min = col;
            }
        }
        return min;
    }

    /**
     * Berechnet die größte Spalte in den gegebenen Koordinaten.
     *
     * @param coords
     *            Koordinaten
     * @return größte Spalte.
     */
    private int getMaxCol(List<Coordinate> coords) {
        int max = Integer.MIN_VALUE;
        for (Coordinate coord : coords) {
            int col = coord.getCol();
            if (col > max) {
                max = col;
            }
        }
        return max;
    }

}
