package de.duehl.pentris.logic.stones;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import de.duehl.pentris.logic.Constants;
import de.duehl.pentris.logic.Coordinate;
import de.duehl.pentris.logic.GameType;

/**
 * Steuert die Erstellung von Spielsteinen.
 *
 * @version 1.01     2013-08-30
 * @author Christian Dühl
 */

public class StoneFactory {

    /** Mittelpunkt der Reihe. */
    private final int middle;

    /** Objekt für Zufallswahl. */
    private Random random;

    /** Liste für Zufallsauswahl. */
    private Map<Integer, StoneType> choice;

    /** Liste für Zufallsauswahl von Tetrissteinen. */
    private Map<Integer, StoneType> tetrisChoice;

    /** Zuordnung von Farben zu den Steinen. */
    private Map<StoneType, String> typeToColor;

    /** Zuordnung von Bonuswerten zu den Steinen. */
    private Map<StoneType, Integer> typeToBonus;

    /** Zuordnung von Startkoordinaten zu den Steinen. */
    private Map<StoneType, List<Coordinate>> typeToCoordinates;

    /** Zuordnung von "Auftrittswahrscheinlichkeit" zu den Steinen. */
    private Map<StoneType, Integer> typeToRandomWeight;

    /** Spieltyp (Pentris oder Tetris). */
    private GameType gameType;

    /** Konstruktor. */
    public StoneFactory() {
        middle = Constants.COLS / 2; // -1
        random = new Random();

        typeToColor = new HashMap<>();
        typeToBonus = new HashMap<>();
        typeToCoordinates = new HashMap<>();
        typeToRandomWeight = new HashMap<>();
        fillColorMap();
        fillBonusMap();
        fillCoordiantesMap();
        adjustCoordinates();
        fillRandomWeightMap();

        choice = new HashMap<>();
        int number = 0;
        for (StoneType type : StoneType.values()) {
            for (int i=0; i<typeToRandomWeight.get(type); ++i) {
                choice.put(number++, type);
            }
        }

        tetrisChoice = new HashMap<>();
        number = 0;
        for (StoneType type : StoneType.values()) {
            if (type.name().startsWith("TETRIS_")) {
                for (int i=0; i<typeToRandomWeight.get(type); ++i) {
                    tetrisChoice.put(number++, type);
                }
            }
        }
    }

    /**
     * Legt die Farben der Steine fest. Diese Dinge sind bewusst hier in
     * Methoden und nicht mit beim Enum StoneType oder auf Methoden verteilt,
     * damit man die Farben und die anderen Werte direkt an einer zentralen
     * Stelle bearbeiten kann.
     */
    private void fillColorMap() {
        typeToColor.put(StoneType.TETRIS_I,         "EE9900");
        typeToColor.put(StoneType.TETRIS_L_RIGHT,   "FF0033");
        typeToColor.put(StoneType.TETRIS_L_LEFT,    "2200FF");
        typeToColor.put(StoneType.TETRIS_N_RIGHT,   "22CCCC");
        typeToColor.put(StoneType.TETRIS_N_LEFT,    "CC22CC");
        typeToColor.put(StoneType.TETRIS_SQUARE,    "EEDD00");
        typeToColor.put(StoneType.TETRIS_T,         "00BB00");

        typeToColor.put(StoneType.EXTRA_SQUARE_ONE, "883300");
        typeToColor.put(StoneType.EXTRA_I_TWO,      "AA5500");
        typeToColor.put(StoneType.EXTRA_I_THREE,    "CC7700");
        typeToColor.put(StoneType.EXTRA_V,          "660066");

        typeToColor.put(StoneType.PENTRIS_I,   "FFAA11");
        typeToColor.put(StoneType.PENTRIS_L_RIGHT,  "1100CC");
        typeToColor.put(StoneType.PENTRIS_L_LEFT,   "CC0011");
        typeToColor.put(StoneType.PENTRIS_Y_RIGHT,  "00AA44");
        typeToColor.put(StoneType.PENTRIS_Y_LEFT,   "AA5577");
        typeToColor.put(StoneType.PENTRIS_V,        "990099");
        typeToColor.put(StoneType.PENTRIS_T,        "009900");
        typeToColor.put(StoneType.PENTRIS_X,        "006600");
        typeToColor.put(StoneType.PENTRIS_P_RIGHT,  "1100AA");
        typeToColor.put(StoneType.PENTRIS_P_LEFT,   "AA0011");
        typeToColor.put(StoneType.PENTRIS_F_RIGHT,  "CCCCFF");
        typeToColor.put(StoneType.PENTRIS_F_WRONG,  "CCFFCC");
        typeToColor.put(StoneType.PENTRIS_U,        "88EEAA");
        typeToColor.put(StoneType.PENTRIS_Z_RIGHT,  "FF00FF");
        typeToColor.put(StoneType.PENTRIS_Z_LEFT,   "00FF00");
        typeToColor.put(StoneType.PENTRIS_W,        "110099");
        typeToColor.put(StoneType.PENTRIS_N_RIGHT,  "11DDAA");
        typeToColor.put(StoneType.PENTRIS_N_LEFT,   "DD88AA");
    }

    /** Legt die Bonuswerte der Farben fest. */
    private void fillBonusMap() {
        typeToBonus.put(StoneType.TETRIS_I,          0);
        typeToBonus.put(StoneType.TETRIS_L_RIGHT,    0);
        typeToBonus.put(StoneType.TETRIS_L_LEFT,     0);
        typeToBonus.put(StoneType.TETRIS_N_RIGHT,    0);
        typeToBonus.put(StoneType.TETRIS_N_LEFT,     0);
        typeToBonus.put(StoneType.TETRIS_SQUARE,     0);
        typeToBonus.put(StoneType.TETRIS_T,          0);

        typeToBonus.put(StoneType.EXTRA_SQUARE_ONE,  0);
        typeToBonus.put(StoneType.EXTRA_I_TWO,       0);
        typeToBonus.put(StoneType.EXTRA_I_THREE,     0);
        typeToBonus.put(StoneType.EXTRA_V,           0);

        typeToBonus.put(StoneType.PENTRIS_I,        10);
        typeToBonus.put(StoneType.PENTRIS_L_RIGHT,  10);
        typeToBonus.put(StoneType.PENTRIS_L_LEFT,   10);
        typeToBonus.put(StoneType.PENTRIS_Y_RIGHT,  20);
        typeToBonus.put(StoneType.PENTRIS_Y_LEFT,   20);
        typeToBonus.put(StoneType.PENTRIS_V,        20);
        typeToBonus.put(StoneType.PENTRIS_T,        15);
        typeToBonus.put(StoneType.PENTRIS_X,        25);
        typeToBonus.put(StoneType.PENTRIS_P_RIGHT,  15);
        typeToBonus.put(StoneType.PENTRIS_P_LEFT,   15);
        typeToBonus.put(StoneType.PENTRIS_F_RIGHT,  30);
        typeToBonus.put(StoneType.PENTRIS_F_WRONG,  30);
        typeToBonus.put(StoneType.PENTRIS_U,        20);
        typeToBonus.put(StoneType.PENTRIS_Z_RIGHT,  30);
        typeToBonus.put(StoneType.PENTRIS_Z_LEFT,   30);
        typeToBonus.put(StoneType.PENTRIS_W,        30);
        typeToBonus.put(StoneType.PENTRIS_N_RIGHT,  15);
        typeToBonus.put(StoneType.PENTRIS_N_LEFT,   15);
    }

    /** Legt die Startkoordinaten der Steine fest. */
    private void fillCoordiantesMap() {
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(0, middle+2));
            typeToCoordinates.put(StoneType.TETRIS_I,         coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle));
            typeToCoordinates.put(StoneType.TETRIS_L_RIGHT,   coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle));
            typeToCoordinates.put(StoneType.TETRIS_L_LEFT,    coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(2, middle+1));
            typeToCoordinates.put(StoneType.TETRIS_N_RIGHT,   coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(2, middle));
            typeToCoordinates.put(StoneType.TETRIS_N_LEFT,    coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(1, middle+1));
            typeToCoordinates.put(StoneType.TETRIS_SQUARE,    coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(1, middle));
            typeToCoordinates.put(StoneType.TETRIS_T,         coords);
        }

        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            typeToCoordinates.put(StoneType.EXTRA_SQUARE_ONE, coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(1, middle));
            typeToCoordinates.put(StoneType.EXTRA_I_TWO,      coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(0, middle+2));
            typeToCoordinates.put(StoneType.EXTRA_I_THREE,    coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle));
            typeToCoordinates.put(StoneType.EXTRA_V,          coords);
        }

        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(0, middle+2));
            coords.add(new Coordinate(0, middle-2));
            typeToCoordinates.put(StoneType.PENTRIS_I,        coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(2, middle));
            coords.add(new Coordinate(3, middle));
            typeToCoordinates.put(StoneType.PENTRIS_L_RIGHT,  coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle+1));
            coords.add(new Coordinate(3, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_L_LEFT,   coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle));
            coords.add(new Coordinate(3, middle));
            coords.add(new Coordinate(1, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_Y_RIGHT,  coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(2, middle+1));
            coords.add(new Coordinate(3, middle+1));
            coords.add(new Coordinate(1, middle));
            typeToCoordinates.put(StoneType.PENTRIS_Y_LEFT,   coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(1, middle-1));
            coords.add(new Coordinate(2, middle-1));
            typeToCoordinates.put(StoneType.PENTRIS_V,        coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(2, middle));
            typeToCoordinates.put(StoneType.PENTRIS_T,        coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(1, middle-1));
            typeToCoordinates.put(StoneType.PENTRIS_X,        coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_P_RIGHT,  coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(2, middle+1));
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            typeToCoordinates.put(StoneType.PENTRIS_P_LEFT,   coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle));
            coords.add(new Coordinate(1, middle-1));
            coords.add(new Coordinate(0, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_F_RIGHT,  coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle-1));
            typeToCoordinates.put(StoneType.PENTRIS_F_WRONG,  coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(1, middle-1));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_U,        coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(1, middle-1));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(2, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_Z_RIGHT,  coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(1, middle-1));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(2, middle-1));
            coords.add(new Coordinate(0, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_Z_LEFT,   coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(0, middle-1));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(2, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_W,        coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(0, middle));
            coords.add(new Coordinate(2, middle+1));
            coords.add(new Coordinate(3, middle+1));
            typeToCoordinates.put(StoneType.PENTRIS_N_RIGHT,  coords);
        }
        {
            List<Coordinate> coords = new ArrayList<>();
            coords.add(new Coordinate(1, middle));
            coords.add(new Coordinate(1, middle+1));
            coords.add(new Coordinate(0, middle+1));
            coords.add(new Coordinate(2, middle));
            coords.add(new Coordinate(3, middle));
            typeToCoordinates.put(StoneType.PENTRIS_N_LEFT,   coords);
        }
    }

    /** Korrigiert die rows, die in 0, 1, ... angelegt wurden ans obere Ende. */
    private void adjustCoordinates() {
        for (List<Coordinate> list : typeToCoordinates.values()) {
            for (Coordinate coord : list) {
                coord.adjustCoordinates();
            }
        }
    }

    /** Legt die Häufigkeit des Vorkommens der Steine fest. */
    private void fillRandomWeightMap() {
        typeToRandomWeight.put(StoneType.TETRIS_I,         6);
        typeToRandomWeight.put(StoneType.TETRIS_L_RIGHT,   6);
        typeToRandomWeight.put(StoneType.TETRIS_L_LEFT,    6);
        typeToRandomWeight.put(StoneType.TETRIS_N_RIGHT,   6);
        typeToRandomWeight.put(StoneType.TETRIS_N_LEFT,    6);
        typeToRandomWeight.put(StoneType.TETRIS_SQUARE,    6);
        typeToRandomWeight.put(StoneType.TETRIS_T,         6);

        typeToRandomWeight.put(StoneType.EXTRA_SQUARE_ONE, 3);
        typeToRandomWeight.put(StoneType.EXTRA_I_TWO,      3);
        typeToRandomWeight.put(StoneType.EXTRA_I_THREE,    3);
        typeToRandomWeight.put(StoneType.EXTRA_V,          3);

        typeToRandomWeight.put(StoneType.PENTRIS_I,        1);
        typeToRandomWeight.put(StoneType.PENTRIS_L_RIGHT,  1);
        typeToRandomWeight.put(StoneType.PENTRIS_L_LEFT,   1);
        typeToRandomWeight.put(StoneType.PENTRIS_Y_RIGHT,  1);
        typeToRandomWeight.put(StoneType.PENTRIS_Y_LEFT,   1);
        typeToRandomWeight.put(StoneType.PENTRIS_V,        1);
        typeToRandomWeight.put(StoneType.PENTRIS_T,        1);
        typeToRandomWeight.put(StoneType.PENTRIS_X,        1);
        typeToRandomWeight.put(StoneType.PENTRIS_P_RIGHT,  1);
        typeToRandomWeight.put(StoneType.PENTRIS_P_LEFT,   1);
        typeToRandomWeight.put(StoneType.PENTRIS_F_RIGHT,  1);
        typeToRandomWeight.put(StoneType.PENTRIS_F_WRONG,  1);
        typeToRandomWeight.put(StoneType.PENTRIS_U,        1);
        typeToRandomWeight.put(StoneType.PENTRIS_Z_RIGHT,  1);
        typeToRandomWeight.put(StoneType.PENTRIS_Z_LEFT,   1);
        typeToRandomWeight.put(StoneType.PENTRIS_W,        1);
        typeToRandomWeight.put(StoneType.PENTRIS_N_RIGHT,  1);
        typeToRandomWeight.put(StoneType.PENTRIS_N_LEFT,   1);
    }

    /** Erstellt einen zufälligen Stein. */
    public Stone createRandomStone() {
        if (gameType == GameType.PENTRIS) {
            return createRandomPentrisStone();
        }
        else {
            return createRandomTetrisStone();
        }
    }

    /** Erstellt einen zufälligen Pentris-Stein (beinhaltet alle anderen). */
    private Stone createRandomPentrisStone() {
        int randomChoiceIndex = random.nextInt(choice.size());
        StoneType type = choice.get(randomChoiceIndex);
        Stone stone = create(type);

        return stone;
    }

    /** Erstellt einen zufälligen Tetris-Stein. */
    private Stone createRandomTetrisStone() {
        int randomChoiceIndex = random.nextInt(tetrisChoice.size());
        StoneType type = tetrisChoice.get(randomChoiceIndex);
        Stone stone = create(type);

        return stone;
    }

    /**
     * Erstellt einen Stein abhängig vom Typ des Steins.
     *
     * @param stoneType
     *            Typ des Steins.
     * @return Stein
     */
    private Stone create(StoneType stoneType) {
        String color = typeToColor.get(stoneType);
        int bonus = typeToBonus.get(stoneType);
        GameType gameType;
        if (stoneType.name().startsWith("TETRIS")) {
            gameType = GameType.TETRIS;
        }
        else {
            gameType = GameType.PENTRIS;
        }
        /*
         * Die Koordinaten müssen kopiert werden, sonst ist ein Stein einer
         * schon auf dem Spielfeld vorhandenen Steinart plötzlich nicht mehr
         * schön oben, sondern unten, wo der andere eingebaut wurde!
         */
        List<Coordinate> coords = new ArrayList<>();
        for (Coordinate coord : typeToCoordinates.get(stoneType)) {
            Coordinate copy = new Coordinate(coord);
            coords.add(copy);
        }
        Stone stone = new Stone(stoneType, color, bonus, gameType, coords);

        return stone;
    }

    /**
     * Setter für den Spieltyp. Er entscheidet darüber, welche Art von Steinen
     * erzeugt werden soll.
     *
     * @param gameType
     *            Spieltyp (Pentris oder Tetris)
     */
    public void setGameType(GameType gameType) {
        this.gameType = gameType;
    }

    /**
     * Ermittelt die in den Steinen vorkommenden Farben für die zufälligen
     * Farben bei der Pause.
     *
     * @return Liste mit den Steinfarben.
     */
    public List<String> getStoneColors() {
        List<String> stoneColors = new ArrayList<>();
        for (StoneType type : StoneType.values()) {
            Stone stone = create(type);
            String stoneColor = stone.getColor();
            if (!stoneColors.contains(stoneColor)) {
                stoneColors.add(stoneColor);
            }
        }
        return stoneColors;
    }

}
