package de.duehl.pentris.ui;

import java.awt.Color;
import java.awt.Graphics;

import javax.swing.JPanel;

import de.duehl.pentris.logic.Constants;
import de.duehl.pentris.logic.Coordinate;
import de.duehl.pentris.logic.stones.Stone;

import de.duehl.swing.ui.colors.Colorizer;

/**
 * Diese Klasse stellt die Anzeige des nächsten Steines rechts oben dar.
 *
 * @version 1.01     2013-09-03
 * @author Christian Dühl
 */

public class NextStonePanel extends JPanel {

    private static final long serialVersionUID = -5317026944356679440L;
    private static final boolean DEBUG = false;

    /** Stein, der angezeigt werden soll. */
    private Stone stone;

    /** Farbverwaltung für die Gui. */
    private Colorizer colorizer;

    /** Gibt an, ob das Grid eingezeichnet wird. */
    private boolean showGrid;

    /**
     * Konstruktor.
     *
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param data
     *            Spieldaten (für Grid an / aus).
     */
    public NextStonePanel(Colorizer colorizer) {
        this.colorizer = colorizer;
        colorizer.setColors(this);
        setBackground(Color.BLACK);
        showGrid = true;
    }

    /**
     * Auf Panel zeichnen.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    @Override
    public void paintComponent(Graphics graphics) {
        super.paintComponent(graphics);

        if (stone != null) {
            paintStone(graphics);
            if (showGrid) {
                paintGrid(graphics);
            }
            paintStoneName(graphics);
        }
    }

    /**
     * Während der Entwicklung wird der Steintyp unten auch ausgeschrieben
     * angezeigt, falls einer der Steine nicht so dargestellt wird, wie er
     * soll.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    private void paintStoneName(Graphics graphics) {
        if (DEBUG) {
            String stoneColor = stone.getColor();
            Color color = colorizer.hexColorToJavaColor(stoneColor);
            color = colorizer.anticolor(color);
            graphics.setColor(color);
            String text = "Stein: " + stone.getStoneType().name();
            graphics.drawString(text, 10, 145);
        }
    }

    /**
     * Zeichnet den Stein ein.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    private void paintStone(Graphics graphics) {
        String stoneColor = stone.getColor();
        Color color = colorizer.hexColorToJavaColor(stoneColor);

        /*
         * Da die Steine mit Koordinaten irgendwo oben in der Mitte des
         * Spielfeldes erzeugt werden, müssen wir sie für eine schöne Anzeige im
         * 5x5 Felder umfasssenden Vorschaufenster umrechnen:
         *
         * Zunächst werden die maximalen Ausmaße berechnet:
         */
        int maxRow = Integer.MIN_VALUE;
        int minRow = Integer.MAX_VALUE;
        int maxCol = Integer.MIN_VALUE;
        int minCol = Integer.MAX_VALUE;
        for (Coordinate coordinate : stone.getCoordinates()) {
            int row = coordinate.getRow();
            if (row < minRow) {
                minRow = row;
            }
            if (row > maxRow) {
                maxRow = row;
            }
            int col = coordinate.getCol();
            if (col < minCol) {
                minCol = col;
            }
            if (col > maxCol) {
                maxCol = col;
            }
        }
        debugPrint("Minimal und Maximalwerte: minRow=" + minRow
                + ", maxRow=" + maxRow + ", minCol=" + minCol + ", maxCol="
                + maxCol);
        /*
         * Logisch ist für uns die Null unten, beim Zeichnen ist sie dummer
         * Weise oben. Deshalb drehen wir oben und unten um:
         */
        maxRow = Constants.ROWS-1 - maxRow;
        minRow = Constants.ROWS-1 - minRow;
        debugPrint("Minimal und Maximalwerte nach drehen: minRow=" + minRow
                + ", maxRow=" + maxRow + ", minCol=" + minCol + ", maxCol="
                + maxCol);

        /* links bzw. oben anzuzeigenden Leerraum berechnen: */
        int leftOverRow = (5 - (maxRow+1 - minRow)) / 2;
        int leftOverCol = (5 - (maxCol+1 - minCol)) / 2;
        debugPrint("Leftovers: leftOverRow=" + leftOverRow
                + ", leftOverCol=" + leftOverCol);

        for (Coordinate coordinate : stone.getCoordinates()) {
            int row = coordinate.getRow();
            debugPrint("row vorher : " + row);

            /*
             * Logisch ist für uns die Null unten, beim Zeichnen ist sie dummer
             * Weise oben. Deshalb drehen wir oben und unten um:
             */
            row = Constants.ROWS-1 - row;
            debugPrint("row nach umdrehen: " + row);

            row = row - minRow + leftOverRow;
            debugPrint("row nachher: " + row);
            int col = coordinate.getCol();
            debugPrint("col vorher : " + col);
            col = col - minCol + leftOverCol;
            debugPrint("col nachher: " + col);

            int x = col * Constants.COL_PIXEL;
            int y = row * Constants.ROW_PIXEL;

            graphics.setColor(color);
            graphics.fillRect(x, y, Constants.COL_PIXEL, Constants.ROW_PIXEL);
        }
    }

    /**
     * Gibt eine Meldung aus, wenn der Debugmodus eingeschaltet ist.
     *
     * @param message
     *            Auszugebende Nachricht.
     */
    private void debugPrint(String message) {
        if (DEBUG) {
            System.out.println(message);
        }
    }

    /**
     * Zeichnet das Grid, wenn gewünscht.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    private void paintGrid(Graphics graphics) {
        Color gridColor = Color.WHITE;
        graphics.setColor(gridColor);
        int numberOfColumns = 5;
        int numberOfRows = 5;
        int minX = 0;
        int maxX = (numberOfColumns + 1) * Constants.COL_PIXEL;
        int minY = 0;
        int maxY = (numberOfRows + 1) * Constants.ROW_PIXEL;
        /* Horizontale Linien zeichnen: */
        for (int i=0; i<=numberOfRows; ++i) {
            int y = i * Constants.COL_PIXEL;
            graphics.drawLine(minX, y, maxX, y);
            graphics.drawLine(minX, y-1, maxX, y-1);
        }
        /* Vertikale Linien zeichnen: */
        for (int i=0; i<=numberOfRows; ++i) {
            int x = i * Constants.ROW_PIXEL;
            graphics.drawLine(x, minY, x, maxY);
            graphics.drawLine(x-1, minY, x-1, maxY);
        }
    }

    /**
     * Setzt den anzuzeigenden Stein.
     *
     * @param stone
     *            Anzuzeigender Stein.
     */
    public void setNextStone(Stone stone) {
        this.stone = stone;
        repaint();
    }

    /** Schaltet das Grid an oder aus. */
    public void setShowGrid(boolean showGrid) {
        this.showGrid = showGrid;
        repaint();
    }

}
