package de.duehl.pentris.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;

import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.swing.ui.listener.ClosingWindowListener;
import de.duehl.swing.ui.resources.IconLoader;
import de.duehl.pentris.logic.Constants;
import de.duehl.pentris.logic.GameType;
import de.duehl.pentris.logic.Logic;
import de.duehl.pentris.ui.dialogs.GameStartDialog;
import de.duehl.pentris.ui.dialogs.QuitDialog;
import de.duehl.pentris.ui.gameloop.GameLoopGui;
import de.duehl.pentris.ui.resources.IconDefinitions;

/**
 * Diese Klasse erzeugt die grafische Oberfläche von Pentris.
 *
 * @version 1.01     2019-07-09
 * @author Christian Dühl
 */

public class PentrisGui implements Gui {

    public static final boolean WINDOWS_10 = true;

    /** Zugriff auf die Programmlogik */
    private Logic logic;

    /** Farbverwaltung für die Gui. */
    private Colorizer colorizer;

    /** Mainframe. */
    private JFrame frame;

    /** Bild des Programms. */
    private Image programImage;

    /** Breite des linken Teils der Gui. */
    private int leftPartWidth;

    /** Breite des rechten Teils der Gui. */
    private int rightPartWidth;

    /** Breite des zentralen Teils der Gui. */
    private int centerPartWidth;

    /** Höhe der Gui. */
    private int height;

    /** Panel mit dem Spielfeld. */
    private PentrisPanel pentrisPanel;

    /** Panel mit der Anzeige des nächsten Steins. */
    private NextStonePanel nextStonePanel;

    /** Anzeige des aktuellen Punktestandes. */
    private JLabel pointsLabel;

    private JPanel highscorePanel;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Verbindung zur Logik.
     */
    public PentrisGui(Logic logic) {
        this.logic = logic;

        leftPartWidth = 200;
        rightPartWidth = 51 + 5 * Constants.COL_PIXEL;
        centerPartWidth = 6 + Constants.COLS * Constants.COL_PIXEL;
        height = 25 + Constants.ROWS * Constants.ROW_PIXEL;
        // Diese magischen Zahlen stimmen unter Windows 7. Warum auch immer.

        // Unter Windows 10 nötige Korrektur:
        if (WINDOWS_10) {
            centerPartWidth += 12;
            height += 15;
        }

        colorizer = new PentrisColorizer();

        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        programImage = iconLoader.loadProgramIconImage();

        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                createGui();
                setKeyBindings();
            }
        });
    }

    /** Erstellt die grafische Oberfläche. */
    private void createGui() {
        frame = new JFrame();
        colorizer.setColors(frame);
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.setIconImage(programImage);
        frame.setLayout(new BorderLayout());
        frame.setResizable(false);
        //frame.setUndecorated(true);
        frame.addWindowListener(new ClosingWindowListener(logic));
        String title = "Pentris "
                + logic.getVersion().getVersion() + " vom "
                + logic.getVersion().getDate();
        frame.setTitle(title);

        Dimension dimension = new Dimension(
                leftPartWidth + centerPartWidth + rightPartWidth, height);
        frame.setPreferredSize(dimension);
        frame.setLocation(300, 200);


        /* Linken Teil anlegen: */
        frame.add(createLeftPart(), BorderLayout.WEST);

        /* Mittleren Teil anlegen: */
        frame.add(createCenterPart(), BorderLayout.CENTER);

        /* Unteren Teil anlegen: */
        frame.add(createRightPart(), BorderLayout.EAST);

        GameLoopGui gameLoopGui = new GameLoopGui(frame, programImage,
                pentrisPanel, nextStonePanel, pointsLabel, highscorePanel,
                colorizer);
        logic.setGameLoopGui(gameLoopGui);
    }

    /** Erstellt den linken Bereich mit der Erklärung der Tastenbelegung. */
    private Component createLeftPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new VerticalLayout(10, VerticalLayout.BOTH));
        colorizer.setColors(panel);
        //colorizer.setStandardBorder(panel, "left");
        panel.setPreferredSize(new Dimension(leftPartWidth, height));

        JLabel titleLabel = new JLabel("<html><h1>Tastenbelegung</h1></html>");
        colorizer.setColors(titleLabel);
        panel.add(titleLabel);

        List<String> keyTexts = new ArrayList<>();
        String start = "<html>&nbsp;&nbsp;<b><span style=\"font-size:13px\">";
        String mid = "</span></b><br/>"
                + "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
        String end = "</html>";
        keyTexts.add(start + "P, Pause" + mid + "Pause" + end);
        keyTexts.add(start + "Space" + mid + "Stein fallen lassen" + end);
        keyTexts.add(start + "Pause" + mid + "Pause" + end);
        keyTexts.add(start + "links" + mid + "Stein nach links" + end);
        keyTexts.add(start + "rechts" + mid + "Stein nach rechts" + end);
        keyTexts.add(start + "oben" + mid + "Stein drehen" + end);
        keyTexts.add(start + "unten" + mid + "Stein eine Reihe nach unten" + end);
        keyTexts.add(start + "F12" + mid + "Grid an- bzw. ausschalten" + end);
        for (String keyText : keyTexts) {
            JLabel label = new JLabel(keyText);
            colorizer.setColors(label);
            panel.add(label);
        }

        return panel;
    }

    /** Erstellt den mittleren Bereich mit dem Spielfeld. */
    private Component createCenterPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        colorizer.setColors(panel);
        //colorizer.setStandardBorder(panel, "center");
        panel.setPreferredSize(new Dimension(centerPartWidth, height));

        pentrisPanel = new PentrisPanel(colorizer, logic.getField());
        panel.add(pentrisPanel, BorderLayout.CENTER);

        return panel;
    }

    /**
     * Erstellt den Bereich auf der rechten Seite mit der Anzeige des nächsten
     * Steins.
     *
     * @return Erzeugter Panel.
     */
    private Component createRightPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        colorizer.setColors(panel);
        panel.setPreferredSize(new Dimension(rightPartWidth, height));

        {
            JPanel subPanel = new JPanel();
            subPanel.setLayout(new VerticalLayout(10, VerticalLayout.CENTER));
            colorizer.setColors(subPanel);
            //colorizer.setStandardBorder(panel, "right");

            JLabel label = new JLabel("<html><h1>Nächster Stein</h1></html>");
            colorizer.setColors(label);
            subPanel.add(label);

            /* Anzeige für nächsten Stein: */
            nextStonePanel = new NextStonePanel(colorizer);
            nextStonePanel.setPreferredSize(new Dimension(
                    1 + 5 * Constants.COL_PIXEL,
                    1 + 5 * Constants.ROW_PIXEL
                    ));
            subPanel.add(nextStonePanel);

            JLabel label2 = new JLabel("<html><h1>Punkte</h1></html>");
            colorizer.setColors(label2);
            subPanel.add(label2);

            pointsLabel = new JLabel("0");
            colorizer.setColors(pointsLabel);
            subPanel.add(pointsLabel);
            panel.add(subPanel, BorderLayout.NORTH);
        }

        {
            highscorePanel = new JPanel();
            highscorePanel.setLayout(new VerticalLayout(10, VerticalLayout.BOTH));
            colorizer.setColors(highscorePanel);
            panel.add(highscorePanel, BorderLayout.CENTER);
        }

        return panel;
    }

    /** Erstellt Tastenkombinationen. */
    private void setKeyBindings() {
        JRootPane rootPane = frame.getRootPane();
        InputMap inputMap = rootPane.getInputMap(
                JComponent.WHEN_IN_FOCUSED_WINDOW);

        setKeyBindingEscape(rootPane, inputMap);
        setKeyBindingP(rootPane, inputMap);
        setKeyBindingSpace(rootPane, inputMap);
        setKeyBindingLeft(rootPane, inputMap);
        setKeyBindingRight(rootPane, inputMap);
        setKeyBindingUp(rootPane, inputMap);
        setKeyBindingDown(rootPane, inputMap);
        setKeyBindingF12(rootPane, inputMap);
    }

    /**
     * Fügt die Tastaturbindung Escape hinzu.
     *
     * @param rootPane
     *            RootPane des Frames.
     * @param inputMap
     *            InputMap des Frames im fokussierten Fenster.
     */
    private void setKeyBindingEscape(JRootPane rootPane, InputMap inputMap) {
        KeyStroke escapeStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
        inputMap.put(escapeStroke, "ESCAPE");

        Action escapeAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                logic.quit();
            }
        };

        rootPane.getActionMap().put("ESCAPE", escapeAction);
    }

    /**
     * Fügt die Tastaturbindung Escape hinzu.
     *
     * @param rootPane
     *            RootPane des Frames.
     * @param inputMap
     *            InputMap des Frames im fokussierten Fenster.
     */
    private void setKeyBindingP(JRootPane rootPane, InputMap inputMap) {
        KeyStroke pStroke = KeyStroke.getKeyStroke(KeyEvent.VK_P, 0);
        inputMap.put(pStroke, "PAUSE");
        KeyStroke pauseStroke = KeyStroke.getKeyStroke(KeyEvent.VK_PAUSE, 0);
        inputMap.put(pauseStroke, "PAUSE");

        Action pauseAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                logic.pause();
            }
        };

        rootPane.getActionMap().put("PAUSE", pauseAction);
    }

    /**
     * Fügt die Tastaturbindung Leertaste hinzu.
     *
     * @param rootPane
     *            RootPane des Frames.
     * @param inputMap
     *            InputMap des Frames im fokussierten Fenster.
     */
    private void setKeyBindingSpace(JRootPane rootPane, InputMap inputMap) {
        KeyStroke spaceStroke = KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0);
        inputMap.put(spaceStroke, "FREE FALL");

        Action freeFallAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                logic.freeFall();
            }
        };

        rootPane.getActionMap().put("FREE FALL", freeFallAction);
    }

    /**
     * Fügt die Tastaturbindung Pfeil nach links hinzu.
     *
     * @param rootPane
     *            RootPane des Frames.
     * @param inputMap
     *            InputMap des Frames im fokussierten Fenster.
     */
    private void setKeyBindingLeft(JRootPane rootPane, InputMap inputMap) {
        KeyStroke leftStroke = KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0);
        inputMap.put(leftStroke, "STEIN NACH LINKS");

        Action leftAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                logic.moveStoneLeft();
            }
        };

        rootPane.getActionMap().put("STEIN NACH LINKS", leftAction);
    }

    /**
     * Fügt die Tastaturbindung Pfeil nach rechts hinzu.
     *
     * @param rootPane
     *            RootPane des Frames.
     * @param inputMap
     *            InputMap des Frames im fokussierten Fenster.
     */
    private void setKeyBindingRight(JRootPane rootPane, InputMap inputMap) {
        KeyStroke rightStroke = KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0);
        inputMap.put(rightStroke, "STEIN NACH RECHTS");

        Action rightAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                logic.moveStoneRight();
            }
        };

        rootPane.getActionMap().put("STEIN NACH RECHTS", rightAction);
    }

    /**
     * Fügt die Tastaturbindung Pfeil nach oben hinzu.
     *
     * @param rootPane
     *            RootPane des Frames.
     * @param inputMap
     *            InputMap des Frames im fokussierten Fenster.
     */
    private void setKeyBindingUp(JRootPane rootPane, InputMap inputMap) {
        KeyStroke upStroke = KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0);
        inputMap.put(upStroke, "STEIN DREHEN");

        Action turnAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                logic.turnStone();
            }
        };

        rootPane.getActionMap().put("STEIN DREHEN", turnAction);
    }

    /**
     * Fügt die Tastaturbindung Pfeil nach unten hinzu.
     *
     * @param rootPane
     *            RootPane des Frames.
     * @param inputMap
     *            InputMap des Frames im fokussierten Fenster.
     */
    private void setKeyBindingDown(JRootPane rootPane, InputMap inputMap) {
        KeyStroke downStroke = KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0);
        inputMap.put(downStroke, "STEIN NACH UNTEN");

        Action dropOneRowAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                logic.dropStoneOneRow();
            }
        };

        rootPane.getActionMap().put("STEIN NACH UNTEN", dropOneRowAction);
    }

    /**
     * Fügt die Tastaturbindung Pfeil nach unten hinzu.
     *
     * @param rootPane
     *            RootPane des Frames.
     * @param inputMap
     *            InputMap des Frames im fokussierten Fenster.
     */
    private void setKeyBindingF12(JRootPane rootPane, InputMap inputMap) {
        KeyStroke f12Stroke = KeyStroke.getKeyStroke(KeyEvent.VK_F12, 0);
        inputMap.put(f12Stroke, "GRID AN AUS");

        Action dropOneRowAction = new AbstractAction() {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                logic.switchGrid();
            }
        };

        rootPane.getActionMap().put("GRID AN AUS", dropOneRowAction);
    }

    /** Zeigt die Oberfläche an. */
    @Override
    public void show() {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                frame.pack();
                frame.setVisible(true);
            }
        });
    }

    /** Fragt den Benutzer, ob das Programm beendet werden soll. */
    @Override
    public boolean askUserToQuit() {
        QuitDialog dialog = new QuitDialog(colorizer, frame, programImage);
        dialog.setVisible(true);
        boolean quit = dialog.doQuit();
        return quit;
    }

    /** Beendet die Oberfläche. */
    @Override
    public void quit() {
        frame.setVisible(false);
        frame.dispose();
    }

    /**
     * Fragt den Benutzer, ob er ein neues Spiel anfangen will.
     *
     * @return Spieltyp oder null bei Abbruch durch den Benutzer.
     */
    @Override
    public GameType askUserToStartANewGame() {
        GameStartDialog dialog = new GameStartDialog(colorizer, frame,
                programImage);
        dialog.setVisible(true);
        GameType gameType = dialog.getGameType();
        return gameType;
    }

    /** Holt die Gui in den Vordergrund. */
    @Override
    public void getToFront() {
        frame.requestFocus();
    }

}
