package de.duehl.pentris.ui;

import java.awt.Color;
import java.awt.Graphics;

import javax.swing.JPanel;

import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.pentris.logic.Constants;
import de.duehl.pentris.logic.Coordinate;
import de.duehl.pentris.logic.field.GameField;
import de.duehl.pentris.logic.stones.Stone;

/**
 * Diese Klasse stellt das Spielfeld in der Mitte dar.
 *
 * @version 1.01     2013-08-29
 * @author Christian Dühl
 */

public class PentrisPanel extends JPanel {

    private static final long serialVersionUID = 3466625716141953648L;

    /** Farbverwaltung für die Gui. */
    private Colorizer colorizer;

    /** Spielfeld. */
    private volatile GameField field;

    /** Stein, der angezeigt werden soll. */
    private Stone stone;

    /** Gibt an, ob der fallende Stein angezeigt werden soll. */
    private boolean paintStone;

    /** Gibt an, ob das Grid eingezeichnet wird. */
    private boolean showGrid;

    /**
     * Konstruktor.
     *
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param field
     *            Spielfeld.
     */
    public PentrisPanel(Colorizer colorizer, GameField field) {
        this.colorizer = colorizer;
        this.field = field;
        paintStone = true;
        showGrid = true;

        colorizer.setColors(this);
        setBackground(Color.BLACK);
    }

    /**
     * Auf Panel zeichnen.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    @Override
    public void paintComponent(Graphics graphics) {
        super.paintComponent(graphics);

        if (stone != null) {
            paintField(graphics);
            if (paintStone) { // bei Pause false
                paintStone(graphics);
            }
            if (showGrid) {
                paintGrid(graphics);
            }
        }
    }

    /**
     * Zeichnet das Spielfeld.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    private void paintField(Graphics graphics) {
        for (int row=0; row<Constants.ROWS; ++row) {
            for (int col=0; col<Constants.COLS; ++col) {
                boolean free = field.isFree(row, col);
                if (!free) {
                    String colorString = field.getColor(row, col);
                    Color fieldColor =
                            colorizer.hexColorToJavaColor(colorString);

                    /*
                     * Logisch ist für uns die Null unten, beim Zeichnen ist sie
                     * dummer Weise oben. Deshalb drehen wir oben und unten um:
                     */
                    int turnedRow = Constants.ROWS-1 - row;

                    int x = col * Constants.COL_PIXEL;
                    int y = turnedRow * Constants.ROW_PIXEL;
                    graphics.setColor(fieldColor);
                    graphics.fillRect(x, y, Constants.COL_PIXEL,
                            Constants.ROW_PIXEL);
                }
            }
        }
    }

    /**
     * Zeichnet den Stein ein.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    private void paintStone(Graphics graphics) {
        String stoneColor = stone.getColor();
        Color color = colorizer.hexColorToJavaColor(stoneColor);

        for (Coordinate coordinate : stone.getCoordinates()) {
            int row = coordinate.getRow();
            /*
             * Logisch ist für uns die Null unten, beim Zeichnen ist sie dummer
             * Weise oben. Deshalb drehen wir oben und unten um:
             */
            row = Constants.ROWS-1 - row;
            int col = coordinate.getCol();

            int x = col * Constants.COL_PIXEL;
            int y = row * Constants.ROW_PIXEL;
            graphics.setColor(color);
            graphics.fillRect(x, y, Constants.COL_PIXEL, Constants.ROW_PIXEL);
        }
    }

    /**
     * Zeichnet das Grid, wenn gewünscht.
     *
     * @param graphics
     *            Grafikobjekt.
     */
    private void paintGrid(Graphics graphics) {
        Color gridColor = Color.WHITE;
        graphics.setColor(gridColor);
        int numberOfColumns = Constants.COLS;
        int numberOfRows = Constants.ROWS;
        int minX = 0;
        int maxX = numberOfColumns * Constants.COL_PIXEL;
        int minY = 0;
        int maxY = numberOfRows * Constants.ROW_PIXEL;
        /* Horizontale Linien zeichnen: */
        for (int i=0; i<=numberOfRows; ++i) {
            int y = i * Constants.COL_PIXEL;
            graphics.drawLine(minX, y, maxX, y);
            graphics.drawLine(minX, y-1, maxX, y-1);
        }
        /* Vertikale Linien zeichnen: */
        for (int i=0; i<=numberOfColumns; ++i) {
            int x = i * Constants.ROW_PIXEL;
            graphics.drawLine(x, minY, x, maxY);
            graphics.drawLine(x-1, minY, x-1, maxY);
        }
    }

    /**
     * Setzt den anzuzeigenden, fallenden Stein.
     *
     * @param stone
     *            Anzuzeigender Stein.
     */
    public void setStone(Stone stone) {
        this.stone = stone;
        repaint();
    }

    /**
     * Setter, ob der Stein gezeichnet werden soll (in Pausen nicht!)
     *
     * @param paintStone
     *            Wahrheitswert, ob der Stein dargestellt werden soll.
     */
    public void setPaintStone(boolean paintStone) {
        this.paintStone = paintStone;
        repaint();
    }

    /** schaltet das Grid an oder aus. */
    public void setShowGrid(boolean showGrid) {
        this.showGrid = showGrid;
        repaint();
    }

}
