package de.duehl.pentris.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Image;
import java.awt.Point;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import de.duehl.pentris.logic.GameType;
import de.duehl.pentris.ui.resources.IconDefinitions;

import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.swing.ui.resources.IconLoader;

import static de.duehl.pentris.ui.PentrisGui.WINDOWS_10;

/**
 * Diese Klasse stellt einen Dialog dar, mit dem ein neues Spiel gestartet oder das Programm
 * beendet werden kann.
 *
 * @version 1.02     2019-07-09
 * @author Christian Dühl
 */

public class GameStartDialog extends ModalDialogBase {

    /** Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private JFrame parentFrame;

    /** Ergebnis des Dialogs. */
    private GameType gameType;

    /**
     * Konstruktor.
     *
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param parentFrame
     *            Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Bild des Programms.
     */
    public GameStartDialog(Colorizer colorizer, JFrame parentFrame, Image programImage) {
        super(parentFrame.getLocation(), programImage, "", colorizer);
        this.parentFrame = parentFrame;
        gameType = null;

        setKeyBindings();

        fillDialog();
    }

    /** Erstellt Tastenkombinationen. */
    private void setKeyBindings() {
        setKeyBindingEscape(() -> chooseToQuit());
        setKeyBindingEnter(() -> choosePentris());
    }

    /** Erstellt den Dialog. */
    @Override
    protected void populateDialog() {
        setUndecorated(true);

        add(createButtonPart(), BorderLayout.CENTER);

        setDialogLocation();
    }

    private void setDialogLocation() {
        if (parentFrame != null) {
            Point location = parentFrame.getLocation();
            int x = (int) location.getX();
            int y = (int) location.getY();
            if (WINDOWS_10) {
                x += 12;
            }
            setLocation(x + 240, y + 90);
        }
        else {
            setLocation(350, 300);
        }
    }

    /** Erzeugt den Bereich mit der Überschrift und den Buttons. */
    private Component createButtonPart() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new BorderLayout());

        panel.add(createInnerTitle(), BorderLayout.NORTH);
        panel.add(createButtonPanel(), BorderLayout.CENTER);

        return panel;
    }

    private JLabel createInnerTitle() {
        JLabel innerTitle = new JLabel("Möchten Sie ein Spiel starten oder Pentris beenden?");
        innerTitle.setBorder(new EmptyBorder(5, 10, 5, 10));
        setColors(innerTitle);
        return innerTitle;
    }

    private Component createButtonPanel() {
        JPanel subPanel = new JPanel();
        subPanel.setLayout(new VerticalLayout(10, VerticalLayout.CENTER));
        setColors(subPanel);

        subPanel.add(createPentrisButton());
        subPanel.add(createTetrisButton());
        subPanel.add(createStartButton());

        return subPanel;
    }

    private Component createPentrisButton() {
        JButton pentris = createStartDialogButton();
        pentris.setIcon(loadIcon(IconDefinitions.PENTRIS_BUTTON));
        pentris.addActionListener(e -> choosePentris());
        return pentris;
    }

    private Component createTetrisButton() {
        JButton tetris = createStartDialogButton();
        tetris.setIcon(loadIcon(IconDefinitions.TETRIS_BUTTON));
        tetris.addActionListener(e -> chooseTetris());
        return tetris;
    }

    private Component createStartButton() {
        JButton quit = createStartDialogButton();
        quit.setText("Beenden");
        quit.setIcon(loadIcon(IconDefinitions.PROGRAMM_IMAGE));
        quit.addActionListener(e -> chooseToQuit());
        return quit;
    }

    private Icon loadIcon(String iconDefinition) {
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        return iconLoader.loadIcon(iconDefinition);
    }

    private JButton createStartDialogButton() {
        JButton button = new JButton();
        button.setFocusable(false);
        button.setContentAreaFilled(false);
        button.setBorder(BorderFactory.createEtchedBorder());
        button.setBorderPainted(false);
        setColors(button);
        return button;
    }

    /** Wählt Spieltyp Pentris als Dialogergebnis. */
    private void choosePentris() {
        gameType = GameType.PENTRIS;
        closeDialog();
    }

    /** Wählt Spieltyp Tetris als Dialogergebnis. */
    private void chooseTetris() {
        gameType = GameType.TETRIS;
        closeDialog();
    }

    /** Wählt als Dialogergebnis, das Spiel zu verlassen. */
    private void chooseToQuit() {
        gameType = null;
        closeDialog();
    }

    /**
     * Ermittelt die gewählte Spielart.
     *
     * @return Ausgewählte Spielart: GameType.PENTRIS, GameType.TETRIS oder
     *         null, wenn kein (neues) Spiel stattfinden soll.
     */
    public GameType getGameType() {
        return gameType;
    }

}
