package de.duehl.pentris.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;

import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import de.duehl.basics.text.NumberString;
import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.swing.ui.resources.IconLoader;
import de.duehl.pentris.logic.GameType;
import de.duehl.pentris.ui.resources.IconDefinitions;

import static de.duehl.pentris.ui.PentrisGui.WINDOWS_10;

/**
 * Diese Klasse stellt einen Dialog dar, mit dem dem Spieler zu seiner erreichten Punktezahl und
 * dem erreichten Platz gratuliert wird. Außerdem kann er hier seinen Namen eingeben.
 *
 * @version 1.02     2019-07-09
 * @author Christian Dühl
 */

public class GratulationDialog extends ModalDialogBase {

    /** Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private final JFrame parentFrame;

    /** Erreichter Platz. */
    private final int place;

    /** Erreichte Punkte. */
    private final int points;

    /** Art des Spiels (Pentris oder Tetris). */
    private final GameType gameType;

    /** Name des Spielers. */
    private String playerName;

    /** Textfeld für die Eingabe des Benutzernamens. */
    private final JTextField playerNameTextField;

    /**
     * Konstruktor.
     *
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param parentFrame
     *            Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Bild des Programms.
     * @param place
     *            Erreichter Platz.
     * @param points
     *            Erreichte Punkte.
     * @param gameType
     *            Art des Spiels (Pentris oder Tetris).
     * @param playerName
     *            Name des Spielers (falls vorher schon bekannt).
     */
    public GratulationDialog(Colorizer colorizer, JFrame parentFrame, Image programImage,
            int place, int points, GameType gameType, String playerName) {
        super(parentFrame.getLocation(), programImage, "", colorizer);
        this.parentFrame = parentFrame;
        this.place = place;
        this.points = points;
        this.gameType = gameType;

        if ("default".equals(playerName)) {
            this.playerName = "";
        }
        else {
            this.playerName = playerName;
        }

        playerNameTextField = new JTextField();

        setKeyBindings();

        fillDialog();
    }

    /** Erstellt Tastenkombinationen. */
    private void setKeyBindings() {
        setKeyBindingEnter(() -> closeDialog());
    }

    /** Erstellt den Dialog. */
    @Override
    protected void populateDialog() {
        setUndecorated(true);

        add(createDialogPanel(), BorderLayout.CENTER);

        setDeilogLocation();
    }

    private void setDeilogLocation() {
        if (parentFrame != null) {
            Point location = parentFrame.getLocation();
            int x = (int) location.getX();
            int y = (int) location.getY();
            if (WINDOWS_10) {
                x += 45;
            }
            setLocation(x + 240, y + 90);
        }
        else {
            setLocation(350, 300);
        }
    }

    /** Erzeugt den eigentlichen Inhalt des Dialogs. */
    private Component createDialogPanel() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new VerticalLayout(10, VerticalLayout.CENTER));

        panel.add(createGameTypePicture());
        panel.add(createGratulationLabel());
        panel.add(createNameField());

        return panel;
    }

    /** Erzeugt passend zum Spieltyp ein Bild. */
    private Component createGameTypePicture() {
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();

        Icon icon;
        if (gameType == GameType.PENTRIS) {
            icon = iconLoader.loadIcon(IconDefinitions.PENTRIS_BUTTON);
        }
        else {
            icon = iconLoader.loadIcon(IconDefinitions.TETRIS_BUTTON);
        }

        JLabel imageLabel = new JLabel(icon);
        setColors(imageLabel);
        imageLabel.setBorder(new EmptyBorder(20, 20, 20, 20));
        return imageLabel;
    }

    /** Erzeugt den Bereich mit der Gratulation. */
    private Component createGratulationLabel() {
        String text = "<html><head></head><body><center>"
                + "Herzlichen Glückwunsch, "
                + "<br />"
                + " Sie haben mit "
                + "<br />"
                + NumberString.taupu(points)
                + "<br />"
                + " Punkten den Platz "
                + "<br />"
                + place
                + "<br />"
                + " erreicht!"
                + "<br />"
                + generateCommentByPlace()
                + "</center></body></html>";
        JLabel label = new JLabel(text);
        setColors(label);
        return label;
    }

    /** Erzeugt einen Kommentar passend zum Platzierung. */
    private String generateCommentByPlace() {
        if (place == 1) {
            return "Hervorragend!";
        }
        else if (place > 1 && place < 4) {
            return "Sehr gut!";
        }
        else if (place > 3 && place < 7) {
            return "Gut gemacht!";
        }
        else if (place > 6 && place < 11) {
            return "Lobenswert!";
        }
        else if (place > 10 && place < 20) {
            return "Ausbaufähig";
        }
        else {
            return "Üben, üben, üben...";
        }
    }

    /** Erzeugt das Panel mit dem Spielernamen: */
    private Component createNameField() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new VerticalLayout(0, VerticalLayout.CENTER));

        panel.add(createPlayerNamLabel());
        panel.add(createPlayerNameTextField());

        return panel;
    }

    private Component createPlayerNamLabel() {
        JLabel label = new JLabel("Ihr Name:");
        setColors(label);
        return label;
    }

    private Component createPlayerNameTextField() {
        playerNameTextField.setText(playerName);
        setColors(playerNameTextField);
        playerNameTextField.requestFocus();
        playerNameTextField.setPreferredSize(new Dimension(100, 20));
        playerNameTextField.setCaretColor(Color.RED);
        playerNameTextField.setBackground(Color.BLACK);
        return playerNameTextField;
    }

    /** Getter für den Spielernamen. */
    public String getPlayerName() {
        playerName = playerNameTextField.getText().trim();
        return playerName;
    }

}
