package de.duehl.pentris.ui.gameloop;

import java.awt.EventQueue;
import java.awt.Image;
import java.lang.reflect.InvocationTargetException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import de.duehl.swing.ui.colors.Colorizer;

import de.duehl.pentris.logic.GameType;
import de.duehl.pentris.logic.highscore.HighScoreElement;
import de.duehl.pentris.logic.stones.Stone;
import de.duehl.pentris.ui.NextStonePanel;
import de.duehl.pentris.ui.PentrisPanel;
import de.duehl.pentris.ui.dialogs.GratulationDialog;

/**
 * Diese Klasse enthält die Oberfläche zur Game-Loop. Änderungen an der
 * Oberfläche müssen im EDT (Event-Dispatch-Thread von Swing) erfolgen,
 * die Game-Loop läuft in einem eigenen Thread (Game-Loop-Thread genannt).       <br /><br />
 *
 * @version 1.01     2013-09-06
 * @author Christian Dühl
 */

public class GameLoopGui {

    /** Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private JFrame parentFrame;

    /** Bild des Programms. */
    private Image programImage;

    /** Panel, in dem das Spielfeld angezeigt wird. */
    private PentrisPanel pentrisPanel;

    /** Panel, in dem der nächste Stein angezeigt wird. */
    private NextStonePanel nextStonePanel;

    /** Label, in dem der Spielstand angezeigt wird. */
    private JLabel pointLabel;

    /** Panel, in dem die Highscoreliste angezeigt wird. */
    private JPanel highScorePanel;

    /** Farbverwaltung für die Gui. */
    private Colorizer colorizer;

    /** Name des Spielers. */
    private String playerName;

    /**
     * Konstruktor.
     *
     * @param parentFrame
     *            Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Bild des Programms.
     * @param pentrisPanel
     *            Panel, in dem das Spielfeld angezeigt wird.
     * @param nextStonePanel
     *            Panel, in dem der nächste Stein angezeigt wird.
     * @param pointLabel
     *            Label, in dem der Spielstand angezeigt wird.
     * @param highScorePanel
     *            Panel, in dem die Highscoreliste angezeigt wird.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     */
    public GameLoopGui(JFrame parentFrame, Image programImage,
            PentrisPanel pentrisPanel, NextStonePanel nextStonePanel,
            JLabel pointLabel, JPanel highScorePanel, Colorizer colorizer) {
        this.parentFrame = parentFrame;
        this.programImage = programImage;
        this.pentrisPanel = pentrisPanel;
        this.nextStonePanel = nextStonePanel;
        this.pointLabel = pointLabel;
        this.highScorePanel = highScorePanel;
        this.colorizer = colorizer;
        playerName = "default";
    }

    /**
     * Aktualisiert die Anzeige des aktuellen Spielsteins.                       <br /><br />
     *
     * Der Aufruf erfolgt aus der Game-Loop, daher wird hier mit invokeLater
     * gearbeitet, damit die Aktualisierung durch den EDT erfolgt.
     */
    public void showActualStone(final Stone actualStone) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                pentrisPanel.setStone(actualStone);
            }
        });
    }

    /**
     * Aktualisiert die Anzeige des nächsten Spielsteins.                        <br /><br />
     *
     * Der Aufruf erfolgt aus der Game-Loop, daher wird hier mit invokeLater
     * gearbeitet, damit die Aktualisierung durch den EDT erfolgt.
     */
    public void setNextStone(final Stone nextStone) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                nextStonePanel.setNextStone(nextStone);
            }
        });
    }

    /**
     * Schaltet die Pause ein.                                                   <br /><br />
     *
     * Der Aufruf erfolgt aus der Game-Loop, daher wird hier mit invokeLater
     * gearbeitet, damit die Änderung des Spielfeldes durch den EDT angezeigt
     * wird.
     */
    public void pauseOn() {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                pentrisPanel.setPaintStone(false);
            }
        });
    }

    /**
     * Schaltet die Pause aus.                                                   <br /><br />
     *
     * Der Aufruf erfolgt aus der Game-Loop, daher wird hier mit invokeLater
     * gearbeitet, damit die Änderung des Spielfeldes durch den EDT angezeigt
     * wird.
     */
    public void pauseOff() {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                pentrisPanel.setPaintStone(true);
            }
        });
    }

    /**
     * Aktualisiert die Anzeige des Spielfeldes.                                 <br /><br />
     *
     * Der Aufruf erfolgt aus der Game-Loop, daher wird hier mit invokeLater
     * gearbeitet, damit die Änderung des Spielfeldes durch den EDT
     * angezeigt wird.
     */
    public void repaint() {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                pentrisPanel.repaint();
                pentrisPanel.revalidate();
            }
        });
    }

    /**
     * Schaltet das Gitter in Spielfeld und Anzeige des nächsten Steines ein
     * oder aus.                                                                 <br /><br />
     *
     * Der Aufruf erfolgt aus der Game-Loop, daher wird hier mit invokeLater
     * gearbeitet, damit die Änderungen durch den EDT vorgenommen werden.
     */
    public void setShowGrid(final boolean showGrid) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                pentrisPanel.setShowGrid(showGrid);
                nextStonePanel.setShowGrid(showGrid);
            }
        });
    }

    /**
     * Aktualisiert die Anzeige des Punktestandes.                               <br /><br />
     *
     * Der Aufruf erfolgt aus der Game-Loop, daher wird hier mit invokeLater
     * gearbeitet, damit die Änderung des Punktestandes durch den EDT
     * angezeigt wird.
     */
    public void actualizePoints(final int points) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                pointLabel.setText(taupu(points));
            }
        });
    }

    /** Stellt eine Zahl mit Tausenderpunkten dar. */
    private String taupu(int number) {
        DecimalFormat df = new DecimalFormat(",###");
        return df.format(number);
    }

    /**
     * Aktualisiert die Anzeige der Highscoreliste. <br />
     * <br />
     *
     * Der Aufruf erfolgt aus der Game-Loop, daher wird hier mit invokeLater
     * gearbeitet, damit die Änderung der Highscoreliste durch den EDT angezeigt
     * wird.
     *
     * @param elements
     *            Elemente der Highscoreliste.
     */
    public void actualizeHighscore(final List<HighScoreElement> elements) {
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                internalActualizeHighscore(elements);
            }
        });
    }

    /**
     * Aktualisiert die Anzeige der Highscoreliste.                              <br /><br />
     *
     * Der Aufruf erfolgt aus dem EDT.
     *
     * @param elements
     *            Elemente der Highscoreliste.
     */
    private void internalActualizeHighscore(List<HighScoreElement> elements) {
        highScorePanel.removeAll();
        List<String> highscoreTexts = new ArrayList<>();
        String start = "<html>&nbsp;&nbsp;<b><span style=\"font-size:9px\">";
        String mid = "</span></b><br/>"
                + "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
        String end = "</html>";
        int place = 0;
        for (HighScoreElement element : elements) {
            ++place;
            String date = element.getDate().toString();
            String name = element.getName();
            int points = element.getScore();
            String text = start + place + ". " + taupu(points) + " Punkte "
                    + mid + name + " am " + date + end;
            highscoreTexts.add(text);
            if (place > 10) {
                break;
            }
        }
        for (String highscoreText : highscoreTexts) {
            JLabel label = new JLabel(highscoreText);
            colorizer.setColors(label);
            highScorePanel.add(label);
        }
        highScorePanel.repaint();
        highScorePanel.revalidate(); // komisch, hier braucht man das!
    }

    /**
     * Gratuliert dem Benutzer zu einem guten Platz in der HighScore-Liste.      <br /><br />
     *
     * Der Aufruf erfolgt aus der Game-Loop. Da auf das Ergebnis gewartet werden
     * soll, wird hier invokeAndWait statt invokeLater verwendet.
     *
     * @param place
     *            Platz, der erreicht wurde.
     * @param points
     *            Erzielte Punkte.
     * @param gameType
     *            Spielart.
     * @return
     */
    public String gratulation(final int place, final int points,
            final GameType gameType) {
        try {
            EventQueue.invokeAndWait(new Runnable() {
                @Override
                public void run() {
                    internalGratulation(place, points, gameType);
                }
            });
        }
        catch (InvocationTargetException e) {
            e.printStackTrace();
            playerName = "failure";
        }
        catch (InterruptedException e) {
            e.printStackTrace();
            playerName = "failure";
        }

        return playerName;
    }

    /**
     * Gratuliert dem Benutzer zu einem guten Platz in der HighScore-Liste.      <br /><br />
     *
     * Der Aufruf erfolgt aus dem EDT.
     *
     * @param place
     *            Platz, der erreicht wurde.
     * @param points
     *            Erzielte Punkte.
     * @param gameType
     *            Spielart.
     * @return
     */
    protected void internalGratulation(int place, int points,
            GameType gameType) {
        GratulationDialog dialog = new GratulationDialog(colorizer, parentFrame,
                programImage, place, points, gameType, playerName);
        dialog.setVisible(true);
        playerName = dialog.getPlayerName();
    }

}
