package de.duehl.basics.collections;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import de.duehl.basics.text.Text;

/**
 * Diese Klasse enthält Hilfsmethoden rund um Collections.
 *
 * @version 1.01     2019-05-08
 * @author Christian Dühl
 */

public class CollectionsHelper {

    /**
     * Oft hat man als KeySet einer HashMap Integerwerte, die man dann auch gerne sortiert nach
     * ihrer Reihenfolge abarbeiten möchte. Diese Methode liefert die Schlüssel numerisch sortiert
     * zurück.
     *
     * @param keySet
     *            Unsortierte Menge an Integerwerten, wie sie map.keySet() zurückliefert.
     * @return Sortierte Liste mit den Schlüsseln.
     */
    public static List<Integer> getSortedIndices(Set<Integer> keySet) {
        List<Integer> ids = new ArrayList<>();
        ids.addAll(keySet);
        Collections.sort(ids);
        return ids;
    }

    /**
     * Oft hat man als KeySet einer HashMap Zeichenketten, die man dann auch gerne sortiert nach
     * ihrer Reihenfolge abarbeiten möchte. Diese Methode liefert die Schlüssel alphanumerisch
     * sortiert zurück.
     *
     * @param keySet
     *            Unsortierte Menge an Zeichenketten, wie sie map.keySet() zurückliefert.
     * @return Sortierte Liste mit den Schlüsseln.
     */
    public static List<String> getSortedStringIndices(Set<String> keySet) {
        List<String> strings = new ArrayList<>();
        strings.addAll(keySet);
        Collections.sort(strings);
        return strings;
    }

    /**
     * Oft hat man als KeySet einer HashMap Integerwerte, die man dann auch gerne sortiert nach
     * ihrer Reihenfolge abarbeiten möchte. Diese Methode liefert die Schlüssel numerisch sortiert
     * zurück.
     *
     * @param map
     *            Verzeichnis mit Integerwerten als Schlüsseln.
     * @return Sortierte Liste mit den Schlüsseln.
     */
    public static List<Integer> getSortedMapIndices(Map<Integer, ?> map) {
        Set<Integer> keySet = map.keySet();
        return getSortedIndices(keySet);
    }

    /**
     * Oft hat man als KeySet einer HashMap Zeichenketten, die man dann auch gerne sortiert nach
     * ihrer Reihenfolge abarbeiten möchte. Diese Methode liefert die Schlüssel alphanumerisch
     * sortiert zurück.
     *
     * @param map
     *            Verzeichnis mit Zeichenketten als Schlüsseln.
     * @return Sortierte Liste mit den Schlüsseln.
     */
    public static List<String> getSortedMapStringIndices(Map<String, ?> map) {
        Set<String> keySet = map.keySet();
        return getSortedStringIndices(keySet);
    }

    /**
     * Erzeugt eine neue Liste mit der umgedrehten Reihenfolge der Elemente.
     *
     * @param <T>
     *            Typ der Listenelemente.
     * @param list
     *            Umzudrehende Liste.
     * @return Neue, umgedrehte Liste.
     */
    public static <T> List<T> reverseList(List<T> list) {
        List<T> reverse = new ArrayList<>();
        reverse.addAll(list);
        Collections.reverse(reverse);
        return reverse;
    }

    /** Gibt eine Liste von Strings schön aus. Mit Indices und in eckigen Klammern eingefasst. */
    public static void printListOfStrings(List<?> list) {
        System.out.print(listListOfStrings(list));
    }

    /** Stellt eine Liste von Strings schön dar. Mit Indices und in eckigen Klammern eingefasst. */
    public static String listListOfStrings(List<?> list) {
        StringBuilder builder = new StringBuilder();

        for (int index = 0; index < list.size(); ++index) {
            String element = String.format("    %2d. [%s]", index, list.get(index).toString());
            builder.append(element).append(Text.LINE_BREAK);
        }

        return builder.toString();
    }

    /** Macht aus einer Liste von Parametern eines Typs eine echte ArrayList. */
    @SafeVarargs
    public static <T> List<T> buildListFrom(T ... texts) {
        // https://stackoverflow.com/questions/12462079/potential-heap-pollution-via-varargs-parameter
        List<T> list = new ArrayList<>();

        for (T text : texts) {
            list.add(text);
        }

        return list;
    }

    /** Macht aus einer Liste von int-Parametern eine echte ArrayList.
     * Es sollte statt dessen buildListFrom verwendet werden.
     */
    @Deprecated
    static List<Integer> buildIntListFrom(int ... values) {
        List<Integer> list = new ArrayList<>();

        for (int value : values) {
            list.add(value);
        }

        return list;
    }

    /**
     * Macht aus einer Liste von String-Parametern eine echte ArrayList aus den kleingeschriebenen
     * Elementen.
     */
    public static List<String> buildLoweredListFrom(String ... texts) {
        List<String> list = new ArrayList<>();

        for (String text : texts) {
            list.add(text.toLowerCase(Locale.GERMAN));
        }

        return list;
    }

    /**
     * Macht aus einer Liste von String-Parametern eine echte ArrayList, die absteigend nach der
     * Länge der Texte sortiert ist.
     */
    public static List<String> buildSortByLengthDescandingListFrom(String ... texts) {
        List<String> list = buildListFrom(texts);

        Collections.sort(list, new Comparator<String>() {
            @Override
            public int compare(String t1, String t2) {
                return t2.length() - t1.length();
            }
        });

        return list;
    }

    /**
     * Macht aus einer Liste von String-Parametern eine echte ArrayList, die aufsteigend nach der
     * Länge der Texte sortiert ist.
     */
    public static List<String> buildSortByLengthListFrom(String ... texts) {
        List<String> list = buildListFrom(texts);

        Collections.sort(list, new Comparator<String>() {
            @Override
            public int compare(String t1, String t2) {
                return t1.length() - t2.length();
            }
        });

        return list;
    }

    /**
     * Macht aus den String-Parametern eine echte ArrayList, die aufsteigend nach den Texten
     * sortiert ist.
     */
    public static List<String> buildSortedListFrom(String ... texts) {
        List<String> list = buildListFrom(texts);

        Collections.sort(list);

        return list;
    }

    /**
     * Macht aus den String-Parametern eine echte ArrayList, die absteigend nach den Texten
     * sortiert ist.
     */
    public static List<String> buildDescandingSortedListFrom(String ... texts) {
        List<String> list = buildSortedListFrom(texts);

        Collections.reverse(list);

        return list;
    }

    /** Entfernt aus einer Liste von Strings alle leeren Strings. */
    public static void removeEmptyStringsFromList(List<String> list) {
        for (int index = list.size() - 1; index >= 0; --index) {
            String element = list.get(index);
            if (element.isEmpty()) {
                list.remove(index);
            }
        }
    }

    /**
     * Entfernt aus einer Liste von Strings alle leeren Strings und solche, die nur aus Whitespace
     * bestehen.
     */
    public static void removeEmptyAndOnlyWhitespaceStringsFromList(List<String> list) {
        for (int index = list.size() - 1; index >= 0; --index) {
            String element = list.get(index);
            if (element.trim().isEmpty()) {
                list.remove(index);
            }
        }
    }

    /**
     * Erzeugt eine dublettenfreie Liste.
     *
     * @param list
     *            Liste mit ggf. mehrfach vorkommenden Werten.
     * @return Dublettenfreie Liste.
     */
    public static List<String> removeMultipleValues(List<String> list) {
        List<String> singles = new ArrayList<>();

        for (String value : list) {
            if (!singles.contains(value)) {
                singles.add(value);
            }
        }

        return singles;
    }

    /** Entfernt das letzte Element der Liste, falls diese nicht leer ist. */
    public static void removeLastListEntry(List<?> list) {
        if (!list.isEmpty()) {
            int lastIndex = list.size() - 1;
            list.remove(lastIndex);
        }
    }

    /** Erzeugt aus einem String-Array eine Liste von Strings mit dem gleichen Inhalt. */
    public static List<String> stringArrayToList(String[] array) {
        return new ArrayList<>(Arrays.asList(array));
    }

    /** Erzeugt aus der übergebenen Liste von Strings ein String-Array mit dem gleichen Inhalt. */
    public static String[] stringListToArray(List<String> list) {
        String[] array = new String[list.size()];
        array = list.toArray(array);
        /* -> http://stackoverflow.com/questions/5374311/convert-arrayliststring-to-string-array */
        return array;
    }

    /** Gibt den Inhalt eines Arrays in einem einzeiligen String aus. */
    public static String stringArrayToString(String[] array) {
        List<String> list = new ArrayList<>(Arrays.asList(array));
        return Text.join(", ", list);
    }

    /** Entfernt leere Strings zu Beginn einer Liste von Strings. */
    public static void removeEmptyStringsAtFrontOfList(List<String> list) {
        int indexOfFirstNotEmptyElement = determineIndexOfFirstNotEmptyElement(list);
        for (int index = list.size() - 1; index >= 0; --index) {
            if (index < indexOfFirstNotEmptyElement) {
                list.remove(index);
            }
        }
    }

    /**
     * Ermittelt den Index des ersten nicht leeren Elements der Liste. Sind alle Elemente leer, so
     * wird die Anzahl der Elemente der Liste zurückgegeben. Ist das erste Element vorhanden und
     * nicht leer, so wird -1 zurückgegeben.
     */
    static int determineIndexOfFirstNotEmptyElement(List<String> list) {
        for (int index = 0; index < list.size(); ++index) {
            String element = list.get(index);
            if (!element.isEmpty()) {
                return index;
            }
        }
        return list.size();
    }

    /** Entfernt leere Strings am Ende einer Liste von Strings. */
    public static void removeEmptyStringsAtEndOfList(List<String> list) {
        int indexOfLastNotEmptyElement = determineIndexOfLastNotEmptyElement(list);
        for (int index = list.size() - 1; index >= 0; --index) {
            if (index > indexOfLastNotEmptyElement) {
                list.remove(index);
            }
        }
    }

    /**
     * Ermittelt den Index des letzten nicht leeren Elements der Liste. Sind alle Elemente leer, so
     * wird -1 zurückgegeben. Ist das letzte Element vorhanden und nicht leer, so wird die Anzahl
     * der Elemente der Liste zurückgegeben.
     */
    static int determineIndexOfLastNotEmptyElement(List<String> list) {
        for (int index = list.size() - 1; index >= 0; --index) {
            String element = list.get(index);
            if (!element.isEmpty()) {
                return index;
            }
        }
        return -1;
    }

    /**
     * Sortiert eine Liste von Strings absteigend nach der Länge ihrer Elemente. Bei gleichlangen
     * Elementen wird alphabetisch sortiert.
     */
    public static void sortStringListByLengthDescanding(List<String> list) {
        Collections.sort(list, createSortStringListByLengthDescandingComparator());
    }

    private static Comparator<String> createSortStringListByLengthDescandingComparator() {
        return new Comparator<String>() {
            @Override
            public int compare(String o1, String o2) {
                int length1 = o1.length();
                int length2 = o2.length();

                int cmp = length2 - length1;
                if (cmp != 0) {
                    return cmp;
                }
                else {
                    return o1.compareTo(o2);
                }
            }
        };
    }

    /** Gibt alle Elemente der Liste in Zeilen untereinander aus. */
    public static void printElements(List<?> list) {
        for (Object element : list) {
            System.out.println(element);
        }
    }

    /** Stellt fest, ob die übergebene Menge disjunkt / unikat ist. */
    public static boolean isDisjunct(List<?> list) {
        for (int i = 0; i < list.size() - 1; ++i) {
            for (int j = i + 1; j < list.size(); ++j) {
                if (list.get(i).equals(list.get(j))) {
                    return false;
                }
            }
        }

        return true;
    }

    /** Erzeugt aus einer Liste mit Strings eine unikate Liste mit disjunkten Inhalten. */
    public static List<String> createDisjunctLowerCaseList(List<String> list) {
        return createDisjunctList(toLowerCase(list));
        // Selbst bei unikaten Listen können durch toLowerCase welche gleich werden.
    }

    /** Erzeugt aus einer Liste mit Strings eine unikate Liste mit disjunkten Inhalten. */
    public static <Element> List<Element> createDisjunctList(List<Element> list) {
        List<Element> disjunctList = new ArrayList<>();

        for (Element value : list) {
            if (!disjunctList.contains(value)) {
                disjunctList.add(value);
            }
        }

        return disjunctList;
    }

    /** Wandelt alle Elemente der Liste in Kleinbuchstaben. */
    public static List<String> toLowerCase(List<String> list) {
        List<String> loweredList = new ArrayList<>();
        for (String text : list) {
            loweredList.add(text.toLowerCase(Locale.GERMAN));
        }
        return loweredList;
    }

    /**
     * Erzeugt eine neue Liste, in der die Elemente der übergebenen Liste mit den angegebenen
     * vorher und hinterher einzufügenden Texten umgeben sind.
     *
     * @param list
     *            Liste, deren Elemente dekoriert werden sollen.
     * @param before
     *            Text der jedem Element der Liste vorzustellen ist.
     * @param after
     *            Text der an jedes Element der Liste anzuhängen ist.
     * @return Liste mit den dekorierten Elementen.
     */
    public static List<String> surroundElements(List<String> list, String before,
            String after) {
        List<String> surroundedList = new ArrayList<>();
        for (String element : list) {
            String surroundedElement = before + element + after;
            surroundedList.add(surroundedElement);
        }

        return surroundedList;
    }

    /**
     * Behält von der übergebenen Liste nur so viele Zeilen vom Anfang, wie angegeben.
     *
     * @param list
     *            Liste
     * @param numberOfLinesToKeep
     *            Anzahl zu behaltender Zeilen.
     * @return Liste mit den Anfangszeilen.
     */
    public static List<String> keepOnlyFirstLines(List<String> list, int numberOfLinesToKeep) {
        if (list.size() <= numberOfLinesToKeep) {
            return list;
        }
        else {
            List<String> firstLines = new ArrayList<>();
            for (int index = 0; index < numberOfLinesToKeep; ++index) {
                firstLines.add(list.get(index));
            }
            return firstLines;
        }
    }

    /** Erzeugt aus einem String und einen StringArray ein neues StringArray. */
    public static String[] appendInfrontOfArray(String firstValue, String[] otherValues) {
        String[] newArray = new String[otherValues.length + 1];
        newArray[0] = firstValue;
        for (int index = 0; index < otherValues.length; ++index) {
            newArray[index + 1] = otherValues[index];
        }
        return newArray;
    }

    /** Erzeugt aus einem String und einen StringArray ein neues StringArray. */
    public static String[] appendAtEndOfArray(String[] otherValues, String lastValue) {
        String[] newArray = new String[otherValues.length + 1];
        for (int index = 0; index < otherValues.length; ++index) {
            newArray[index] = otherValues[index];
        }
        newArray[otherValues.length] = lastValue;
        return newArray;
    }

    /**
     * Erzeugt einen Ausschnitt der Liste, beginnend mit der Zeile nach der ersten Zeile, welche
     * start enthält und endend vor der nächsten Zeile, welche end enthält.
     *
     * @param lines
     *            Liste von Strings.
     * @param start
     *            Begriff vor der ersten Zeile des Ausschnitts.
     * @param end
     *            Begriff nach dem Ausschnitt.
     * @return Teil der Liste.
     */
    public static List<String> getSection(List<String> lines, String start, String end) {
        List<String> section = new ArrayList<>();
        boolean startFound = false;

        for (String line : lines) {
            if (startFound) {
                if (line.contains(end)) {
                    return section;
                }
                else {
                    section.add(line);
                }
            }
            else {
                if (line.contains(start)) {
                    startFound = true;
                }
            }
        }

        if (!startFound) {
            throw new RuntimeException("Der Startbegriff '" + start
                    + "' wurde nicht in der Liste gefunden.");
        }
        else {
            throw new RuntimeException("Der Endbegriff '" + end
                    + "' wurde nach dem Startbegriff nicht in der Liste gefunden.");
        }
    }

    /**
     * Entfernt den ersten Ausschnitt der Liste, beginnend mit der Zeile nach der ersten Zeile,
     * welche start enthält und endend vor der nächsten Zeile, welche end enthält.
     *
     * @param lines
     *            Liste von Strings.
     * @param start
     *            Begriff vor der ersten Zeile des Ausschnitts.
     * @param end
     *            Begriff nach dem Ausschnitt.
     * @return Liste ohne den Teil.
     */
    public static List<String> removeSectionExcludingStartAndEnd(List<String> lines, String start,
            String end) {
        List<String> listWithoutSection = new ArrayList<>();
        boolean startFound = false;
        boolean endFound = false;

        for (String line : lines) {
            if (startFound && !endFound) {
                if (line.contains(end)) {
                    endFound = true;
                    listWithoutSection.add(line);
                }
            }
            else if (endFound) {
                listWithoutSection.add(line);
            }
            else {
                if (line.contains(start)) {
                    startFound = true;
                }
                listWithoutSection.add(line);
            }
        }

        if (!startFound) {
            throw new RuntimeException("Der Startbegriff '" + start
                    + "' wurde nicht in der Liste gefunden.");
        }
        else if (!endFound) {
            throw new RuntimeException("Der Endbegriff '" + end
                    + "' wurde nach dem Startbegriff nicht in der Liste gefunden.");
        }
        else {
            return listWithoutSection;
        }
    }

    /**
     * Entfernt den ersten Ausschnitt der Liste, beginnend mit der ersten Zeile, welche start
     * enthält und endend mit der Zeile, welche end enthält.
     *
     * @param lines
     *            Liste von Strings.
     * @param start
     *            Begriff in der ersten Zeile des Ausschnitts.
     * @param end
     *            Begriff in der letzten Zeile des Ausschnitts.
     * @return Liste ohne den Teil.
     */
    public static List<String> removeSectionIncludingStartAndEnd(List<String> lines, String start,
            String end) {
        List<String> listWithoutSection = new ArrayList<>();
        boolean startFound = false;
        boolean endFound = false;

        for (String line : lines) {
            if (startFound && !endFound) {
                if (line.contains(end)) {
                    endFound = true;
                }
            }
            else {
                if (!endFound && line.contains(start)) {
                    startFound = true;
                }
                else {
                    listWithoutSection.add(line);
                }
            }
        }

        if (!startFound) {
            throw new RuntimeException("Der Startbegriff '" + start
                    + "' wurde nicht in der Liste gefunden.");
        }
        else if (!endFound) {
            throw new RuntimeException("Der Endbegriff '" + end
                    + "' wurde nach dem Startbegriff nicht in der Liste gefunden.");
        }
        else {
            return listWithoutSection;
        }
    }

    /** Stellt fest, ob der Suchbegriff in einer der Zeilen vorkommt. */
    public static boolean anyLineContains(List<String> lines, String search) {
        for (String line : lines) {
            if (line.contains(search)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Sucht alle Vorkommen eines Suchbegriffs in einer Liste von Strings.
     *
     * @param searchText
     *            Text, nachdem gesucht wird.
     * @param list
     *            Liste von Strings, die durchsucht wird.
     * @return Liste mit allen Indices des Suchtextes in der Liste, ggf. leer.
     */
    public static <T> List<Integer> findAllIndces(T searchText, List<T> list) {
        List<Integer> indices = new ArrayList<>();

        for (int index = 0; index < list.size(); ++index) {
            T listElement = list.get(index);
            if (listElement.equals(searchText)) {
                indices.add(index);
            }
        }

        return indices;
    }

    /** Konkatiniert beide Listen zu einer. */
    public static List<String> joinStringLists(List<String> list1, List<String> list2) {
        List<String> totalList = new ArrayList<>();
        totalList.addAll(list1);
        totalList.addAll(list2);
        return totalList;
    }

    /** Fertigt eine flache Kopie der Liste an. */
    public static <Element> List<Element> copyList(List<Element> list) {
        List<Element> copyiedList = new ArrayList<>();

        for (Element element : list) {
            copyiedList.add(element);
        }

        return copyiedList;
    }

    /** Trimmt alle Elemente der Liste. */
    public static List<String> trimList(List<String> texts) {
        List<String> trimmed = new ArrayList<>();

        for (String text : texts) {
            trimmed.add(text.trim());
        }

        return trimmed;
    }

    /** Macht in allen Elementen der Liste aus mehrere Leerzeichen ein Leerzeichen. */
    public static List<String> cleanUselessWhitspace(List<String> texts) {
        List<String> cleaned = new ArrayList<>();

        for (String text : texts) {
            cleaned.add(Text.cleanUselessWhitspace(text));

        }

        return cleaned;
    }

}

