package de.duehl.basics.io.textfile;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;

/**
 * Diese Klasse liest eine Liste mit den Texten im Format des DictionaryFiles aus den Zeilen einer
 * Textdatei ein.
 *
 * Das Format sieht wie folgt aus:
 *
 * Englisches Wort
 *     Deutsches Wort 1
 *     Deutsches Wort 2
 *     Deutsches Wort 3
 *     Deutsches Wort 4
 *
 * Kommt in einer Datenzeile weiter hinten das Kommentarzeichen vor, so wird ab dort abgeschnitten
 * und getrimmt.
 *
 * @version 1.01     2019-07-05
 * @author Christian Dühl
 */

public class DictionaryFromTextFileReader {

    private final StringsFromTextFileReader reader;

    public DictionaryFromTextFileReader(StringsFromTextFileReader reader) {
        this.reader = reader;
        reader.doNotTrimLines();
    }

    /** Legt fest, dass der Reader ruhig sein soll. */
    public void beQuiet() {
        reader.beQuiet();
    }

    /**
     * Gibt an, dass die eingelesenen Zeilen in Kleinbuchstaben gewandelt werden.
     *
     * Hierbei wird Locale.GERMAN eingesetzt.
     */
    public void changeToLowerCase() {
        reader.changeToLowerCase();
    }

    /**
     * Liest die Listen aus der Datei ein. das erste Wort in der Liste ist die Vokabel und die
     * anderen die Übersetzungen.
     */
    public List<List<String>> read() {
        List<List<String>> lists = new ArrayList<>();
        String vocable = "";
        List<String> translations = null;

        for (String line : reader.read()) {
            if (!isLineIndentedComment(line)) {
                boolean indented = isIndented(line);
                if (indented) {
                    if (vocable.isEmpty()) {
                        throw new IllegalArgumentException("Zeile '" + line
                                + "' startet mit vier Buchstaben, aber es fehlt die Vokabel davor!.");
                    }
                    translations.add(line.trim());
                }
                else {
                    if (!vocable.isEmpty()) {
                        lists.add(translations);
                    }
                    translations = new ArrayList<>();
                    vocable = line.trim();
                    translations.add(vocable);
                }
            }
        }
        if (!vocable.isEmpty()) {
            lists.add(translations);
        }

        return lists;
    }

    private boolean isLineIndentedComment(String line) {
        if (line.isBlank()) {
            return true;
        }
        else {
            /*
             * Da im StringsFromTextFileReader bereits hinten stehende Kommentar entfernt
             * wurden, aber im Falle von
             *     reader.doNotTrimLines();
             * wie es hier der Fall ist, die Leerzeichen davor stehen bleiben, wird hier überprüft,
             * ob es der Rest einer solchen Zeile ist.
             *
             * Daher kommt dies folgende gar nicht mehr mit
             *     return true;
             * zum Tragen, wird aber der Vollständigkeit halber stehen gelassen.
             */
            String trimmed = line.trim();
            return trimmed.startsWith(reader.getCommentChar());
        }
    }

    private boolean isIndented(String line) {
        if (line.startsWith("    ")) {
            String test = line.substring(4);
            if (test.startsWith(" ")) {
                throw new IllegalArgumentException(
                        "Zeile '" + line + "' startet mit mehr als vier Buchstaben.");
            }
            return true;
        }
        else {
            if (line.startsWith(" ")) {
                throw new IllegalArgumentException(
                        "Zeile '" + line + "' startet mit weniger als vier Buchstaben.");
            }
            return false;
        }
    }

}
