package de.duehl.basics.text;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.duehl.basics.debug.Assure;
import de.duehl.basics.io.Charset;
import de.duehl.basics.text.data.FoundSearch;

/**
 * Diese Klasse bietet Methoden rund um die Behandlung von Texten.
 *
 * @version 1.01     2019-05-09
 * @author Christian Dühl
 */

public class Text {

    public static final String BIG_CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÜÁÉÍÓÚÀÈÌÒÙÂÊÎÔÛ";
    public static final String SMALL_CHARS = "abcdefghijklmnopqrstuvwxyzäöüßáéíóúàèìòùâêîôû";
    public static final String MIXED_CHARS = BIG_CHARS + SMALL_CHARS;
    public static final String BIG_CHARS_REGEX = "A-ZÄÖÜÁÉÍÓÚÀÈÌÒÙÂÊÎÔÛ";
    public static final String SMALL_CHARS_REGEX = "a-zäöüßáéíóúàèìòùâêîôû";
    public static final String MIXED_CHARS_REGEX = BIG_CHARS + SMALL_CHARS;
    public static final String BIG = "[" + BIG_CHARS_REGEX + "]";
    public static final String SMALL = "[" + SMALL_CHARS_REGEX + "]";
    public static final String MIXED = "[" + MIXED_CHARS_REGEX + "]";
    public static final String NO_SMALL = "[^" + SMALL_CHARS_REGEX + "]";
    public static final String DOT = "\\.";
    public static final String BULLET = "·";

    /** Systemunabhängiger Zeilenumbruch. */
    public static final String LINE_BREAK = System.getProperty("line.separator");

    public static final int NOT_FOUND_INDEX = -1;

    static final String LINE_BREAK_PLACE_HOLDER =  "###LINE_BREAK###";
    //static final String LINE_BREAK_PLACE_HOLDER =  "\\n";

    private static final Pattern CAMEL_CASE_PATTERN = Pattern.compile(
            "(?:" + BIG + SMALL + "+){2,}");
    private static final Pattern CAMEL_CASE_PATTERN_WITH_DOTS = Pattern.compile(
            "(?:" + BIG + SMALL + "+\\.)+(?:" + BIG + SMALL + "+\\.?)");

    private Text() {}

    /**
     * Versieht einen langen Text mit Zeilenumbrüchen, so dass die Zeilen - so überhaupt
     * Leerzeichen vorhanden sind, höchstens lineLength Zeichen lang sind. Hierbei wird dieser
     * zuvor an schon vorhandenen Umbrüchen umgebrochen und nur die Teile dazwischen behandelt.
     *
     * @param text
     *            Gegebener Text
     * @param lineLength
     *            Zeilenlänge
     * @return umgebrochener Text
     */
    public static String addLineBreaks(String text, int lineLength) {
        TextBreaker breaker = new TextBreaker(lineLength);
        return breaker.addLineBreaks(text);
    }

    /**
     * Versieht einen langen Text mit Zeilenumbrüchen, so dass die Zeilen - so überhaupt
     * Leerzeichen vorhanden sind, höchstens lineLength Zeichen lang sind. Hierbei wird dieser
     * zuvor an schon vorhandenen Umbrüchen umgebrochen und nur die Teile dazwischen behandelt.
     *
     * @param text
     *            Gegebener Text
     * @param lineLength
     *            Zeilenlänge
     * @param numberOfFrontSpaces
     *            Anzahl einzufügender Leerzeichen am Anfang der Zeile.
     * @return umgebrochener Text
     */
    public static String addLineBreaks(String text, int lineLength, int numberOfFrontSpaces) {
        TextBreaker breaker = new TextBreaker(lineLength, numberOfFrontSpaces);
        return breaker.addLineBreaks(text);
    }

    /**
     * Entfernt Zeilenumbrüche aus einem String und ersetzt sie durch Leerzeichen.
     *
     * @param text
     *            Zu bearbeitender Text.
     * @return Text ohne Zeilenumbrüche.
     */
    public static String removeLineBreaks(String text) {
        String editedText = text;

        editedText = editedText.replace(LINE_BREAK, " ");
        editedText = editedText.replace("\r\n", " ");
        editedText = editedText.replace("\n", " ");
        editedText = editedText.replace("\r", " ");

        return editedText;
    }

    /**
     * Überführt alle Zeilenumbrüche in den zum System passenden Zeilenumbruch.
     *
     * @param text
     *            Zu bearbeitender Text.
     * @return Text mit den korrekten Zeilenumbrüche.
     */
    public static String repairLineBreaks(String text) {
        String editedText = text;

        editedText = editedText.replace(LINE_BREAK, LINE_BREAK_PLACE_HOLDER);
        editedText = editedText.replace("\r\n", LINE_BREAK_PLACE_HOLDER);
        editedText = editedText.replace("\n", LINE_BREAK_PLACE_HOLDER);
        editedText = editedText.replace("\r", LINE_BREAK_PLACE_HOLDER);
        editedText = editedText.replace(LINE_BREAK_PLACE_HOLDER, LINE_BREAK);

        return editedText;
    }

    /**
     * Zählt das Vorkommen eines Zeilenumbruchs in einem Text.
     *
     * @param text
     *            Zu bearbeitender Text.
     * @return Anzahl des Vorkommens von Zeilenumbrüchen.
     */
    public static int countLineBreaks(String text) {
        return countPartInString(text, LINE_BREAK);
    }

    /**
     * Ersetzt im String alle Backslashes (\) durch Slashes (/).
     *
     * @param file
     *            Eingabe
     * @return String mit umgesetzten Slashes
     */
    public static String allBackslashesToSlashes(String file) {
        return file.replace("\\", "/");
    }

    /**
     * Ersetzt im String alle Slashes (/) durch Backslashes (\).
     *
     * @param file
     *            Eingabe
     * @return String mit umgesetzten Slashes
     */
    public static String allSlashesToBackslashes(String file) {
        return file.replace("/", "\\");
    }

    /**
     * Entfernt am Ende ein oder mehrere Slashes oder Backslashes, falls vorhanden.
     *
     * @param dir
     *            Übergebenes Verzeichnis (etwa "c:/temp/").
     * @return Bereinigtes Verzeichnis (etwa "c:/temp").
     */
    public static String removeTrailingSlash(String dir) {
        String editedDir = dir;

        while (editedDir.endsWith("/") || editedDir.endsWith("\\")) {
            editedDir = editedDir.substring(0, editedDir.length() - 1);
        }

        return editedDir;
    }

    /** Baut Perls " "x8 nach. */
    public static String multipleString(String text, int times) {
        if (times <= 0) {
            return "";
        }

        StringBuilder multi = new StringBuilder(text.length() * times);
        for (int i = 0; i < times; ++i) {
            multi.append(text);
        }
        return multi.toString();
    }

    /** Baut Perls " "x8 für einzelne Zeichen nach. */
    public static String multipleString(char character, int length) {
        String text = Character.toString(character);
        return multipleString(text, length);
    }

    /**
     * Zählt das Vorkommen eines Teilstrings in einem String.
     *
     * @param source
     *            Quelle, die durchsucht wird,
     * @param searchPart
     *            Teil, der gesucht wird.
     * @return Anzahl des Vorkommens.
     */
    public static int countPartInString(String source, String searchPart) {
        int count = 0;
        int startIndex = 0;
        boolean found = true;
        while (found) {
            int index = source.indexOf(searchPart, startIndex);
            if (index == NOT_FOUND_INDEX) {
                found = false;
            }
            else {
                ++count;
                startIndex = index + searchPart.length();
            }
        }
        return count;
    }

    /**
     * Gibt die übergebene Nachricht nach StDOUT aus.
     *
     * @param message
     *            Auszugebende Nachricht.
     */
    public static void say(String message) {
        System.out.println(message);
    }

    /** Gibt einen Trennstrich auf STDOUT aus. */
    public static void bar() {
        say(multipleString("-", 80));
    }

    /** Gibt einen Trennstrich mit Zeilenumbruch zurück. */
    public static String barLine() {
        return multipleString("-", 80) + "\n";
    }

    /**
     * Ersetzt Whitespace aller Art durch Leerzeichen, trimmt und ersetzt mehrfache Leerzeichen
     * durch eines.
     *
     * @param text
     *            Zu bearbeitender Text.
     * @return Bearbeiteter Text.
     */
    public static String stripWhitespace(String text) {
        String editedText = text;

        editedText = editedText.replaceAll("\\s+", " ");
        editedText = editedText.replaceAll("\\r", " "); // sollte in \s sein, aber wer weiß
        editedText = editedText.replaceAll("\\n", " "); // sollte in \s sein, aber wer weiß
        editedText = editedText.replaceAll("  +", " ");
        editedText = editedText.trim();

        return editedText;
    }

    /**
     * Fügt Leerzeichen zwischen die Zeichen (und hinter das letzte Zeichen)
     * des übergebenen Textes ein.                                               <br><br>
     *
     * Aus "abc" wird "a b c ".
     *
     * @param text
     *            Zu bearbeitender Text.
     * @return Auseinandergezogener Text.
     */
    public static String stretch(String text) {
        StringBuilder builder = new StringBuilder();
        for (char letter : text.toCharArray()) {
            builder.append(letter)
                .append(" ");
        }
        return builder.toString();
    }

    /**
     * Vergleicht zwei Texte in den ersten Stellen.
     *
     * @param text1
     *            Erster Text.
     * @param text2
     *            Zweiter Text.
     * @param numberOfChars
     *            Anzahl der Stellen, bis zu der verglichen wird. Endet einer
     *            der Texte vorher, müssen beide an der gleichen Stelle enden
     *            und gleich sein, damit wahr zurückgegeben wird.
     * @return Wahrheitswert.
     */
    public static boolean firstCharsAreEqual(String text1, String text2, int numberOfChars) {
        if (numberOfChars < 0) {
            throw new IllegalArgumentException("Die Anzahl der zu ersten "
                    + "vergleichenden Stellen darf nicht negativ sein.");
        }

        int n = text1.length();
        int m = text2.length();

        if ((n < numberOfChars || m < numberOfChars) && n != m) {
            return false;
        }

        char[] chars1 = text1.toCharArray();
        char[] chars2 = text2.toCharArray();

        int diffLength = Math.min(numberOfChars, n);
        for (int i=0; i<diffLength; ++i) {
            if (chars1[i] != chars2[i]) {
                return false;
            }
        }

        return true;
    }

    /**
     * Sucht alle Vorkommen eines Suchbegriffs in einem Text.                           <br><br><i>
     *
     * Achtung, es wird nicht verhindert, dass sich die Fundstellen überlappen.
     * Sucht man in 'foofoofoo' nach 'foofoo', wird man zwei Treffer erhalten,
     * an Position 0 und Position 3.                                                          </i>
     *
     * @param searchText
     *            Text, nachdem gesucht wird.
     * @param text
     *            Text, der durchsucht wird.
     * @return Liste mit allen Null-basierten Fundstellen des Suchtextes im
     *         Text, ggf. leer.
     */
    public static List<Integer> findAllPositions(String searchText, String text) {
        List<Integer> list = new ArrayList<>();

        int index = text.indexOf(searchText);
        while (index > NOT_FOUND_INDEX) {
            list.add(index);
            index = text.indexOf(searchText, index + 1);
        }

        return list;
    }

    /**
     * Sucht alle Vorkommen eines Suchbegriffs als ganzes Wort in einem Text.
     *
     * @param searchText
     *            Text, nachdem gesucht wird.
     * @param text
     *            Text, der durchsucht wird.
     * @return Liste mit allen Null-basierten Fundstellen des Suchtextes im Text, ggf. leer.
     */
    public static List<Integer> findAllPositionsOfCompleteWords(String searchText, String text) {
        List<Integer> pos = findAllPositions(searchText, text);
        int length = searchText.length();

        List<Integer> realWordPositions = new ArrayList<>();
        for (int index = 0; index < pos.size(); ++index) {
            int indexInPos = pos.get(index);
            if (isStartAndEndOfWord(indexInPos, indexInPos + length, text)) {
                realWordPositions.add(indexInPos);
            }
        }

        return realWordPositions;
    }

    /**
     * join() wie in Perl.
     *
     * @param fill
     *            Einzusetzendes Zwischenstück.
     * @param list
     *            Liste mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch das einzusetzende Füllstück verbunden
     *         zusammengesetzt.
     */
    public static String join(String fill, List<String> list) {
        if (list.isEmpty()) {
            return "";
        }
        else if (list.size() == 1) {
            return list.get(0);
        }
        else {
            StringBuilder build = new StringBuilder();
            build.append(list.get(0));
            for (int i=1; i<list.size(); ++i) {
                build.append(fill);
                build.append(list.get(i));
            }
            return build.toString();
        }
    }

    /**
     * join() wie in Perl.
     *
     * @param fill
     *            Einzusetzendes Zwischenstück.
     * @param array
     *            Array mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch das einzusetzende Füllstück verbunden
     *         zusammengesetzt.
     */
    public static String join(String fill, String ... array) {
        List<String> list = Arrays.asList(array);
        return join(fill, list);
    }

    /**
     * join() wie in Perl mit Tabulator.
     *
     * @param list
     *            Liste mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch Tabulatoren verbunden zusammengesetzt.
     */
    public static String joinWithTabulator(List<String> list) {
        return join("\t", list);
    }

    /**
     * join() wie in Perl mit Tabulator.
     *
     * @param array
     *            Array mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch Tabulatoren verbunden zusammengesetzt.
     */
    public static String joinWithTabulator(String ... array) {
        List<String> list = Arrays.asList(array);
        return joinWithTabulator(list);
    }

    /**
     * join() wie in Perl mit Zeilenumbrüchen.
     *
     * @param list
     *            Liste mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch Zeilenumbrüche verbunden zusammengesetzt.
     */
    public static String joinWithLineBreak(List<String> list) {
        return join(LINE_BREAK, list);
    }

    /**
     * join() wie in Perl mit Pipezeichen.
     *
     * @param list
     *            Liste mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch Pipezeichen verbunden zusammengesetzt.
     */
    public static String joinWithPipe(List<String> list) {
        return join("|", list);
    }

    /**
     * join() wie in Perl mit Leerzeichen.
     *
     * @param list
     *            Liste mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch Leerzeichen verbunden zusammengesetzt.
     */
    public static String joinWithBlank(List<String> list) {
        return join(" ", list);
    }

    /**
     * join() wie in Perl mit Semikolon.
     *
     * @param list
     *            Liste mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch Semikolons verbunden zusammengesetzt.
     */
    public static String joinWithSemicolon(List<String> list) {
        return join(";", list);
    }

    /**
     * join() wie in Perl mit Doppelpunkt.
     *
     * @param list
     *            Liste mit den zusammen zu setzenden Elementen.
     * @return Alle Elemente der Liste werden durch Doppelpunkte verbunden zusammengesetzt.
     */
    public static String joinWithColon(List<String> list) {
        return join(":", list);
    }

    /**
     * Splittet die Eingabe am gegebenen Ausdruck.
     *
     * @param input
     *            Eingabe.
     * @param regex
     *            Regulärer Ausdruck an dem gesplittet wird.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitBy(String input, String regex) {
        List<String> list = new ArrayList<>();
        if (!input.isEmpty()) {
            list.addAll(Arrays.asList(input.split(regex, -1)));
        }
        /*
         * Einschub wegen der absonderlichen Eigenheiten von input.split, vielleicht sollte ich
         * das Ding mal selbst besser schreiben...:
         */
        if (regex.isEmpty() && !list.isEmpty() && list.get(list.size() - 1).isEmpty()) {
            list.remove(list.size() - 1);
        }
        return list;
    }

    /**
     * Splittet die Eingabe an allem Whitespace.
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByWhitespace(String input) {
        return splitBy(input, "[ \\t]+");
    }

    /**
     * Splittet die Eingabe an aufeinander folgenden Leerzeichen.
     *
     * @param input
     *            Eingabe.
     * @param numberOfSpacesForColumnDistinction
     *            Anzahl an Leerzeichen, die mindestens nebeneinander vorhanden sein müssen, um als
     *            Spaltentrenner zu gelten.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByMultipleSpaces(String input,
            int numberOfSpacesForColumnDistinction) {
        if (numberOfSpacesForColumnDistinction < 1) {
            throw new IllegalArgumentException("Die Anzahl der Leerzeichen muss mindestens 1 sien!");
        }
        return splitBy(input, "[ ]{" + numberOfSpacesForColumnDistinction + ",}") ;
    }

    /**
     * Splittet die Eingabe an allen Pipesymbolen (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByPipe(String input) {
        return splitBy(input, " *\\| *");
    }

    /**
     * Splittet die Eingabe an allen Pipesymbolen (ohne Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByPipeNotConsumingWhitespace(String input) {
        return splitBy(input, "\\|");
    }

    /**
     * Splittet die Eingabe an allen in einzelne Anführungszeichen gehüllte Leerzeichen oder
     * Hashsymbole (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByQuotedBlankOrHash(String input) {
        return splitBy(input, " *'(?:#| )' *");
    }

    /**
     * Splittet die Eingabe an allen Pluszeichen (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByAddition(String input) {
        return splitBy(input, " *\\+ *");
    }

    /**
     * Splittet die Eingabe an allen Minuszeichen (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitBySubtraction(String input) {
        return splitBy(input, " *- *");
    }

    /**
     * Splittet die Eingabe an allen Kommata (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByKomma(String input) {
        return splitBy(input, " *, *");
    }

    /**
     * Splittet die Eingabe an allen Tabulatoren (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByTabulator(String input) {
        return splitBy(input, " *\t *");
    }

    /**
     * Splittet die Eingabe an allen Tabulatoren (ohne Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByTabulatorNotConsumingWhitespace(String input) {
        return splitBy(input, "\t");
    }

    /**
     * Splittet die Eingabe an allen Semikolons (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitBySemicolon(String input) {
        return splitBy(input, " *; *");
    }

    /**
     * Splittet die Eingabe an allen Semikolons (ohne Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitBySemicolonNotConsumingWhitespace(String input) {
        return splitBy(input, ";");
    }

    /**
     * Splittet die Eingabe an allen Doppelpunkten (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByColon(String input) {
        return splitBy(input, " *: *");
    }

    /**
     * Splittet die Eingabe an allen Doppelpunkten (ohne Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByColonNotConsumingWhitespace(String input) {
        return splitBy(input, ":");
    }

    /**
     * Splittet die Eingabe an allen Spezialtrennern #;# (mit ggf. Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitBySpecial(String input) {
        return splitBy(input, " *#;# *");
    }

    /**
     * Splittet die Eingabe an allen Spezialtrennern #;# (ohne Whitespace drumherum).
     *
     * @param input
     *            Eingabe.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitBySpecialNotConsumingWhitespace(String input) {
        return splitBy(input, "#;#");
    }

    /**
     * Splittet die Eingabe an Zeilenumbrüchen.
     *
     * @param input
     *            Eingabe.
     * @param regex
     *            Regulärer Ausdruck an dem gesplittet wird.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByLineBreaks(String input) {
        return splitBy(input, "(\\Q" + LINE_BREAK + "\\E|\r?\n|\r)");
    }

    /**
     * Splittet die Eingabe an Punkten.
     *
     * @param input
     *            Eingabe.
     * @param regex
     *            Regulärer Ausdruck an dem gesplittet wird.
     * @return Liste aller Teile der Eingabe.
     */
    public static List<String> splitByDot(String input) {
        return splitBy(input, " *\\. *");
    }

    /**
     * Entfernt den angegebenen Bereich aus dem String und fügt ein Leerzeichen an seiner Stelle
     * ein.
     *
     * @param text
     *            Zu bearbeitender Text.
     * @param position
     *            Position an der ausgeschnitten wird.
     * @param length
     *            Länge des ausgeschnittenen Textes.
     * @return Text ohne den angegebenen Bereich.
     */
    public static String removeFromText(String text, int position, int length) {
        if (length < 0) {
            throw new IllegalArgumentException("length " + length + " ist zu klein, muss "
                    + "mindestens 0 sein");
        }
        return replaceInText(text, " ", position, position + length);
    }

    /**
     * Tauscht ein Stück des Eingabetextes aus.
     *
     * @param text
     *            Eingabetext.
     * @param replace
     *            Neu einzufügendes Textstück.
     * @param start
     *            Erste Position des zu entfernenden Teiles.
     * @param end
     *            Erste Position nach dem Ende des zu entfernenden Teiles.
     * @return Ersetzter String.
     */
    public static String replaceInText(String text, String replace, int start, int end) {
        checkStartAndEnd(text, start, end);

        String front = text.substring(0, start);
        String rear = text.substring(end);
        String newText = front + replace + rear;

        return newText;
    }

    private static void checkStartAndEnd(String text, int start, int end) {
        if (start < 0) {
            throw new IllegalArgumentException("start " + start + " ist zu klein, muss mindestens "
                    + "0 sein");
        }
        if (start > 0 && start > text.length()) {
            throw new IllegalArgumentException("start " + start + " ist zu groß, darf höchstens "
                    + text.length() + " sein");
        }
        if (end > text.length()) {
            throw new IllegalArgumentException("end " + end + " ist zu groß, darf höchstens "
                    + text.length() + " sein");
        }
        if (start > end) {
            throw new IllegalArgumentException("start (" + start + ") darf höchstens so groß wie "
                    + "end (" + end + ") sein");
        }
    }

    /**
     * Tauscht ein Stück des Eingabetextes durch eine Zeichenfolge gleicher Länge aus.
     *
     * @param text
     *            Eingabetext.
     * @param character
     *            Zeichen, das so oft eingefügt werden soll, wie das Fundstück lang ist.
     * @param start
     *            Erste Position des zu entfernenden Teiles.
     * @param end
     *            Erste Position nach dem Ende des zu entfernenden Teiles.
     * @return Ersetzter String.
     */
    public static String replaceInTextWithEqualLengthCharacterSequence(String text,
            char character, int start, int end) {
        checkStartAndEnd(text, start, end);
        int length = end - start;
        String replace = multipleString(character, length);
        return replaceInText(text, replace, start, end);
    }

    /**
     * Ersetzt alle zu ersetzenden Worte im Text durch eine Zeichenfolge gleicher Länge.
     *
     * @param text
     *            Zu ersetzenden Text.
     * @param replaceWords
     *            Worte die ersetzt werden sollen.
     * @return Text mit durch Zeichenfolgen gleicher Länge ersetzen Suchbegriffen.
     */
    public static String replaceInTextWithEqualLengthCharacterSequence(String text,
            List<String> replaceWords) {
        String replacedText = text;
        for (String replace : replaceWords) {
            String oldReplacedText = "";
            while (!replacedText.equals(oldReplacedText)) {
                oldReplacedText = replacedText;
                replacedText = replaceInTextWithEqualLengthCharacterSequenceOnce(replacedText,
                        replace);
            }
        }

        return replacedText;
    }

    /**
     * Ersetzt das erste Vorkommen des Suchbegriffs
     *
     * @param text
     *            Zu ersetzender Text.
     * @param replace
     *            Suchbegriffs.
     * @return Text mit einem durch eine Zeichenfolge gleicher Länge ersetzen Suchbegriff.
     */
    static String replaceInTextWithEqualLengthCharacterSequenceOnce(String text, String replace) {
        String replacedText = text;
        int index = text.indexOf(replace);
        if (index > -1) {
            replacedText = replaceInTextWithEqualLengthCharacterSequence(text, '#', index, index
                    + replace.length());
        }
        return replacedText;
    }


    /** Ersetzt Zeilenumbrüche durch einen Platzhalter. */
    public static String hideLineBreaks(String input) {
        return replaceLineBreaks(input, LINE_BREAK_PLACE_HOLDER);
    }

    /** Ersetzt Zeilenumbrüche durch ein literales \n. */
    public static String hideLineBreaksWithLiteralBackslashN(String input) {
        return replaceLineBreaks(input, "\\n");
    }

    /** Ersetzt die Platzhalter wieder durch echte Zeilenumbrüche. */
    public static String showLineBreaksAfterHiding(String input) {
        String text = input;

        text = text.replace(LINE_BREAK_PLACE_HOLDER, LINE_BREAK);

        return text;
    }

    /** Ersetzt Zeilenumbrüche durch ein Leerzeichen. */
    public static String lineBreaksToSpace(String input) {
        return replaceLineBreaks(input, " ");
    }

    /** Ersetzt Zeilenumbrüche durch \n. */
    public static String lineBreaksToBackslashN(String input) {
        return replaceLineBreaks(input, "\n");
    }

    /**
     * Ersetzt Zeilenumbrüche durch ein anderes Zeichen.
     *
     * @param input
     *            Eingabetext, der bearbeitet werden soll.
     * @param replacement
     *            Ersetzungstext für den Zeilenumbruch.
     * @return Text mit ersetzten Zeilenumbrüchen.
     */
    private static String replaceLineBreaks(String input, String replacement) {
        String text = input;

        text = text.replace(LINE_BREAK, replacement);
        text = text.replace("\r\n", replacement);
        text = text.replace("\n", replacement);
        text = text.replace("\r", replacement);

        return text;
    }

    /**
     * Ermittelt den Text aus dem Feld, trimmt ihn und wandelt Tabulatoren und Umbrüche in einfache
     * Leerzeichen um.
     */
    public static String removeLineBreaksAndTabulators(String input) {
        String output = input;
        output = lineBreaksToSpace(output);
        output = output.replace("\t", " ");
        output = output.replaceAll(" +", " ");
        output = output.trim();
        return output;
    }

    /**
     * Entfernt alle Kommentarzeilen (die mit "#" beginnen) und liefert alle anderen zurück.
     *
     * @param lines
     *            Liste mit den zu bearbeitenden Zeilen.
     * @return Liste mit allen Zeilen, die keine Kommentare sind.
     */
    public static List<String> stripCommentsFromLines(List<String> lines) {
        List<String> stripped = new ArrayList<>();

        for (String line : lines) {
            if (!line.startsWith("#")) {
                stripped.add(line);
            }
        }

        return stripped;
    }

    /**
     * Entfernt die letzten x Zeichen aus einem String. Ist der String kürzer als die angegebene
     * Anzahl an Zeichen, wird der leere String zurückgegeben.
     *
     * @param input
     *            Zu bearbeitender String
     * @param numberOfCharacters
     *            Anzahl der am Ende zu entfernenden Zeichen.
     * @return String ohne die letzten x Zeichen.
     */
    public static String removeLastCharacters(String input, int numberOfCharacters) {
        if (input.length() <= numberOfCharacters) {
            return "";
        }
        else {
            return input.substring(0,  input.length() - numberOfCharacters);
        }
    }

    /**
     * Füllt einen String hinten bis zur angegebenen Gesamtlänge mit Leerzeichen auf.
     *
     * fillWithSpaces("abc", 5) ergibt den String "abc  ".
     *
     * Ist der Text mindestens so lang wie die angegebene Länge, wird er unverändert zurückgegeben.
     *
     * @param text
     *            Aufzufüllender Text.
     * @param length
     *            Länge, bis zu der der Text mit Leerzeichen aufzufüllen ist.
     * @return Aufgefüllter Text.
     */
    public static String fillWithSpaces(String text, int length) {
        if (length <= text.length()) {
            return text;
        }
        else {
            String fill = multipleString(" ", length - text.length());
            return text + fill;
        }
    }

    /**
     * Füllt einen String vorn bis zur angegebenen Gesamtlänge mit Leerzeichen auf.
     *
     * fillWithSpacesAtFront("abc", 5) ergibt den String "  abc".
     *
     * Ist der Text mindestens so lang wie die angegebene Länge, wird er unverändert zurückgegeben.
     *
     * @param text
     *            Aufzufüllender Text.
     * @param length
     *            Länge, bis zu der der Text mit Leerzeichen aufzufüllen ist.
     * @return Aufgefüllter Text.
     */
    public static String fillWithSpacesAtFront(String text, int length) {
        if (length <= text.length()) {
            return text;
        }
        else {
            String fill = multipleString(" ", length - text.length());
            return fill + text;
        }
    }

    /**
     * Füllt einen String hinten bis zur angegebenen Gesamtlänge mit Minuszeichen auf.
     *
     * fillWithSpaces("abc", 5) ergibt den String "abc--".
     *
     * Ist der Text mindestens so lang wie die angegebene Länge, wird er unverändert zurückgegeben.
     *
     * @param text
     *            Aufzufüllender Text.
     * @param length
     *            Länge, bis zu der der Text mit Minuszeichen aufzufüllen ist.
     * @return Aufgefüllter Text.
     */
    public static String fillWithLines(String text, int length) {
        if (length <= text.length()) {
            return text;
        }
        else {
            String fill = multipleString("-", length - text.length());
            return text + fill;
        }
    }

    /**
     * Bereinigt die Liste mit den Begriffen derart, dass keine Teilbegriffe eines Begriffs in
     * einem anderen mehr
     * auftauchen.
     *
     * @param terms
     *            Menge mit den zu bereinigenden Begriffen.
     * @return Bereinigte Liste mit den Begriffen.
     */
    public static List<String> cleanCollectionOfStringsToNotContainingEachOther(
            Collection<String> terms) {
        List<String> cleanedTerms = new ArrayList<>();
        cleanedTerms.addAll(terms);

        boolean removing = true;
        while (removing) {
            removing = cleanCollectionOfStringsToNotContainingEachOtherOneTime(cleanedTerms);
        }

        return cleanedTerms;
    }

    /**
     * Entfernt einen Term, der in einem anderen enthalten ist.
     *
     * @param cleanedTerms
     *            Liste mit den zu bereinigenden Termen.
     * @return Wahrheitswert: true genau dann, wenn ein Term entfernt wurde.
     */
    private static boolean cleanCollectionOfStringsToNotContainingEachOtherOneTime(
            List<String> cleanedTerms) {
        for (int i=0; i<cleanedTerms.size(); ++i) {
            String partI = cleanedTerms.get(i);
            for (int j=i+1; j<cleanedTerms.size(); ++j) {
                String partJ = cleanedTerms.get(j);
                if (partI.contains(partJ)) {
                    cleanedTerms.remove(j);
                    return true;
                }
                if (partJ.contains(partI)) {
                    cleanedTerms.remove(i);
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Gibt den Teil des Strings hinter dem letzten Vorkommen des Trenners zurück. Kommt der
     * Trenner nicht vor, wird die ganze Eingabe zurückgegeben.
     *
     * @param input
     *            Eingabetext.
     * @param divider
     *            Trenner
     * @return Teil des Strings hinter dem letzten Trenner.
     */
    public static String getLastPartAfterDivider(String input, String divider) {
        int lastDividerIndex = input.lastIndexOf(divider);
        if (NOT_FOUND_INDEX == lastDividerIndex) {
            return input;
        }
        else {
            return input.substring(lastDividerIndex + 1);
        }
    }

    /**
     * Entfernt Leerzeichen zu Beginn und am Ende und macht aus allen mehrfachen Leerzeichen ein
     * einzelnes.
     *
     * @param input
     *            Zu bearbeitender Text.
     * @return Bearbeiteter Text.
     */
    public static String trimAndCompactWhitespace(String input) {
        String output = input;

        output = output.replaceAll("^ +", "");
        output = output.replaceAll(" +$", "");
        output = output.replaceAll("[ ]{2,}", " ");

        return output;
    }

    /** Setzt die Zeilen zu einem String zusammen. */
    public static String listOfLinesToString(List<String> lines) {
        StringBuilder builder = new StringBuilder();

        for (String line : lines) {
            builder.append(line);
            builder.append(LINE_BREAK);
        }

        return builder.toString();
    }

    /**
     * Nimmt einen durch System.getProperty("line.separator") zusammengesetzten Text in einem
     * String wieder auseinander, so dass einzelne Zeilen entstehen.
     */
    public static List<String> stringToListOfLines(String text) {
        List<String> lines = splitBy(text, Pattern.quote(LINE_BREAK));

        if (!lines.isEmpty() && lines.get(lines.size() - 1).isEmpty()) {
            lines.remove(lines.size() - 1);
        }

        return lines;
    }

    /**
     * Sucht die erste Zeile aus der übergebenen Liste von Zeilen heraus, die den gesuchten String
     * enthält, und gibt deren Index (0-basiert) zurück.
     *
     * @param lines
     *            Liste von zu durchsuchenden Zeilen.
     * @param search
     *            Suchbegriff.
     * @return Index der ersten Zeile, die den Suchbegriff enthält oder NOT_FOUND_INDEX (-1), falls
     *         der Suchbegriff in keiner Zeile enthalten ist.
     */
    public static int searchFirstLineIndexContainig(List<String> lines, String search) {
        for (int index = 0; index < lines.size(); ++index) {
            String line = lines.get(index);
            if (line.contains(search)) {
                return index;
            }
        }

        return NOT_FOUND_INDEX;
    }

    /**
     * Ermittelt ein Textstück dass zwischen den genannten Start- und Endzeichen liegt
     * (ausschließlich dieser).
     *
     * @param text
     *            Zu durchsuchender Text.
     * @param from
     *            Start-Text.
     * @param to
     *            End-Text.
     * @throws RuntimeException
     *             falls die Randstücke nicht oder nicht eindeutig sind im Text.
     */
    public static String findInText(String text, String from, String to) {
        int indexFrom = text.indexOf(from);
        if (-1 == indexFrom) {
            throw new RuntimeException("Das Anfangsstück '" + from + "' wurde nicht im Text "
                    + "gefunden.\n\ttext = " + text);
        }
        int indexNextFrom = text.indexOf(from, indexFrom + from.length());
        if (-1 != indexNextFrom) {
            throw new RuntimeException("Das Anfangsstück '" + from + "' ist nicht eindeutig im "
                    + "Text.\n\ttext = " + text);
        }
        int indexTo = text.indexOf(to);
        if (-1 == indexTo) {
            throw new RuntimeException("Das Endstück '" + to + "' wurde nicht im Text "
                    + "gefunden.\n\ttext = " + text);
        }
        int indexNextTo = text.indexOf(to, indexTo + to.length());
        if (-1 != indexNextTo) {
            throw new RuntimeException("Das Endstück '" + to + "' ist nicht eindeutig im "
                    + "Text.\n\ttext = " + text);
        }
        int start = indexFrom + from.length();
        int end = indexTo;
        return text.substring(start, end);
    }

    /**
     * Stellt einen double Wert auf zwei Nachkommastellen gerundet dar.
     *
     * @param value
     *            Wert der double-Variablen.
     * @return String mit dem Wert der Variablen auf zwei Nachkommastellen gerundet.
     */
    public static String doubleTwoDecimalPlaces(double value) {
        return String.format("%.2f", value);
    }

    /**
     * Diese Methode bildet aus einem Text einen einheitlich klein geschriebenen Text, in dem die
     * Umlaute und das ß durch ausgeschrieben Versionen ('ae', 'oe', 'ue' und 'ss') ersetzt sind.
     *
     * @param text
     *            Zu verändernden Text.
     * @return Vergleichstext
     */
    public static String buildCompareText(String text) {
        String compare = text.toLowerCase(Locale.GERMAN);

        compare = compare.replace("ä", "ae");
        compare = compare.replace("ö", "oe");
        compare = compare.replace("ü", "ue");
        compare = compare.replace("ß", "ss");

        return compare;
    }

    /**
     * Ermittelt, ob das übergebene Zeichen ein Zeichen innerhalb eines Wortes ist.
     *
     * @param c
     *            Zeichen
     * @return Wahrheitswert
     */
    public static boolean isWordChar(char c) {
        if (isWhitespace(c) || isPunctuation(c) || isBrace(c))
            return false;
        else
            return true;
    }

    /**
     * Ermittelt, ob das übergebene Zeichen Whitespace ist.
     *
     * @param c
     *            Zeichen
     * @return Wahrheitswert
     */
    public static boolean isWhitespace(char c) {
        return c == ' ' || c == '\n' || c == '\t';
    }

    /**
     * Ermittelt, ob das übergebene Zeichen eine Klammer ist.
     *
     * @param c
     *            Zeichen
     * @return Wahrheitswert
     */
    public static boolean isBrace(char c) {
        return c == '(' || c == ')' || c == '[' || c == ']' || c == '{' || c == '}'
                || c == '<' || c == '>';
    }

    /**
     * Ermittelt, ob das übergebene Zeichen ein Satzzeichen ist.
     *
     * @param c
     *            Zeichen
     * @return Wahrheitswert
     */
    public static boolean isPunctuation(char c) {
        return c == '.' || c == ',' || c == ':' || c == ';' || c == '!' || c == '?' || c == '-'
                || c == '_';
    }

    /**
     * Ermittelt, ob das übergebene Zeichen ein Großbuchstabe ist.
     *
     * @param c
     *            Zeichen
     * @return Wahrheitswert
     */
    public static boolean isCapitalLetter(char c) {
        return c >= 'A' && c <= 'Z' || 'Ä' == c || 'Ö' == c || 'Ü' == c;
    }

    /**
     * Ermittelt, ob dar übergebene text mit einem Großbuchstabe beginnt.
     *
     * @param text
     *            Zu prüfender Text.
     * @return Wahrheitswert
     */
    public static boolean firstLetterIsCapitalLetter(String text) {
        if (text.isEmpty()) {
            return false;
        }
        char firstLetter = text.charAt(0);
        return isCapitalLetter(firstLetter);
    }

    /** Liefert den überlappenden Teil der beiden Texte zurück. */
    public static String findIntersection(String text1, String text2) {
        String intersection = "";

        int maxLength = Math.min(text1.length(), text2.length());

        for (int length = 0; length <= maxLength; ++length) {
            String testIntersection = text2.substring(0,  length);
            if (text1.endsWith(testIntersection)) {
                intersection = testIntersection;
            }
        }

        return intersection;
    }

    /** Prüft, ob der Text einen Großbuchstaben enthält. */
    public static boolean containsUpperLetter(String input) {
        for (char c : input.toCharArray()) {
            if (isCapitalLetter(c)) {
                return true;
            }
        }

        return false;
    }

    /** Prüft, ob ein Begriff aus der Liste im Text vorkommt. */
    public static boolean contains(String input, List<String> searches) {
        for (String searchWord : searches) {
            if (input.contains(searchWord)) {
                return true;
            }
        }
        return false;
    }

    /** Erzeugt aus einer Eingabe einen String, dessen erstes Zeichen groß und der Rest klein ist. */
    public static String toFirstUpperCase(String input) {
        if (input.isEmpty()) {
            return input;
        }
        if (input.length() == 1) {
            return input.toUpperCase(Locale.GERMAN);
        }
        else  {
            String firstChar = input.substring(0, 1);
            String rest = input.substring(1);
            return firstChar.toUpperCase(Locale.GERMAN) + rest.toLowerCase(Locale.GERMAN);
        }
    }

    /** Entfernt aus einer Eingabe die Leerzeichen und macht CamelCase daraus. */
    public static String toCamelCase(String input) {
        if (input.isEmpty()) {
            return "";
        }

        List<String> parts = new ArrayList<>();
        for (String part : splitBy(input, "[_ \\t]+")) {
            if (part.isEmpty()) {
                throw new RuntimeException("Es wurde an Whitespace getrennt, da darf kein "
                        + "leerer String auftauchen!");
            }
            String firstChar = part.substring(0, 1);
            String rest = part.substring(1);
            part = firstChar.toUpperCase() + rest; // kein lowerCase beim Rest!
            parts.add(part);
        }

        StringBuilder builder = new StringBuilder();
        for (String part : parts) {
            builder.append(part);
        }

        return builder.toString();
    }

    /** Säubert einen String, so dass er als Sas-Variablennamen geeignet ist. */
    public static String cleanForSasVariableName(String input) {
        String output = input;

        output = output.replaceAll("[-!?.,#\"'\\d;:/\\\\]", "_");
        output = output.replace("Ä", "Ae");
        output = output.replace("Ö", "Oe");
        output = output.replace("Ü", "Ue");
        output = output.replace("ä", "ae");
        output = output.replace("ö", "oe");
        output = output.replace("ü", "ue");
        output = output.replace("ß", "ss");
        output = output.trim();
        output = output.replace(" ", "_");
        output = output.replaceAll("__+", "_");
        output = output.replaceAll("^_+", "");
        output = output.replaceAll("_+$", "");

        /* Sas-Dataset-Variablen dürfen nur 32 Zeichen lang sein: */
        if (output.length() > 32) {
            output = output.substring(0, 32);
        }

        return output;
    }

    /**
     * Trimmt den String und ersetzt alle '/' durch '\'.
     *
     * @param input
     *            Eingabe.
     * @return Umgesetzte Ausgabe
     */
    public static String nicePath(String input) {
        String output = input.trim();
        output = allSlashesToBackslashes(output);
        return output;
    }

    /**
     * Trimmt alle String aus der Liste und ersetzt bei allen Strings alle '/' durch '\'.
     *
     * @param list
     *            Liste mit den Eingaben.
     * @return Liste mit den umgesetzten Ausgaben.
     */
    public static List<String> nicePathes(List<String> list) {
        List<String> outputPathes = new ArrayList<>();

        for (String input : list) {
            String output = nicePath(input);
            outputPathes.add(output);
        }

        return outputPathes;
    }

    /**
     * Entfernt aus einem Link, etwa einem zum String gewandelten URL, alle ggf. vorhandenen Teile
     * am Ende des Links, die auf einen bestimmten Abschnitt auf der davor angegebenen Seite
     * verweisen.
     *
     * Aus "hilfe.html#why" wird damit "hilfe.html".
     *
     * @param input
     *            Vollständiger Link, ggf. mit "#" Teil.
     * @return Link ohne "#" Teil.
     */
    public static String skipSublinkPart(String input) {
        int hashPosition = input.indexOf("#");
        if (hashPosition == -1) {
            return input;
        }
        else {
            return input.substring(0, hashPosition);
        }
    }

    /** Überprüft, ob der gegebene String nur aus Groß- und Kleinbuchstaben ohne Umlauten besteht. */
    public static boolean containsOnlyNormalLetters(String input) {
        return input.matches("[A-Za-z]+");
    }

    /** Ersetzt Tabulatoren durch vier Leerzeichen. */
    public static String tabToSpace(String line) {
        return line.replace("\t", "    ");
    }

    /**
     * Extrahiert die in einfache Quotes gehüllten Parameter in einem konstruktoraritgem
     * Klammerausdruck.
     *
     * @param input
     *            Klammerausdruck der Art
     *            MM_openBrWindow('impressum.html','PopUp','scrollbars=yes,width=560,height=565').
     */
    public static List<String> extractQuotetBraceParameters(String input) {
        List<Integer> openingBracePositions = findAllPositions("(", input);
        Assure.hasOneElement(openingBracePositions);
        List<Integer> closingBracePositions = findAllPositions(")", input);
        Assure.hasOneElement(closingBracePositions);
        int openIndex = openingBracePositions.get(0);
        int closeIndex = closingBracePositions.get(0);
        Assure.isAtLeast(closeIndex, openIndex);
        String inner = input.substring(openIndex + 1, closeIndex);
        if (inner.startsWith("'") && inner.endsWith("'")) {
            inner = inner.substring(1, inner.length() - 1);
            List<String> parts = splitBy(inner, "' *, *'");
            return parts;
        }
        else {
            throw new IllegalArgumentException("Der Text in der Klammer muss mit einem einfachen "
                    + "Quote beginnen und aufhören.\n\t"
                    + "input = [" + input + "]"
                    + "inner = [" + inner + "]\n\t");
        }
    }

    /**
     * Entfernt den gegebene Anfang aus dem gegebenen Text, falls dieser wirklich damit anfängt.
     *
     * @param textToChange
     *            Text, dessen Anfang ggf. entfernt werden soll.
     * @param frontToRemove
     *            Anfang, der entfernt werden soll.
     * @return Text, ggf. ohne den Anfang.
     */
    public static String removeTextAtFrontIfStartsWith(String textToChange, String frontToRemove) {
        if (textToChange.startsWith(frontToRemove)) {
            return textToChange.substring(frontToRemove.length());
        }
        else {
            return textToChange;
        }
    }

    /**
     * Entfernt das gegebene Ende aus dem gegebenen Text, falls dieser wirklich damit endet.
     *
     * @param textToChange
     *            Text, dessen Ende ggf. entfernt werden soll.
     * @param rearToRemove
     *            Ende, das entfernt werden soll.
     * @return Text, ggf. ohne das Ende.
     */
    public static String removeTextAtEndIfEndsWith(String textToChange, String rearToRemove) {
        if (textToChange.endsWith(rearToRemove)) {
            return textToChange.substring(0, textToChange.length() - rearToRemove.length());
        }
        else {
            return textToChange;
        }
    }

    /**
     * Entfernt einen Zeilenumbruch am Ende des übergebenen Textes, falls der Text auf einen
     * solchen endet.
     *
     * @param textToChange
     *            Text, dessen Zeilenumbruch am Ende ggf. entfernt werden soll.
     * @return Text, ggf. ohne Zeilenumbruch am Ende.
     */
    public static String removeLineBreakAtEndIfEndsWith(String textToChange) {
        String text = removeTextAtEndIfEndsWith(textToChange, LINE_BREAK);
        if (text.equals(textToChange)) {
            text = removeTextAtEndIfEndsWith(text, "\n");
        }
        return text;
    }

    /**
     * Entfernt den Teil aus dem Text, der vor dem Suchtext steht, sofern der Suchtext im Text
     * vorkommt.
     *
     * @param text
     *            Text in dem etwas vorn gelöscht werden soll.
     * @param searchText
     *            Suchtext, vor diesem soll alles im Text entfernt werden.
     * @return Ggf. veränderten Text.
     */
    public static String removePartBeforeSearchText(String text, String searchText) {
        int index = text.indexOf(searchText);
        if (index > 0) {
            return text.substring(index);
        }
        else {
            return text;
        }
    }

    /** Vereinheitlicht alle Zeilenumbrüche im Text. */
    public static String unifyLineBreaks(String text) {
        String editedText = text;

        editedText = editedText.replace(LINE_BREAK, LINE_BREAK_PLACE_HOLDER);
        editedText = editedText.replace("\r\n", LINE_BREAK_PLACE_HOLDER);
        editedText = editedText.replace("\n", LINE_BREAK_PLACE_HOLDER);
        editedText = editedText.replace("\r", LINE_BREAK_PLACE_HOLDER);

        editedText = editedText.replace(LINE_BREAK_PLACE_HOLDER, LINE_BREAK);

        return editedText;
    }

    /** Entfernt alle Leerzeilen aus dem Text. Auch solche mit Leerzeichen darin. */
    public static String removeEmptyLines(String text) {
        String editedText = unifyLineBreaks(text);

        /* Mitte: */
        editedText = editedText.replaceAll(
                "(?:" + LINE_BREAK + " *)+" + LINE_BREAK,
                LINE_BREAK);

        /* Vorn: */
        editedText = editedText.replaceAll(
                "^(?: *" + LINE_BREAK + ")+",
                "");

        /* Hinten: */
        editedText = editedText.replaceAll(
                "(?:" + LINE_BREAK + " *)+$",
                "");

        return editedText;
    }

    /**
     * Entfernt alle Leerzeilen aus dem Text, die vor der ersten Zeile mit echtem Inhalt und nach
     * der letzten Zeile mit echtem Inhalt kommen. Auch solche mit Leerzeichen darin.
     */
    public static String removeEmptyLinesAtFrontAndEnd(String text) {
        List<String> lines = splitByLineBreaks(text);

        int firstFilledLineIndex = findFirstFilledLineIndex(lines);
        if (firstFilledLineIndex == -1) {
            return "";
        }
        int lastFilledLineIndex = findLastFilledLineIndex(lines);

        List<String> goodLines = new ArrayList<>();
        for (int index = firstFilledLineIndex; index <= lastFilledLineIndex; ++index) {
            goodLines.add(lines.get(index));
        }

        return joinWithLineBreak(goodLines);
    }

    private static int findFirstFilledLineIndex(List<String> lines) {
        for (int index = 0; index < lines.size(); ++index) {
            String line = lines.get(index);
            if (!line.trim().isEmpty()) {
                return index;
            }
        }

        return -1;
    }

    private static int findLastFilledLineIndex(List<String> lines) {
        for (int index = lines.size() - 1; index >= 0; --index) {
            String line = lines.get(index);
            if (!line.trim().isEmpty()) {
                return index;
            }
        }

        return -1;
    }

    /** Entfernt Zeilen aus dem Text, die nicht länger als die angegebene Länge sind. */
    public static String removeShortLines(String text, int maxLegthToRemove) {
        String editedText = unifyLineBreaks(text);

        //String noLineBreakCharacters = "[^" + LINE_BREAK + "]{," + maxLegthToRemove + "}";
        String noLineBreakCharacters = "[^\\r\\n]{1," + maxLegthToRemove + "}";

        /* Mitte: */
        editedText = editedText.replaceAll(
                "(?:" + LINE_BREAK + noLineBreakCharacters + ")+" + LINE_BREAK,
                LINE_BREAK);

        /* Vorn: */
        editedText = editedText.replaceAll(
                "^(?:" + noLineBreakCharacters + LINE_BREAK + ")+",
                "");

        /* Hinten: */
        editedText = editedText.replaceAll(
                "(?:" + LINE_BREAK + noLineBreakCharacters + ")+$",
                "");

        return editedText;
    }

    public static String loremImpsum() {
        return "Lorem ipsum dolor sit amet, consectetur adipisici elit, sed eiusmod tempor "
                + "incidunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis "
                + "nostrud exercitation ullamco laboris nisi ut aliquid ex ea commodi consequat. "
                + "Quis aute iure reprehenderit in voluptate velit esse cillum dolore eu fugiat "
                + "nulla pariatur. Excepteur sint obcaecat cupiditat non proident, sunt in culpa "
                + "qui officia deserunt mollit anim id est laborum." + LINE_BREAK
                + LINE_BREAK
                + "--" + LINE_BREAK
                + LINE_BREAK
                + "Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie "
                + "consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan "
                + "et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis "
                + "dolore te feugait nulla facilisi. Lorem ipsum dolor sit amet, consectetuer "
                + "adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore "
                + "magna aliquam erat volutpat."
                + LINE_BREAK
                + "Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit "
                + "lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure "
                + "dolor in hendrerit in vulputate velit esse molestie consequat, vel illum "
                + "dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio "
                + "dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te "
                + "feugait nulla facilisi."
                + LINE_BREAK
                + "Nam liber tempor cum soluta nobis eleifend option congue nihil imperdiet "
                + "doming id quod mazim placerat facer possim assum. Lorem ipsum dolor sit amet, "
                + "consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut "
                + "laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, "
                + "quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea "
                + "commodo consequat."
                + LINE_BREAK
                + "Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie "
                + "consequat, vel illum dolore eu feugiat nulla facilisis."
                + LINE_BREAK
                + "At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd "
                + "gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum "
                + "dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor "
                + "invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero "
                + "eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no "
                + "sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit "
                + "amet, consetetur sadipscing elitr, At accusam aliquyam diam diam dolore "
                + "dolores duo eirmod eos erat, et nonumy sed tempor et et invidunt justo labore "
                + "Stet clita ea et gubergren, kasd magna no rebum. sanctus sea sed takimata ut "
                + "vero voluptua. est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, "
                + "consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore "
                + "et dolore magna aliquyam erat."
                + LINE_BREAK
                + "Consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore "
                + "et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et "
                + "justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata "
                + "sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur "
                + "sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore "
                + "magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo "
                + "dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est "
                + "Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing "
                + "elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna "
                + "aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores "
                + "et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem "
                + "ipsum dolor sit amet.";
        // Quelle: https://la.wikisource.org/wiki/Lorem_ipsum
    }

    /** Erzeugt eine Liste mit allen Zeichen aus dem String als Strings der Länge 1. */
    public static List<String> eachCharFromString(String text) {
        List<String> characters = new ArrayList<>();

        for (int i = 0; i < text.length(); ++i) {
            String character = text.substring(i, i + 1);
            characters.add(character);
        }

        return characters;
    }

    /** Liest einen UTF-8 kodierten InputStream in einen String ein. */
    public static String utf8InputStreamToString(InputStream inputStream) {
        return inputStreamToString(inputStream, Charset.UTF_8);
    }

    /** Liest einen kodierten InputStream in einen String ein. */
    public static String inputStreamToString(InputStream inputStream, Charset charset) {
        try {
            return tryToInputStreamToString(inputStream, charset);
        }
        catch (Exception exception) {
            throw new RuntimeException(exception);
        }
    }

    private static String tryToInputStreamToString(InputStream inputStream, Charset charset)
            throws IOException {
        final int bufferSize = 1024;
        final char[] buffer = new char[bufferSize];
        final StringBuilder out = new StringBuilder();
        Reader in = new InputStreamReader(inputStream, charset.getCharsetAsString());
        for (; ; ) {
            int rsz = in.read(buffer, 0, buffer.length);
            if (rsz < 0)
                break;
            out.append(buffer, 0, rsz);
        }
        return out.toString();
    }

    /**
     * Sucht die erste Position von den übergebenen möglichen Fundstellen.
     *
     * @param text
     *            Text der durchsucht wird.
     * @param starts
     *            Suchbegriffe die im Text gesucht werden.
     * @return Erster gefundener Suchbegriff.
     */
    public static FoundSearch findFirstPosition(String text, List<String> starts) {
        List<FoundSearch> foundIndices = new ArrayList<>();
        for (String start : starts) {
            int index = text.indexOf(start);
            if (index > -1) {
                foundIndices.add(new FoundSearch(index, start));
            }
        }

        if (foundIndices.isEmpty()) {
            return FoundSearch.notFound();
        }
        else {
            Collections.sort(foundIndices);
            return foundIndices.get(0);
        }
    }

    /**
     * Sucht die erste Position von den übergebenen möglichen Fundstellen ab dem genannten Index.
     *
     * @param text
     *            Text der durchsucht wird.
     * @param starts
     *            Suchbegriffe die im Text gesucht werden.
     * @param fromIndex
     *            Kleinster Index, ab dem gesucht wird.
     * @return Erster gefundener Suchbegriff.
     */
    public static FoundSearch findFirstPosition(String text, List<String> starts, int fromIndex) {
        List<FoundSearch> foundIndices = new ArrayList<>();
        for (String start : starts) {
            int index = text.indexOf(start, fromIndex);
            if (index > -1) {
                foundIndices.add(new FoundSearch(index, start));
            }
        }

        if (foundIndices.isEmpty()) {
            return FoundSearch.notFound();
        }
        else {
            Collections.sort(foundIndices);
            return foundIndices.get(0);
        }
    }

    /** Entfernt alle gefundenen Bereiche mit dem angegebenen Anfang und Ende. */
    public static String removeAllPartsWithStartAndEnd(String html, String start, String end) {
        String changed = html;

        boolean searchForComments = true;

        while (searchForComments) {
            int startIndex = changed.indexOf(start);
            if (startIndex == -1) {
                searchForComments = false;
            }
            else {
                int endIndex = changed.indexOf(end, startIndex + start.length());
                if (endIndex == -1) {
                    changed = changed.substring(0, startIndex);
                }
                else {
                    changed = changed.substring(0, startIndex)
                            + changed.substring(endIndex + end.length());
                }
            }
        }

        return changed;
    }

    /** Entfernt alle gefundenen Bereiche mit dem jeder möglichen Kombination aus Anfang und Ende. */
    public static String removeAllPartsWithStartsAndEnds(String html, List<String> starts,
            List<String> ends) {
        String changed = html;

        boolean searchForLinks = true;

        while (searchForLinks) {
            FoundSearch foundStart = findFirstPosition(changed, starts);
            int startIndex = foundStart.getIndex();
            if (startIndex == -1) {
                searchForLinks = false;
            }
            else {
                String start = foundStart.getSearch();
                FoundSearch foundEnd = findFirstPosition(changed, ends,
                        startIndex + start.length());
                int endIndex = foundEnd.getIndex();
                if (endIndex == -1) {
                    changed = changed.substring(0, startIndex);
                }
                else {
                    String end = foundEnd.getSearch();
                    changed = changed.substring(0, startIndex)
                            + changed.substring(endIndex + end.length());
                }
            }
        }

        return changed;
    }

    /**
     * Prüft, ob abgesehen von der Groß- und Kleinschreibung der übergebene Text mit dem
     * übergebenen Anfang beginnt.
     *
     * @param text
     *            Zu prüfender Text.
     * @param start
     *            Anfang mit dem der Text beginnen soll.
     * @return Wahrheitswert, true genau dann, wenn der Text abgesehen von der Groß- und
     *         Kleinschreibung mit dem gegebenen Anfang beginnt.
     */
    public static boolean startsWithIgnoreCase(String text, String start) {
        String loweredText = text.toLowerCase(Locale.GERMAN);
        String loweredStart = start.toLowerCase(Locale.GERMAN);
        return loweredText.startsWith(loweredStart);
    }

    /** Entfernt Kommentare aus dem Text. */
    public static String removeSlashStarComments(String line) {
        return removeAllPartsWithStartAndEnd(line, "/*", "*/");
    }

    /** Entfernt Leerzeichen aus dem Text. */
    public static String removeSpaces(String input) {
        return input.replace(" ", "");
    }

    /** Entfernt Leerzeichen am Beginn und am Ende und macht aus mehrfachen Leerzeichen eines. */
    public static String cleanUselessWhitspace(String input) {
        String cleaned = input;
        cleaned = cleaned.replaceAll("^ +", "");
        cleaned = cleaned.replaceAll(" +$", "");
        cleaned = cleaned.replaceAll(" +", " ");
        return cleaned;
    }

    /**
     * Prüft, ob die angegebenen Indices im Namen der Beginn und das Ende eines Wortes sind. Nicht
     * geprüft wird, ob ggf. noch Leerzeichen oder andere Wortgrenzen dazwischen liegen.
     */
    public static boolean isStartAndEndOfWord(int startIndex, int endIndex, String name) {
        return isStartOfWord(startIndex, name) && isEndOfWord(endIndex, name);
    }

    /** Prüft, ob der angegebene Index im Namen der Beginn eines Wortes ist. */
    public static boolean isStartOfWord(int index, String name) {
        if (index < 0 || index >= name.length()) {
            return false;
        }
        else if (index == 0) {
            return true;
        }
        else {
            String charBefore = name.substring(index - 1, index);
            return !MIXED_CHARS.contains(charBefore);
        }
    }

    /** Prüft, ob der angegebene Index im Namen nach dem Ende eines Wortes ist. */
    public static boolean isEndOfWord(int index, String name) {
        if (index <= 0 || index > name.length()) {
            return false;
        }
        else if (index == name.length()) {
            return true;
        }
        else {
            String charBehind = name.substring(index, index + 1);
            return !MIXED_CHARS.contains(charBehind);
        }
    }

    /** Versieht Anführungszeichen mit einem Backlash. */
    public static String escapeDoubleQuotes(String input) {
        return input.replace("\"", "\\\"");
    }

    /** Prüft, ob der übergebene Text komplett großgeschrieben ist. */
    public static boolean isUpperCase(String input) {
        return input.matches(NO_SMALL + "*");
    }

    /** Prüft, ob der übergebene Text im CamelCase vorliegt. */
    public static boolean isCamelCase(String input) {
        Matcher matcher = CAMEL_CASE_PATTERN.matcher(input);
        return matcher.matches();
    }

    /**
     * Prüft, ob der übergebene Text im CamelCase mit Punkt am Ende vorliegt.
     *
     * Beispiele: Grund.Wert.Vermögen.
     *            Grund.Wert.Vermögen
     */
    public static boolean isCamelCaseWithDots(String input) {
        Matcher matcher = CAMEL_CASE_PATTERN_WITH_DOTS.matcher(input);
        return matcher.matches();
    }

    /** Gibt den ersten Buchstaben (das erste Zeichen) in Kleinbuchstaben gewandelt zurück. */
    public static String firstLoweredChar(String input) {
        if (input.isEmpty()) {
            return "";
        }

        String firstCharacter = input.substring(0, 1);
        return firstCharacter.toLowerCase(Locale.GERMAN);
    }

    /** Wandelt den ersten Buchstaben (das erste Zeichen) in Kleinbuchstaben um. */
    public static String toFirstLoweredChar(String input) {
        if (input.isEmpty()) {
            return "";
        }
        String firstCharacter = firstLoweredChar(input);
        String rest = input.substring(1);
        return firstCharacter + rest;
    }

    /** Gibt die Liste der Buchstaben des übergebenen Wortes zurück. */
    public static List<String> wordToLetters(String word) {
        List<String> letters = new ArrayList<>();

        for (int index = 0; index < word.length(); ++index) {
            String letter = word.substring(index, index + 1);
            letters.add(letter);
        }

        return letters;
    }

    /** Prüft, ob der übergebene String von doppelten Anführungszeichen umgeben ist. */
    public static boolean isQuoted(String input) {
        return input.startsWith("\"") && input.endsWith("\"");
    }

    /**
     * Entfernt die doppelten Anführungszeichen am Anfang und am Ende.
     *
     * Fehlen diese, wird eine Ausnahme geworfen.
     */
    public static String deQuoteQuotedString(String input) {
        if (!Text.isQuoted(input)) {
            throw new IllegalArgumentException("Die Eingabe '" + input
                    + "' beginnt oder endet nicht mit doppeltem Anführungszeichen.");
        }
        return input.substring(1, input.length() - 1);
    }

    /** Prüft, ob das Erste Zeichen des Textes ein kleingeschriebenes Zeichen ist. */
    public static boolean startsWithLowerLetter(String input) {
        if (input.isEmpty()) {
            return false;
        }

        String firstCharakter = input.substring(0, 1);
        String loweredFirstCharakter = firstCharakter.toLowerCase(Locale.GERMAN);

        return firstCharakter.equals(loweredFirstCharakter);
    }

    /** Gibt an, ob das Zeichen vor dem (ersten) Vorkommen des Teils in der Eingabe leer ist. */
    public static boolean positionBeforePartIsEmpty(String input, String part) {
        if (input.isEmpty() || part.isEmpty()) {
            return false;
        }

        int index = input.indexOf(part);
        if (index == -1) {
            return false; // kann nicht passieren, falls part wirklich Teil von input ist.
        }

        if (index == 0) {  // falls Part am Anfang des Namens ist.
            return false;
        }

        String charBefore = input.substring(index - 1, index);
        return " ".equals(charBefore);
    }


}
