package de.duehl.basics.text;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.assertEquals;
import static de.duehl.basics.text.TextBreaker.LINE_BREAK;

import org.junit.Test;

public class TextBreakerTest {

    @Test
    public void breakNoBreak() {
        String text = "Mal sehen was so passiert!";
        String expected = text;
        int lineLength = 80;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakNoBreakNotRemovingTrailingBlank() {
        String text = "Mal sehen was so passiert! ";
        String expected = "Mal sehen was so passiert! ";
        int lineLength = 80;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakAddOneBreak() {
        String text = "Mal sehen was so passiert!";
        String expected = "Mal sehen was" + LINE_BREAK + "so passiert!";
        int lineLength = 15;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakAddFourBreaks() {
        String text = "Mal sehen was so passiert!";
        String expected = "Mal" + LINE_BREAK
                + "sehen" + LINE_BREAK
                + "was" + LINE_BREAK
                + "so" + LINE_BREAK
                + "passiert!";
        int lineLength = 5;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakAddBreaksWithLongParts() {
        String text = "Mal sehen was so AUSSERGEWÖHNLICHLANGES passiert!";
        String expected = "Mal" + LINE_BREAK
                + "sehen" + LINE_BREAK
                + "was" + LINE_BREAK
                + "so" + LINE_BREAK
                + "AUSSERGEWÖHNLICHLANGES" + LINE_BREAK
                + "passiert!";
        int lineLength = 5;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakAddNoBreaksAtUnderscores() {
        String text = "Mal_sehen_was_so_AUSSERGEWÖHNLICHLANGES_passiert!";
        String expected = text;
        int lineLength = 5;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakBreakAtExaktTextPartLengths() {
        String text = "ab cd ef";
        String expected = "ab" + LINE_BREAK + "cd" + LINE_BREAK + "ef";
        int lineLength = 2;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakBreakAtSmallerLineLengthThanTextPartLength() {
        String text = "ab cd ef";
        String expected = "ab" + LINE_BREAK + "cd" + LINE_BREAK + "ef";
        int lineLength = 1;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakBreakAtLineLengthEqualsTextPartLengthAndSpace() {
        String text = "ab cd ef";
        String expected = "ab" + LINE_BREAK + "cd" + LINE_BREAK + "ef";
        int lineLength = 3;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakBeforeLineIsTooLong() {
        String text = "ab cd ef";
        String expected = "ab" + LINE_BREAK + "cd" + LINE_BREAK + "ef";
        int lineLength = 4;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakAfterTwoParts() {
        String text = "ab cd ef";
        String expected = "ab cd" + LINE_BREAK + "ef";
        int lineLength = 5;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakSmallParts() {
        String text = "ab cd ef gh ij kl mn op qr";
        String expected = "ab cd" + LINE_BREAK + "ef gh" + LINE_BREAK + "ij kl"
                + LINE_BREAK + "mn op" + LINE_BREAK + "qr";
        int lineLength = 5;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakHandleExistingBreakes() {
        String text = "Mal\nsehen\r\nwas so AUSSERGEWÖHNLICHLANGES passiert!";
        String expected = "Mal" + LINE_BREAK
                + "sehen" + LINE_BREAK
                + "was so" + LINE_BREAK
                + "AUSSERGEWÖHNLICHLANGES" + LINE_BREAK
                + "passiert!";
        int lineLength = 10;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakExistingBreaksWithSpaces() {
        String text = "ab" + LINE_BREAK + " " + LINE_BREAK + "cd";
        String expected = "ab" + LINE_BREAK + " " + LINE_BREAK + "cd";
        int lineLength = 2;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakExistingBreaksWithSpaces2() {
        String text = "ab ab " + LINE_BREAK + " " + LINE_BREAK + "cd";
        String expected = "ab" + LINE_BREAK + "ab " + LINE_BREAK + " " + LINE_BREAK + "cd";
        int lineLength = 3;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakExistingBreaksWithSpaces3() {
        String text = "ab ab" + LINE_BREAK + " " + LINE_BREAK + "cd";
        String expected = "ab" + LINE_BREAK + "ab" + LINE_BREAK + " " + LINE_BREAK + "cd";
        int lineLength = 3;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakExistingBreaksWithSpaces4() {
        String text = "ab x" + LINE_BREAK + " " + LINE_BREAK + "cd";
        String expected = "ab" + LINE_BREAK + "x" + LINE_BREAK + " " + LINE_BREAK + "cd";
        int lineLength = 3;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakExistingBreaksWithSpaces5() {
        String text = "ab " + LINE_BREAK;
        String expected = "ab " + LINE_BREAK;
        int lineLength = 3;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

//    @Test
//    public void substringTest1() {
//        String text = "ab ";
//        int indexOfSpace = text.indexOf(" ");
//        String rest = text.substring(indexOfSpace + 1);
//        assertEquals("", rest);
//    }
//
//    @Test
//    public void substringTest2() {
//        String text = "ab ";
//        String rest = text.substring(text.length());
//        assertEquals("", rest);
//    }

    @Test
    public void multipleSpaces1() {
        String text = "ab   cd";
        String expected = "ab " + LINE_BREAK + " cd";
        int lineLength = 3;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void multipleSpaces2() {
        String text = "ab   cd";
        String expected = "ab" + LINE_BREAK + " "+ LINE_BREAK + "cd";
        int lineLength = 2;
        TextBreaker breaker = new TextBreaker(lineLength);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

    @Test
    public void breakWithFrontSpaces() {
        String text = "Lorem ipsum dolor sit amet, consectetur adipisici elit, sed eiusmod tempor "
                + "incidunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis "
                + "nostrud exercitation ullamco laboris nisi ut aliquid ex ea commodi consequat. "
                + "Quis aute iure reprehenderit in voluptate velit esse cillum dolore eu fugiat "
                + "nulla pariatur. Excepteur sint obcaecat cupiditat non proident, sunt in culpa "
                + "qui officia deserunt mollit anim id est laborum.";
        String expected = "Lorem" + Text.LINE_BREAK
                + "     ipsum" +  Text.LINE_BREAK
                + "     dolor sit" +  Text.LINE_BREAK
                + "     amet," +  Text.LINE_BREAK
                + "     consectetur" +  Text.LINE_BREAK
                + "     adipisici" +  Text.LINE_BREAK
                + "     elit, sed" +  Text.LINE_BREAK
                + "     eiusmod" +  Text.LINE_BREAK
                + "     tempor" +  Text.LINE_BREAK
                + "     incidunt" +  Text.LINE_BREAK
                + "     ut labore" +  Text.LINE_BREAK
                + "     et dolore" +  Text.LINE_BREAK
                + "     magna" +  Text.LINE_BREAK
                + "     aliqua. Ut" +  Text.LINE_BREAK
                + "     enim ad" +  Text.LINE_BREAK
                + "     minim" +  Text.LINE_BREAK
                + "     veniam," +  Text.LINE_BREAK
                + "     quis" +  Text.LINE_BREAK
                + "     nostrud" +  Text.LINE_BREAK
                + "     exercitation" +  Text.LINE_BREAK
                + "     ullamco" +  Text.LINE_BREAK
                + "     laboris" +  Text.LINE_BREAK
                + "     nisi ut" +  Text.LINE_BREAK
                + "     aliquid ex" +  Text.LINE_BREAK
                + "     ea commodi" +  Text.LINE_BREAK
                + "     consequat." +  Text.LINE_BREAK
                + "     Quis aute" +  Text.LINE_BREAK
                + "     iure" +  Text.LINE_BREAK
                + "     reprehenderit" +  Text.LINE_BREAK
                + "     in" +  Text.LINE_BREAK
                + "     voluptate" +  Text.LINE_BREAK
                + "     velit esse" +  Text.LINE_BREAK
                + "     cillum" +  Text.LINE_BREAK
                + "     dolore eu" +  Text.LINE_BREAK
                + "     fugiat" +  Text.LINE_BREAK
                + "     nulla" +  Text.LINE_BREAK
                + "     pariatur." +  Text.LINE_BREAK
                + "     Excepteur" +  Text.LINE_BREAK
                + "     sint" +  Text.LINE_BREAK
                + "     obcaecat" +  Text.LINE_BREAK
                + "     cupiditat" +  Text.LINE_BREAK
                + "     non" +  Text.LINE_BREAK
                + "     proident," +  Text.LINE_BREAK
                + "     sunt in" +  Text.LINE_BREAK
                + "     culpa qui" +  Text.LINE_BREAK
                + "     officia" +  Text.LINE_BREAK
                + "     deserunt" +  Text.LINE_BREAK
                + "     mollit" +  Text.LINE_BREAK
                + "     anim id" +  Text.LINE_BREAK
                + "     est" +  Text.LINE_BREAK
                + "     laborum.";
        int numberOfFrontSpaces = 5;
        int lineLength = 15;
        TextBreaker breaker = new TextBreaker(lineLength, numberOfFrontSpaces);
        String actual = breaker.addLineBreaks(text);
        assertEquals(expected, actual);
    }

}

