package de.duehl.swing.ui.dialogs.editor;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Point;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;

import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;

/**
 * Diese Klasse stellt den Dialog zum Anzeigen eines Textes im Editor dar.
 *
 * @version 1.01     2018-02-08
 * @author Christian Dühl
 */

public class EditorDialog extends ModalDialogBase {

    private static final Dimension DIALOG_DIMENSION = new Dimension(850, 650);

    private static final EditorOkReaktor IGNORING_REAKTOR = createIgnoringReaktor();

    /** Inhalt der angezeigt werden soll. */
    private final String contents;

    /** Objekt, das informiert wird, wenn der OK-Button gedrückt wird. */
    private final EditorOkReaktor reaktor;

    /** Die TextArea, in der editiert wird. */
    private final JTextArea area;

    /** Die Scrollbar des Anzeigefensters. */
    private final JScrollPane scroll;

    /** Gibt an, ob der Dialog als Viewer (true) oder als Editor (false) verwendet wird. */
    private final boolean viewer;

    /**
     * Konstruktor als Viewer.
     *
     * @param dialogTitle
     *            Titel des Dialogs.
     * @param contents
     *            Inhalt der angezeigt werden soll.
     */
    public EditorDialog(String dialogTitle, String contents) {
        this(dialogTitle, new Point(0, 0), null, null, contents, IGNORING_REAKTOR);
    }

    /**
     * Konstruktor als Viewer.
     *
     * @param dialogTitle
     *            Titel des Dialogs.
     * @param parentLocation
     *            Position des Rahmens des Aufrufers.
     * @param contents
     *            Inhalt der angezeigt werden soll.
     */
    public EditorDialog(String dialogTitle, Point parentLocation, String contents) {
        this(dialogTitle, parentLocation, null, null, contents, IGNORING_REAKTOR);
    }

    /**
     * Konstruktor als Viewer.
     *
     * @param dialogTitle
     *            Titel des Dialogs.
     * @param parentLocation
     *            Position des Rahmens des Aufrufers.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param programImage
     *            Bild des Programms.
     * @param contents
     *            Inhalt der angezeigt werden soll.
     */
    public EditorDialog(String dialogTitle, Point parentLocation, Colorizer colorizer,
            Image programImage, String contents) {
        this(dialogTitle, parentLocation, colorizer, programImage, contents, IGNORING_REAKTOR);
    }

    /**
     * Konstruktor als Editor.
     *
     * @param dialogTitle
     *            Titel des Dialogs.
     * @param parentLocation
     *            Position des Rahmens des Aufrufers.
     * @param programImage
     *            Bild des Programms.
     * @param contents
     *            Inhalt der angezeigt werden soll.
     * @param reaktor
     *            Objekt, das informiert wird, wenn der OK-Button gedrückt wird.
     */
    public EditorDialog(String dialogTitle, Point parentLocation, Image programImage,
            String contents, EditorOkReaktor reaktor) {
        this(dialogTitle, parentLocation, null, programImage, contents, reaktor);
    }

    /**
     * Konstruktor als Editor.
     *
     * @param dialogTitle
     *            Titel des Dialogs.
     * @param parentLocation
     *            Position des Rahmens des Aufrufers.
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param programImage
     *            Bild des Programms.
     * @param contents
     *            Inhalt der angezeigt werden soll.
     * @param reaktor
     *            Objekt, das informiert wird, wenn der OK-Button gedrückt wird.
     */
    private EditorDialog(String dialogTitle, Point parentLocation, Colorizer colorizer,
            Image programImage, String contents, EditorOkReaktor reaktor) {
        super(parentLocation, programImage, dialogTitle, DIALOG_DIMENSION, colorizer);
        this.reaktor = reaktor;
        this.contents = contents;
        viewer = reaktor == IGNORING_REAKTOR;

        area = new JTextArea();
        scroll = new JScrollPane();

        fillDialog();
        setKeyBindings();
        if (viewer) {
            scrollToEnd();
        }
        else {
            scrollToBeginning();
        }
    }

    private static EditorOkReaktor createIgnoringReaktor() {
        return new EditorOkReaktor() {
            @Override
            public void editOk(String contents) {
                // wir tun nichts...
            }
        };
    }

    /** Verwendet einen Font, der für Sourcecode und Daten geeignet ist. */
    public void useMonoscpacedText() {
        area.setFont(new Font("Monospaced", Font.PLAIN, 14));
    }

    /** Schaltet das Umbrechen der Zeilen ab. */
    public void useNoWordWrap() {
        area.setLineWrap(false);
    }

    /** Scrollt die Scrollbar zum Ende des angezeigten Textes. Dies wird automatisch aufgerufen. */
    public void scrollToEnd() {
        GuiTools.scrollScrollbarToMaximum(scroll);
    }

    /** Scrollt die Scrollbar zum Anfang des angezeigten Textes. */
    public void scrollToBeginning() {
        GuiTools.scrollScrollbarToMinimum(scroll);
    }

    /** Erstellt die Dialog-Gui. */
    @Override
    protected void populateDialog() {
        add(createCenterPart(), BorderLayout.CENTER);
        add(createLowerPart(), BorderLayout.SOUTH);
    }

    /** Erzeugt den mittleren Bereich. */
    private Component createCenterPart() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new BorderLayout());
        panel.setBorder(new EmptyBorder(10, 10, 10, 10));

        panel.add(createTextArea(), BorderLayout.CENTER);

        scroll.setViewportView(panel);
        setColors(scroll);

        return scroll;
    }

    private Component createTextArea() {
        setColors(area);
        area.setEditable(true);
        area.setWrapStyleWord(true);
        area.setLineWrap(true);
        area.setText(contents);
        area.setCaretPosition(0);
        area.setEditable(!viewer);

        return area;
    }

    /**
     * Erzeugt den unteren Bereich mit den Buttons.
     *
     * @return Erzeugtes Panel.
     */
    private Component createLowerPart() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new BorderLayout());

        panel.add(createOkButton(), BorderLayout.EAST);
        if (!viewer) {
            panel.add(createQuitButton(), BorderLayout.WEST);
        }

        return panel;
    }

    private Component createOkButton() {
        JButton button = new JButton("OK");
        setColors(button);
        button.addActionListener(e -> ok());
        return button;
    }

    /** Wird beim Beenden über den Ok-Button ausgelöst. */
    private void ok() {
        reaktor.editOk(area.getText());
        quit();
    }

    private Component createQuitButton() {
        JButton button = new JButton("Abbruch");
        setColors(button);
        button.addActionListener(e -> quit());
        return button;
    }

    /** Wird beim Beenden aufgerufen, durch den Abbruch-Button. */
    private void quit() {
        closeDialog();
    }

    /** Erstellt Tastenkombinationen. */
    private void setKeyBindings() {
        setKeyBindingEscape(() -> closeDialog());
        setKeyBindingHome(() -> GuiTools.setScrollPaneToMinimum(scroll));
        setKeyBindingEnd(() -> GuiTools.setScrollPaneToMaximum(scroll));
        setKeyBindingCtrlHome(() -> GuiTools.setScrollPaneToMinimum(scroll));
        setKeyBindingCtrlEnd(() -> GuiTools.setScrollPaneToMaximum(scroll));
        setKeyBindingPageUp(() -> GuiTools.setScrollPaneToPreviousSection(scroll));
        setKeyBindingPageDown(() -> GuiTools.setScrollPaneToNextSection(scroll));
    }

}
