package de.duehl.swing.ui.elements.stopwatch;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import javax.swing.JLabel;
import javax.swing.SwingUtilities;

import de.duehl.basics.datetime.time.StopWatch;

/**
 * Diese Klasse stellt ein Label darf, das die auf ihm angezeigte Stoppuhr automatisch aktualisiert.
 *
 * @version 1.01     2018-01-08
 * @author Christian Dühl
 */

public class StopWatchLabel extends JLabel {

    private static final long serialVersionUID = 1L;

    /** Text vor der Stoppuhr. */
    private volatile String startText;

    /** Text nach der Stoppuhr. */
    private volatile String endText;

    /** Stoppuhr, deren Zeit angezeigt wird. */
    private final StopWatch stopWatch;

    /** Runnable, das die Aktualisierung behandelt. */
    private StopWatchActualisationRunnable actualisationRunnable;

    /** Konstruktor ohne Text vor und nach der Stoppuhr. */
    public StopWatchLabel() {
        this("", new StopWatch(), "");
    }

    /**
     * Konstruktor mit Text vor der Stoppuhr.
     *
     * @param startText
     *            Text vor der Stoppuhr.
     */
    public StopWatchLabel(String startText) {
        this(startText, new StopWatch(), "");
    }

    /**
     * Konstruktor mit Text vor und nach der Stoppuhr.
     *
     * @param startText
     *            Text vor der Stoppuhr.
     * @param endText
     *            Text nach der Stoppuhr.
     */
    public StopWatchLabel(String startText, String endText) {
        this(startText, new StopWatch(), endText);
    }

    /**
     * Konstruktor mit Text vor und nach der Stoppuhr sowie eigener Stoppuhr, die übergeben wird.
     *
     * @param startText
     *            Text vor der Stoppuhr.
     * @param stopWatch
     *            Stoppuhr, deren Zeit angezeigt wird.
     * @param endText
     *            Text nach der Stoppuhr.
     */
    public StopWatchLabel(String startText, StopWatch stopWatch, String endText) {
        super();
        this.startText = startText;
        this.stopWatch = stopWatch;
        this.endText = endText;
        //GuiTools.setMonospacedFont(this);
        createRunnableAndStartActualisationThread();
    }

    /** Erstellt ein neues Runnable-Objekt und startet es in einem Thread. */
    private void createRunnableAndStartActualisationThread() {
        actualisationRunnable = new StopWatchActualisationRunnable(this);
        Thread thread = new Thread(actualisationRunnable);
        thread.start();
    }

    /** Setter für den Text vor der Stoppuhr. */
    public void setStartText(String startText) {
        this.startText = startText;
        setStopWatchText();
    }

    /** Setter für den Text nach der Stoppuhr. */
    public void setEndText(String endText) {
        this.endText = endText;
        setStopWatchText();
    }

    /**
     * Setzt die Millisekunden zwischen den Aktualisierungen des StopWatchLabels. Default ist 50ms.
     */
    public void setSleepTimeMillis(int sleepTimeMillis) {
        actualisationRunnable.setSleepTimeMillis(sleepTimeMillis);
    }

    /** Setzt den richtigen anzuzeigenden Text. */
    void setStopWatchText() {
        SwingUtilities.invokeLater(() -> setText(startText + stopWatch.getTime() + endText));
    }

    /** Startet die Stoppuhr. */
    public void start() {
        actualisationRunnable.stop();
        stopWatch.start();
        createRunnableAndStartActualisationThread();
    }

    /**
     * Liest die Zeit auf der Stoppuhr ab. Wenn die Stoppuhr mit stop()
     * angehalten wurde, wird die angehaltene Zeit zurückgegeben, ansonsten
     * die Zeit, die vom Start bis jetzt vergangen ist.                          <br/><br/>
     *
     * Die Zeit wird in einem von Menschen lesbarem Format erzeugt.
     *
     * @return Zwischenzeit der Stoppuhr im Format HH:MM:SS.
     */
    public String getTime() {
        return stopWatch.getTime();
    }

    /** Hält die Stoppuhr beim ersten Aufruf von stop() an, weitere Aufrufe sind wirkungslos. */
    public void stop() {
        stopWatch.stop();
        actualisationRunnable.stop();
        setStopWatchText(); // falls das Runnable dazu nicht mehr kam.
    }

    /**
     * Liest die Zeit auf der Stoppuhr ab. Wenn die Stoppuhr mit stop() angehalten wurde, wird die
     * angehaltene Zeit zurückgegeben, ansonsten die Zeit, die vom Start bis jetzt vergangen ist.
     *
     * Die Zeit wird in einem von Menschen lesbarem Format erzeugt.
     */
    public String getRuntime() {
        return stopWatch.getTime();
    }

    /** Stringrepräsentation. */
    @Override
    public String toString() {
        return "StopWatchLabel [" + startText + stopWatch.getTime() + endText + "]";
    }

}
